(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[    291485,       9450]
NotebookOptionsPosition[    234543,       7875]
NotebookOutlinePosition[    257188,       8349]
CellTagsIndexPosition[    256306,       8317]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["The Foundations: Logic and Proofs", "BookChapterTitle",
 CounterAssignments->{{"ChapterNumber", 1}}],

Cell[CellGroupData[{

Cell["Introduction", "Section"],

Cell[TextData[{
 "This chapter describes how ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can be used to further your understanding of logic and proofs. In \
particular, we describe how to construct truth tables, check the validity of \
logical arguments, and verify logical equivalence. In the final two sections, \
we provide examples of how ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can be used as part of proofs, specifically to find counterexamples, carry \
out proofs by exhaustion, and to search for witnesses for existence proofs."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["1.1 Propositional Logic", "Section",
 CellTags->"Section1"],

Cell[TextData[{
 "In this section, we will discuss how to use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to explore propositional logic. Specifically, we will see how to use \
logical connectives, describe the connection between logical implication and \
conditional statements in a program, show how ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can be used to create truth tables for compound propositions, and \
demonstrate how ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can be used to carry out bit operations."
}], "Text"],

Cell[TextData[{
 "In ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", the truth values true and false are represented by the symbols ",
 StyleBox[ButtonBox["True",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/True"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["False",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/False"], "FunctionLink"],
 ". Propositions can be represented by symbols (variables) such as ",
 StyleBox["p", "Input"],
 ", ",
 StyleBox["q", "Input"],
 ", or ",
 StyleBox["prop1", "Input"],
 ". Note that if you have not yet made an assignment to a symbol, entering it \
will return the name."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData["prop1"], "Input"],

Cell[BoxData["prop1"], "Output"]
}, Open  ]],

Cell[TextData[{
 "Once you have assigned a value, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " will evaluate the symbol to the assigned value whenever it appears."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"prop1", "=", "True"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["prop1"], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[TextData[{
 "You can cause ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to \[OpenCurlyDoubleQuote]forget\[CloseCurlyDoubleQuote] the assigned \
value using either the function ",
 StyleBox[ButtonBox["Clear",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Clear"], "FunctionLink"],
 " or the ",
 StyleBox[ButtonBox["Unset",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Unset"], "FunctionLink"],
 " (",
 StyleBox["=.", "Input"],
 ") operator. Both of the expressions below have the effect of removing the \
assigned value from the symbol ",
 StyleBox["prop1", "Input"],
 ". Neither expression returns an output."
}], "Text"],

Cell[BoxData[
 RowBox[{"Clear", "[", "prop1", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"prop1", "=."}]], "Input"],

Cell[CellGroupData[{

Cell["Logical Connectives", "Subsection",
 CellTags->"Section1a"],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " supports all of the basic logical operators discussed in the textbook. We \
illustrate the logical operators of negation (",
 StyleBox[ButtonBox["Not",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Not"], "FunctionLink"],
 ", ",
 StyleBox["!", "Input"],
 "), conjunction (",
 StyleBox[ButtonBox["And",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/And"], "FunctionLink"],
 ", ",
 StyleBox["&&", "Input"],
 "), disjunction (",
 StyleBox[ButtonBox["Or",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Or"], "FunctionLink"],
 ", ",
 StyleBox["||", "Input"],
 "), exclusive or (",
 StyleBox[ButtonBox["Xor",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Xor"], "FunctionLink"],
 "), implication (",
 StyleBox[ButtonBox["Implies",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Implies"], "FunctionLink"],
 "), and the biconditional (",
 StyleBox[ButtonBox["Equivalent",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Equivalent"], "FunctionLink"],
 "). Note that these are referred to as Boolean operators, and expressions \
formed from them are Boolean expressions",
 ". "
}], "Text"],

Cell[TextData[{
 "For all of the operators, you can enter expressions in standard form, that \
is, by putting the names of the operators at the head of an expression with \
truth values or other expressions as operands",
 ". ",
 "For example, the computations ",
 Cell[BoxData[
  FormBox[
   RowBox[{"T", " ", "\[Or]", " ", "F"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"T", " ", "\[Implies]", " ", 
    RowBox[{"(", 
     RowBox[{"F", " ", "\[And]", " ", "T"}], ")"}]}], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"T", "\[CirclePlus]", " ", "T"}], TraditionalForm]]],
 " are shown below."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Or", "[", 
  RowBox[{"True", ",", "False"}], "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Implies", "[", 
  RowBox[{"True", ",", 
   RowBox[{"And", "[", 
    RowBox[{"False", ",", "True"}], "]"}]}], "]"}]], "Input"],

Cell[BoxData["False"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Xor", "[", 
  RowBox[{"True", ",", "True"}], "]"}]], "Input"],

Cell[BoxData["False"], "Output"]
}, Open  ]],

Cell[TextData[{
 "For negation, conjunction, and disjunction, you can use the infix operators \
",
 StyleBox["!", "Input"],
 ", ",
 StyleBox["&&", "Input"],
 ", and ",
 StyleBox["||", "Input"],
 " instead. These are common symbols used in place of \[Not], \[And], and \
\[Or] that can be easily typed on a standard keyboard. The computations below \
show ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Not]", "T"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"T", "\[Or]", "F"}], ")"}], "\[And]", "T"}], TraditionalForm]]],
 " using the operators ",
 StyleBox["!", "Input"],
 ", ",
 StyleBox["&&", "Input"],
 ", and ",
 StyleBox["||", "Input"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"!", "True"}]], "Input"],

Cell[BoxData["False"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"True", "||", "False"}], ")"}], "&&", "True"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " also allows you to enter and compute with expressions using the \
traditional symbols. You enter the symbol by pressing the escape key, \
followed by a sequence identifying the symbol, and then the escape key once \
again. ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " refers to this as an alias. For example, entering \[EscapeKey]",
 StyleBox["and\[EscapeKey] ", "InlineCode"],
 "produces the traditional symbol for conjunction."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"True", " ", "\[And]", " ", "False"}]], "Input"],

Cell[BoxData["False"], "Output"]
}, Open  ]],

Cell[TextData[{
 "An alias is the only way to produce an infix implication operator, via \
\[EscapeKey]",
 StyleBox["\[ThinSpace]=>\[EscapeKey] ", "InlineCode"],
 "(escape followed by equals and the greater than sign and terminating with \
escape)."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"False", " ", "\[Implies]", " ", "False"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[TextData[{
 "In this manual, we will typically not use aliases as part of commands, \
since it is more difficult for a reader to imitate such commands",
 ". ",
 "However, for convenience, we include a table of the operators defined in \
the textbook along with their names in ",
 StyleBox["Mathematica ",
  FontSlant->"Italic"],
 "and their infix representations with and without aliases. "
}], "Text"],

Cell[TextData[Cell[BoxData[GridBox[{
   {"name", "function", 
    RowBox[{"without", " ", "alias"}], "alias", "symbol"},
   {"negation", 
    StyleBox["Not", "Input"], 
    StyleBox["!", "Input"], 
    StyleBox[
     RowBox[{"\[EscapeKey]", 
      StyleBox["not", "InlineCode"], "\[EscapeKey]"}]], "\[Not]"},
   {"conjunction", 
    StyleBox["And", "Input"], 
    StyleBox["&&", "Input"], 
    StyleBox[
     RowBox[{"\[EscapeKey]", 
      StyleBox["and", "InlineCode"], "\[EscapeKey]"}]], "\[And]"},
   {
    RowBox[{"exclusive", " ", "or"}], 
    StyleBox["Xor", "Input"], " ", 
    StyleBox[
     RowBox[{"\[EscapeKey]", 
      StyleBox["xor", "InlineCode"], "\[EscapeKey]"}]], "\[Xor]"},
   {"disjunction", 
    StyleBox["Or", "Input"], 
    StyleBox["||", "Input"], 
    StyleBox[
     RowBox[{"\[EscapeKey]", 
      StyleBox["or", "InlineCode"], "\[EscapeKey]"}]], "\[Or]"},
   {"biconditional", 
    StyleBox["Equivalent", "Input"], " ", 
    StyleBox[
     RowBox[{"\[EscapeKey]", 
      StyleBox["equiv", "InlineCode"], "\[EscapeKey]"}]], "\[Equivalent]"},
   {"implication", 
    StyleBox["Implies", "Input"], " ", 
    StyleBox[
     RowBox[{"\[EscapeKey]", "=", 
      RowBox[{">", "\[EscapeKey]"}]}], "InputForm"], "\[Implies]"}
  },
  GridBoxDividers->{
   "Columns" -> {{True}}, "ColumnsIndexed" -> {}, "Rows" -> {{True}}, 
    "RowsIndexed" -> {}}]],
 FontFamily->"Times"]], "Text",
 TextAlignment->Center],

Cell[TextData[{
 "Note that the symbol for exclusive or used by ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " differs from that in the textbook. Also, the order in which the operators \
appear in the table above is the order of precedence that the operators have \
in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ". Observe that the order of the biconditional and implication are the \
reverse of the order specified in the textbook",
 ". ",
 "It is always a good idea to use parentheses liberally whenever precedence \
is in doubt."
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Conditional Statements", "Subsection",
 CellTags->"Section1b"],

Cell[TextData[{
 "We saw above that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " includes the operator ",
 StyleBox[ButtonBox["Implies",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Implies"], "FunctionLink"],
 " for evaluating logical implication. In mathematical logic, \
\[OpenCurlyDoubleQuote]if p, then q\[CloseCurlyDoubleQuote] has a very \
specific meaning, as described in detail in the text. In computer \
programming, and ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " in particular, conditional statements also appear very frequently, but \
have a slightly different meaning."
}], "Text"],

Cell["\<\
From the perspective of logic, a conditional statement is, like any other \
proposition, a sentence that is either true or false. In most computer \
programming languages, when we talk about a conditional statement, we are not \
referring to a kind of proposition. Rather, conditional statements are used \
to selectively execute portions of code. Consider the following example of a \
function, which adds 1 to the input value if the input is less than or equal \
to 5 and not otherwise.\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"ifExample", "[", "x_", "]"}], " ", ":=", " ", 
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{"x", "\[LessEqual]", "5"}], ",", 
    RowBox[{"x", "+", "1"}], ",", "x"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "(To type the inequality into ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", you type \[OpenCurlyDoubleQuote]x<=5\[CloseCurlyDoubleQuote]. The \
graphical front end will automatically turn the key combination \
\[OpenCurlyDoubleQuote]<=\[CloseCurlyDoubleQuote] into \[LessEqual], unless \
you have set options to prevent it from doing so.) We now see that this \
function works as promised."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ifExample", "[", "3", "]"}]], "Input"],

Cell[BoxData["4"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ifExample", "[", "7", "]"}]], "Input"],

Cell[BoxData["7"], "Output"]
}, Open  ]],

Cell[TextData[{
 "Because this is our first ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function, let\[CloseCurlyQuote]s spend a moment breaking down the general \
structure before detailing the workings of the conditional statement. First \
we have the name of the function, ",
 StyleBox["ifExample", "Input"],
 ". Note that symbols for built-in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " functions typically begin with capital letters, so making a habit of \
naming functions you define with initial letters lower case helps ensure that \
you won\[CloseCurlyQuote]t accidentally try to assign to a built-in function. "
}], "Text"],

Cell[TextData[{
 "Following the name of the function, we specify the arguments that will be \
accepted by the function enclosed in brackets. The underscore (",
 StyleBox["_", "Input"],
 "), referred to as ",
 StyleBox[ButtonBox["Blank",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Blank"], "FunctionLink"],
 ", tells ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " that this is a parameter and that the symbol preceding the underscore is \
the name that will be used to refer to the parameter. "
}], "Text"],

Cell[TextData[{
 "Then comes the operator ",
 StyleBox[":=", "Input"],
 ", the delayed assignment operator. The difference between using ",
 StyleBox[ButtonBox["Set",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Set"], "FunctionLink"],
 " (",
 StyleBox["=", "Input"],
 ") and ",
 StyleBox[ButtonBox["SetDelayed",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/SetDelayed"], "FunctionLink"],
 " (",
 StyleBox[":=",
  FontWeight->"Bold"],
 ") is that the delayed assignment ensures that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " does not attempt to evaluate the function definition until the function is \
actually invoked.  ",
 StyleBox[ButtonBox["SetDelayed",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/SetDelayed"], "FunctionLink"],
 " (",
 StyleBox[":=",
  FontWeight->"Bold"],
 ") should be used when you define a function, while ",
 StyleBox[ButtonBox["Set",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Set"], "FunctionLink"],
 " (",
 StyleBox["=", "Input"],
 ") is appropriate for assigning values to variables."
}], "Text"],

Cell[TextData[{
 "On the right hand side of the delayed assignment operator is the expression \
that tells ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " what to do with the argument. In this case, the body of the function makes \
use of the ",
 StyleBox[ButtonBox["If",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/If"], "FunctionLink"],
 " function to choose between two possible results. Note that we provided \
three arguments, separated by commas, to ",
 StyleBox[ButtonBox["If",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/If"], "FunctionLink"],
 ". The first argument, ",
 StyleBox["x<=5", "Input"],
 ", specifies the condition. ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " evaluates this expression to determine which of the branches, that is \
which of the other two arguments, to execute. If the condition is true, then ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " evaluates the second argument, ",
 StyleBox["x+1", "Input"],
 ", and this is the value of the function. This is traditionally called the \
\[OpenCurlyDoubleQuote]then\[CloseCurlyDoubleQuote] clause. If the condition \
specified in the first argument is false, then the third argument, called the \
\[OpenCurlyDoubleQuote]else\[CloseCurlyDoubleQuote] clause, is evaluated."
}], "Text"],

Cell[TextData[{
 "It is important to be aware of two additional variations on the ",
 StyleBox[ButtonBox["If",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/If"], "FunctionLink"],
 " function. First, you are allowed to omit the \[OpenCurlyDoubleQuote]else\
\[CloseCurlyDoubleQuote] and provide only two arguments. As you can see in \
the example below, when the condition is false, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " appears to return nothing. In fact, the expression returns the special \
symbol ",
 StyleBox[ButtonBox["Null",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Null"], "FunctionLink"],
 ", which does not produce output."
}], "Text"],

Cell[BoxData[
 RowBox[{"If", "[", 
  RowBox[{
   RowBox[{"3", "<", "1"}], ",", "5"}], "]"}]], "Input"],

Cell[TextData[{
 "The second variation on ",
 StyleBox[ButtonBox["If",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/If"], "FunctionLink"],
 " has four arguments. ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " is very strict with regards to conditional statements. Specifically, it \
only evaluates the second argument if the result of evaluating the condition \
is the symbol ",
 StyleBox[ButtonBox["True",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/True"], "FunctionLink"],
 ". And it only evaluates the third argument when the result of the condition \
is ",
 StyleBox[ButtonBox["False",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/False"], "FunctionLink"],
 ". But many expressions do not evaluate to either of these symbols. In these \
cases, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " returns the ",
 StyleBox[ButtonBox["If",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/If"], "FunctionLink"],
 " function unevaluated",
 ". ",
 "For example, in the expression below, the symbol z has not been assigned a \
value and thus ",
 StyleBox["z>5", "Input"],
 " cannot be resolved to a truth value."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"If", "[", 
  RowBox[{
   RowBox[{"z", ">", "5"}], ",", "4", ",", "11"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"If", "[", 
  RowBox[{
   RowBox[{"z", ">", "5"}], ",", "4", ",", "11"}], "]"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "By specifying a fourth argument, you can give ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " explicit instructions on how to handle this situation."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"If", "[", 
  RowBox[{
   RowBox[{"z", ">", "5"}], ",", "4", ",", "11", ",", "0"}], "]"}]], "Input"],

Cell[BoxData["0"], "Output"]
}, Open  ]],

Cell[TextData[{
 "This fourth argument is useful if there is some question of whether or not ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " will be able to resolve the condition into a truth value. We will \
typically not use the fourth argument, however, since in nearly all cases, a \
failure to properly evaluate the condition indicates an error in either our \
function definition or the input to it and providing the fourth argument will \
only hide such errors from us."
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Evaluating Expressions", "Subsection",
 CellTags->"Section1c"],

Cell[TextData[{
 "In the textbook, you saw how to construct truth tables by hand. Here we\
\[CloseCurlyQuote]ll see how to have ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " create them for us. We\[CloseCurlyQuote]ll begin by considering the \
simplest case of a compound proposition: the negation of a single \
propositional variable."
}], "Text"],

Cell[BoxData[
 RowBox[{"prop2", " ", ":=", " ", 
  RowBox[{"!", "p"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Note that we\[CloseCurlyQuote]ve defined the proposition ",
 StyleBox["prop2", "Input"],
 " as an expression in terms of the symbol ",
 StyleBox["p", "Input"],
 ", which has not been assigned. We can determine the truth value of ",
 StyleBox["prop2", "Input"],
 " in one of two ways. The obvious way is to assign a truth value to ",
 StyleBox["p", "Input"],
 " and then ask ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " for the value of ",
 StyleBox["prop2", "Input"],
 " as follows."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"p", " ", "=", " ", "False"}]], "Input"],

Cell[BoxData["False"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["prop2"], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[TextData[{
 "The drawback of this approach, however, is that our variable ",
 StyleBox["p", "Input"],
 " is now identified with false and if we want to use it as a name again, we \
need to manually unassign it."
}], "Text"],

Cell[BoxData[
 RowBox[{"p", "=."}]], "Input"],

Cell[TextData[{
 "The better approach is to use the ",
 StyleBox[ButtonBox["ReplaceAll",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ReplaceAll"], "FunctionLink"],
 " operator (",
 StyleBox["/.", "Input"],
 "). This function has a variety of uses, one of which is to allow you to \
evaluate an expression for particular values of variables without the need to \
assign (and then ",
 StyleBox[ButtonBox["Clear",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Clear"], "FunctionLink"],
 ") values to the variables. We first demonstrate its use and then we\
\[CloseCurlyQuote]ll explain the syntax."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"prop2", "/.", 
  RowBox[{"p", "\[Rule]", "True"}]}]], "Input"],

Cell[BoxData["False"], "Output"]
}, Open  ]],

Cell[TextData[{
 "On the left hand side of the ",
 StyleBox["/.", "Input"],
 " operator is the expression to be evaluated. In this case, we have the \
symbol ",
 StyleBox["prop2", "Input"],
 " on the left, which was assigned to be ",
 StyleBox["!p", "Input"],
 ". On the right hand side of the operator, we indicate the substitution to \
be made using the notation ",
 StyleBox["a\[Rule]b", "Input"],
 ", called a rule, to indicate that ",
 StyleBox["a", "Input"],
 " is replaced by ",
 StyleBox["b", "Input"],
 ". (Note that you obtain the arrow by typing a hyphen followed by the \
greater than symbol (",
 StyleBox["->", "Input"],
 "). The ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " front end will automatically turn that into the arrow character.)"
}], "Text"],

Cell[TextData[{
 "In order to substitute for more than one variable, list the substitutions \
as rules separated by commas and enclosed in braces",
 ". ",
 "The following evaluates the proposition ",
 Cell[BoxData[
  FormBox[
   RowBox[{"p", "\[And]", 
    RowBox[{"(", 
     RowBox[{"\[Not]", "q"}], ")"}]}], TraditionalForm]]],
 " for ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " true and ",
 Cell[BoxData[
  FormBox["q", TraditionalForm]]],
 " false."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"p", "&&", 
   RowBox[{"(", 
    RowBox[{"!", "q"}], ")"}]}], "/.", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"p", "\[Rule]", "True"}], ",", 
    RowBox[{"q", "\[Rule]", "False"}]}], "}"}]}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Truth Tables and Loops", "Subsection",
 CellTags->"Section1d"],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " has a built-in function for producing a truth table, ",
 StyleBox[ButtonBox["BooleanTable",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BooleanTable"], "FunctionLink"],
 ", which will be described in Section 1.2. While the built-in function is \
useful, it is worthwhile to consider how such tables can be created using \
more primitive programming tools. In this subsection, we will see how to \
create truth tables using only basic loop constructs."
}], "Text"],

Cell[TextData[{
 "To make a truth table for a proposition, we need to evaluate the \
proposition at all possible truth values of all of the different variables. \
To do this, we make use of loops (refer to the Introduction for a general \
discussion of loops in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "). Specifically, we want to loop over the two possible truth values, true \
and false, so we will construct a loop over the list ",
 StyleBox["{True, False}", "Input"],
 "."
}], "Text"],

Cell[TextData[{
 "In ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", the ",
 StyleBox[ButtonBox["Do",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Do"], "FunctionLink"],
 " function is used to create a loop that executes commands for each member \
of a list. The ",
 StyleBox[ButtonBox["Do",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Do"], "FunctionLink"],
 " function requires two arguments. The first argument is the expression that \
you want evaluated, typically involving one or more variables that change \
during the execution of the loop. The second argument specifies the iterative \
behavior and can take several forms. The form we will be using here is ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"i", ",", 
     RowBox[{"{", 
      RowBox[{
       SubscriptBox["i", "1"], ",", 
       SubscriptBox["i", "2"], ",", "\[Ellipsis]"}], "}"}]}], "}"}], 
   TraditionalForm]]],
 ". ",
 "The character ",
 Cell[BoxData[
  FormBox["i", TraditionalForm]]],
 " represents the loop variable and the list ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{
     SubscriptBox["i", "1"], ",", 
     SubscriptBox["i", "2"], ",", "\[Ellipsis]"}], "}"}], TraditionalForm]]],
 " represents an explicit list of particular values that will be assigned to \
the loop variable."
}], "Text"],

Cell[TextData[{
 "The first example will be to produce a truth table for the proposition ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Not]", "p"}], TraditionalForm]]],
 ". Each iteration in the loop, therefore, should print out one line of the \
truth table. Since a ",
 StyleBox[ButtonBox["Do",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Do"], "FunctionLink"],
 " loop does not produce any output unless explicitly told to do so (it \
normally returns ",
 StyleBox[ButtonBox["Null",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Null"], "FunctionLink"],
 "), we will use the ",
 StyleBox[ButtonBox["Print",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Print"], "FunctionLink"],
 " function to tell the loop what should be displayed. The ",
 StyleBox[ButtonBox["Print",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Print"], "FunctionLink"],
 " function takes any number of arguments and displays them concatenated \
together. In this example, we want to display the value of the propositional \
variable ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " and the truth value of the proposition ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Not]", "p"}], TraditionalForm]]],
 ". We will also explicitly insert some space between the two truth values by \
putting ",
 StyleBox["\[OpenCurlyDoubleQuote] \[OpenCurlyDoubleQuote]", "Input"],
 " as an argument as well",
 ". ",
 "So the first argument to ",
 StyleBox[ButtonBox["Do",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Do"], "FunctionLink"],
 " will be ",
 StyleBox["Print[p,\[CloseCurlyDoubleQuote] \[OpenCurlyDoubleQuote],!p]", 
  "Input"],
 "."
}], "Text"],

Cell[TextData[{
 "For the second argument, the specification of the iteration, we must give \
",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " the name of the loop variable, in this case ",
 StyleBox["p", "Input"],
 ", and the list of values that we want assigned to that variable in each \
iteration, namely true and false",
 ". ",
 "So the second argument will be ",
 StyleBox["{p,{True,False}}", "Input"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Do", "[", 
  RowBox[{
   RowBox[{"Print", "[", 
    RowBox[{"p", ",", "\"\< \>\"", ",", 
     RowBox[{"!", "p"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"p", ",", 
     RowBox[{"{", 
      RowBox[{"True", ",", "False"}], "}"}]}], "}"}]}], "]"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{
  "True", "\[InvisibleSpace]", "\<\" \"\>", "\[InvisibleSpace]", "False"}],
  SequenceForm[True, " ", False],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
  "False", "\[InvisibleSpace]", "\<\" \"\>", "\[InvisibleSpace]", "True"}],
  SequenceForm[False, " ", True],
  Editable->False]], "Print"]
}, Open  ]]
}, Open  ]],

Cell[TextData[{
 "As a second example, we will construct the truth table for ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"p", "\[And]", "q"}], ")"}], "\[Implies]", "p"}], 
   TraditionalForm]]],
 ". Notice that here there are two variables instead of one. This indicates \
that two loops should be used, one for each variable. In most programming \
languages, this is approach that you would need to take, called \
\[OpenCurlyDoubleQuote]nesting\[CloseCurlyDoubleQuote] loops. In effect, you \
would use a ",
 StyleBox[ButtonBox["Do",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Do"], "FunctionLink"],
 " function as the first argument to another ",
 StyleBox[ButtonBox["Do",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Do"], "FunctionLink"],
 " function. Indeed, this approach would work in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " as well, but there is another way. The ",
 StyleBox[ButtonBox["Do",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Do"], "FunctionLink"],
 " syntax allows you to provide more than one iteration specification. For \
this example, we want both variables ",
 StyleBox["p", "Input"],
 " and ",
 StyleBox["q", "Input"],
 " to take on both truth values, so we provide the iteration specifications \
for both of them",
 ". ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " ensures that it executes the expression in the first argument with every \
possible pair of values for ",
 StyleBox["p", "Input"],
 " and ",
 StyleBox["q", "Input"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Do", "[", 
  RowBox[{
   RowBox[{"Print", "[", 
    RowBox[{"p", ",", "\"\< \>\"", ",", "q", ",", "\"\< \>\"", ",", 
     RowBox[{"Implies", "[", 
      RowBox[{
       RowBox[{"p", "&&", "q"}], ",", "p"}], "]"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"p", ",", 
     RowBox[{"{", 
      RowBox[{"True", ",", "False"}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", 
     RowBox[{"{", 
      RowBox[{"True", ",", "False"}], "}"}]}], "}"}]}], "]"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{
  "True", "\[InvisibleSpace]", "\<\" \"\>", "\[InvisibleSpace]", "True", 
   "\[InvisibleSpace]", "\<\" \"\>", "\[InvisibleSpace]", "True"}],
  SequenceForm[True, " ", True, " ", True],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
  "True", "\[InvisibleSpace]", "\<\" \"\>", "\[InvisibleSpace]", "False", 
   "\[InvisibleSpace]", "\<\" \"\>", "\[InvisibleSpace]", "True"}],
  SequenceForm[True, " ", False, " ", True],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
  "False", "\[InvisibleSpace]", "\<\" \"\>", "\[InvisibleSpace]", "True", 
   "\[InvisibleSpace]", "\<\" \"\>", "\[InvisibleSpace]", "True"}],
  SequenceForm[False, " ", True, " ", True],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
  "False", "\[InvisibleSpace]", "\<\" \"\>", "\[InvisibleSpace]", "False", 
   "\[InvisibleSpace]", "\<\" \"\>", "\[InvisibleSpace]", "True"}],
  SequenceForm[False, " ", False, " ", True],
  Editable->False]], "Print"]
}, Open  ]]
}, Open  ]],

Cell[TextData[{
 "Note that the output indicates that the proposition, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"p", "\[And]", "q"}], ")"}], "\[Implies]", "p"}], 
   TraditionalForm]]],
 ", is a tautology",
 ". ",
 "In fact, this is a rule of inference called simplification, discussed in \
Section 1.6 of the textbook."
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Logic and Bit Operations", "Subsection",
 CellTags->"Section1e"],

Cell[TextData[{
 "We can also use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to explore the bit operations OR, AND, and XOR. Recall that bit operations \
correspond to logical operators by equating 1 with true and 0 with false. ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " provides a lot of support for working with bits and bit strings. Here, we \
will briefly introduce the relevant ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " functions. Our main goal of this section, however, will be to develop a \
function essentially from scratch for computing with bit strings, in order to \
further illustrate programming in ",
 StyleBox["Mathematica.",
  FontSlant->"Italic"]
}], "Text"],

Cell[CellGroupData[{

Cell["The Built-in Functions", "Subsubsection"],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " provides several functions corresponding to the basic logical operations \
for operation on bits: ",
 StyleBox[ButtonBox["BitAnd",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BitAnd"], "FunctionLink"],
 ", ",
 StyleBox[ButtonBox["BitOr",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BitOr"], "FunctionLink"],
 ", ",
 StyleBox[ButtonBox["BitXor",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BitXor"], "FunctionLink"],
 ", ",
 StyleBox[ButtonBox["BitNot",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BitNot"], "FunctionLink"],
 ". With the exception of ",
 StyleBox[ButtonBox["BitNot",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BitNot"], "FunctionLink"],
 ", these operations operate as you would expect. For example, you can \
compute ",
 Cell[BoxData[
  FormBox[
   RowBox[{"1", "\[And]", "0"}], TraditionalForm]]],
 " as follows."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BitAnd", "[", 
  RowBox[{"1", ",", "0"}], "]"}]], "Input"],

Cell[BoxData["0"], "Output"]
}, Open  ]],

Cell[TextData[{
 "Also, you are not limited to two arguments",
 ". ",
 "For example, computing ",
 Cell[BoxData[
  FormBox[
   RowBox[{"0", "\[Or]", "0", "\[Or]", "1", "\[Or]", "0"}], TraditionalForm]]],
 " requires only one application of ",
 StyleBox[ButtonBox["BitOr",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BitOr"], "FunctionLink"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BitOr", "[", 
  RowBox[{"0", ",", "0", ",", "1", ",", "0"}], "]"}]], "Input"],

Cell[BoxData["1"], "Output"]
}, Open  ]],

Cell[TextData[{
 "Conveniently, the bitwise functions are ",
 StyleBox[ButtonBox["Listable",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Listable"], "FunctionLink"],
 ". This means that the function is automatically threaded over lists that \
are given as arguments. This can be made clearer by demonstrating with \
another listable function: addition."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{"1", ",", "2", ",", "3"}], "}"}], "+", 
  RowBox[{"{", 
   RowBox[{"a", ",", "b", ",", "c"}], "}"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"1", "+", "a"}], ",", 
   RowBox[{"2", "+", "b"}], ",", 
   RowBox[{"3", "+", "c"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Because addition is listable, when it is applied to two lists of equal \
length, it returns the list formed by acting on corresponding elements of the \
lists. In the current context, this means we can apply the bitwise operations \
to bit strings by representing the bit strings a lists",
 ". ",
 "For example, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"10010", "\[And]", "01011"}], TraditionalForm]]],
 " can be computed as follows."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BitAnd", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "0", ",", "1", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "1", ",", "0", ",", "1", ",", "1"}], "}"}]}], 
  "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "0", ",", "0", ",", "1", ",", "0"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The bitwise functions actually operate on integers, not just the bits 0 and \
1",
 ". ",
 "For example, we can apply ",
 StyleBox[ButtonBox["BitOr",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BitOr"], "FunctionLink"],
 " to 18 and 5."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BitOr", "[", 
  RowBox[{"18", ",", "5"}], "]"}]], "Input"],

Cell[BoxData["23"], "Output"]
}, Open  ]],

Cell[TextData[{
 "The reason for this result is that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " applied the bitwise OR to the binary representations of the integers 18 \
and 5",
 ". ",
 "You can use the function ",
 StyleBox[ButtonBox["IntegerDigits",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/IntegerDigits"], "FunctionLink"],
 " with an integer as the first coordinate and 2 as the second coordinate to \
see the binary representation of an integer."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"IntegerDigits", "[", 
  RowBox[{"18", ",", "2"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "0", ",", "0", ",", "1", ",", "0"}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"IntegerDigits", "[", 
  RowBox[{"5", ",", "2"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "0", ",", "1"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "We need to pad the result for 5 with 0s in order to have lists of equal \
size and then we can apply ",
 StyleBox[ButtonBox["BitOr",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BitOr"], "FunctionLink"],
 " on the lists of bits as we did above."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BitOr", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "0", ",", "1", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "1", ",", "0", ",", "1"}], "}"}]}], 
  "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "0", ",", "1", ",", "1", ",", "1"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["FromDigits",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/FromDigits"], "FunctionLink"],
 " function reverses ",
 StyleBox[ButtonBox["IntegerDigits",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/IntegerDigits"], "FunctionLink"],
 ". ",
 "Given a list of bits and second argument 2, it will return the integer with \
that binary representation."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FromDigits", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "1", ",", "1", ",", "1"}], "}"}], ",", "2"}], 
  "]"}]], "Input"],

Cell[BoxData["23"], "Output"]
}, Open  ]],

Cell[TextData[{
 "Understanding the operation of ",
 StyleBox[ButtonBox["BitNot",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BitNot"], "FunctionLink"],
 " is a bit more complicated. As expected, it accepts only one argument, \
although again, it will automatically thread through a list. The results on 0 \
and 1, however, are not what you would expect."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BitNot", "[", "0", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"-", "1"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BitNot", "[", "1", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"-", "2"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The reason for these results is that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " represents integers in two\[CloseCurlyQuote]s complement form with an \
unlimited number of digits. Interested readers should refer to the \
information prior to Exercise 40 in Section 4.2 of the textbook for an \
explanation of two\[CloseCurlyQuote]s complement. For this context, it is \
enough to know that ",
 StyleBox[ButtonBox["BitNot",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BitNot"], "FunctionLink"],
 " applied to an integer ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " will always return ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"-", "1"}], "-", "n"}], TraditionalForm]]],
 ", but that it will behave exactly as expected relative to the other \
functions. For example ",
 Cell[BoxData[
  FormBox[
   RowBox[{"1", "\[And]", 
    RowBox[{"(", 
     RowBox[{"\[Not]", "0"}], ")"}]}], TraditionalForm]]],
 " results in 1, as it should. "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BitAnd", "[", 
  RowBox[{"1", ",", 
   RowBox[{"BitNot", "[", "0", "]"}]}], "]"}]], "Input"],

Cell[BoxData["1"], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Creating a New Bitwise And", "Subsubsection"],

Cell[TextData[{
 "As mentioned above, we will use the bitwise operations as an opportunity to \
further explore ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "\[CloseCurlyQuote]s programming capabilities and some important functions. \
Specifically, we will build a bitwise conjunction function that behaves much \
like ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "\[CloseCurlyQuote]s for bits and lists of bits",
 ". "
}], "Text"],

Cell[TextData[{
 "We begin by creating a function that applies only to a pair of bits. Later, \
we\[CloseCurlyQuote]ll extend it to bit strings. We name our function ",
 StyleBox["and", "Input"],
 ". Since ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " symbols are case-sensitive, this is different from the built-in function \
",
 StyleBox[ButtonBox["And",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/And"], "FunctionLink"],
 ". "
}], "Text"],

Cell[TextData[{
 "To implement ",
 StyleBox["and", "Input"],
 ", we will make use of the ",
 StyleBox[ButtonBox["Switch",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Switch"], "FunctionLink"],
 " function. ",
 StyleBox[ButtonBox["Switch",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Switch"], "FunctionLink"],
 " is an important mechanism for controlling flow in a program. It is \
equivalent to a series of if statements, but its structure makes it more \
efficient and more easily understood",
 ". ",
 StyleBox[ButtonBox["Switch",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Switch"], "FunctionLink"],
 " is executed in the form"
}], "Text"],

Cell[BoxData[
 RowBox[{"Switch", "[", 
  RowBox[{Cell["expr", "CodeVar"], ",", Cell[TextData[Cell[BoxData[
    FormBox[
     SubscriptBox["form", "1"], TraditionalForm]]]], "CodeVar"], ",", Cell[
   TextData[Cell[BoxData[
    FormBox[
     SubscriptBox["value", "1"], TraditionalForm]]]], "CodeVar"], ",", Cell[
   TextData[Cell[BoxData[
    FormBox[
     SubscriptBox["form", "2"], TraditionalForm]]]], "CodeVar"], ",", Cell[
   TextData[Cell[BoxData[
    FormBox[
     SubscriptBox["value", "2"], TraditionalForm]]]], "CodeVar"], ",", 
   "..."}], "]"}]], "myCode"],

Cell[TextData[{
 "The first argument is an expression that is evaluated. The rest of the \
arguments are in form/value pairs. ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " checks the result of evaluating the expression against the forms, one at a \
time and in order. If it finds a match, then it stops checking and returns \
the value associated with the matching form. If none of the forms match, then \
the result is the ",
 StyleBox[ButtonBox["Switch",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Switch"], "FunctionLink"],
 " function unevaluated. "
}], "Text"],

Cell[TextData[{
 "Our ",
 StyleBox["and", "Input"],
 " function will accept two arguments. The expression we give to ",
 StyleBox[ButtonBox["Switch",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Switch"], "FunctionLink"],
 " will be the list formed from the two arguments. The rest of the ",
 StyleBox[ButtonBox["Switch",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Switch"], "FunctionLink"],
 " will essentially be the truth table for conjunction. The forms will be all \
the possible pairs of 0s and 1s and the values will be 0 or 1 as \
appropriate."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"and", "[", 
   RowBox[{"p_", ",", "q_"}], "]"}], ":=", 
  RowBox[{"Switch", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"p", ",", "q"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"1", ",", "1"}], "}"}], ",", "1", ",", 
    RowBox[{"{", 
     RowBox[{"1", ",", "0"}], "}"}], ",", "0", ",", 
    RowBox[{"{", 
     RowBox[{"0", ",", "1"}], "}"}], ",", "0", ",", 
    RowBox[{"{", 
     RowBox[{"0", ",", "0"}], "}"}], ",", "0"}], "]"}]}]], "Input"],

Cell[TextData[{
 "The ",
 StyleBox["and", "Input"],
 " function we created now works as expected on bits and does nothing if it \
is given other input."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"and", "[", 
  RowBox[{"1", ",", "1"}], "]"}]], "Input"],

Cell[BoxData["1"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"and", "[", 
  RowBox[{"1", ",", "0"}], "]"}]], "Input"],

Cell[BoxData["0"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"and", "[", 
  RowBox[{"18", ",", "5"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"Switch", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"18", ",", "5"}], "}"}], ",", "\[IndentingNewLine]", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1"}], "}"}], ",", "1", ",", "\[IndentingNewLine]", 
   RowBox[{"{", 
    RowBox[{"1", ",", "0"}], "}"}], ",", "0", ",", "\[IndentingNewLine]", 
   RowBox[{"{", 
    RowBox[{"0", ",", "1"}], "}"}], ",", "0", ",", "\[IndentingNewLine]", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0"}], "}"}], ",", "0"}], "]"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "We can handle non-bit input a bit more elegantly by adding one more \
form/value pair. Using a blank (",
 StyleBox["_", "Input"],
 ") for the form will create a default value. By creating a message \
associated to the ",
 StyleBox["and", "Input"],
 " function, we can display a useful error message, as shown below. The \
message is defined by setting the symbol ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "::", "tag"}], TraditionalForm]]],
 " equal to the message in quotation marks, where ",
 Cell[BoxData[
  FormBox["f", TraditionalForm]]],
 " is the name of the function and ",
 Cell[BoxData[
  FormBox["tag", TraditionalForm]]],
 " is the \[OpenCurlyDoubleQuote]name\[CloseCurlyDoubleQuote] of the message. \
When this symbol is given as the argument to the ",
 StyleBox[ButtonBox["Message",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Message"], "FunctionLink"],
 " function, the message is shown."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"and", "::", "arg"}], "=", 
   "\"\<and called with non-bit arguments.\>\""}], ";"}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"and", "[", 
   RowBox[{"p_", ",", "q_"}], "]"}], ":=", 
  RowBox[{"Switch", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"p", ",", "q"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"1", ",", "1"}], "}"}], ",", "1", ",", 
    RowBox[{"{", 
     RowBox[{"1", ",", "0"}], "}"}], ",", "0", ",", 
    RowBox[{"{", 
     RowBox[{"0", ",", "1"}], "}"}], ",", "0", ",", 
    RowBox[{"{", 
     RowBox[{"0", ",", "0"}], "}"}], ",", "0", ",", "_", ",", 
    RowBox[{"Message", "[", 
     RowBox[{"and", "::", "arg"}], "]"}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell["Now, applying and to 18 and 5 has a more useful result.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"and", "[", 
  RowBox[{"18", ",", "5"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"and", "::", "arg"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"and called with non-bit arguments.\"\>"}]], "Message", \
"MSG"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Threading and Listable", "Subsubsection"],

Cell[TextData[{
 "We saw above that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "\[CloseCurlyQuote]s built-in function would extend to lists of integers \
without any additional effort on our part",
 ". ",
 "Here we\[CloseCurlyQuote]ll see that it\[CloseCurlyQuote]s easy to make our \
function do that as well."
}], "Text"],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " provides a general way to cause a function to be applied to lists in the \
functions ",
 StyleBox[ButtonBox["Map",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Map"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["MapThread",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/MapThread"], "FunctionLink"],
 ". We describe ",
 StyleBox[ButtonBox["Map",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Map"], "FunctionLink"],
 " first."
}], "Text"],

Cell[TextData[{
 "Given a function of one argument, such as ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", "x", ")"}], "=", 
    SuperscriptBox["x", "2"]}], TraditionalForm]]],
 ", ",
 StyleBox[ButtonBox["Map",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Map"], "FunctionLink"],
 " allows you to have ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " apply the function to all the elements of a list",
 ". ",
 "First, define the function."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"f", "[", "x_", "]"}], " ", ":=", " ", 
  RowBox[{"x", "^", "2"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Now, call ",
 StyleBox[ButtonBox["Map",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Map"], "FunctionLink"],
 " with the name of the function as the first argument and the list of input \
values as the second."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Map", "[", 
  RowBox[{"f", ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "3", ",", "4", ",", "5", ",", "6"}], "}"}]}], 
  "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "4", ",", "9", ",", "16", ",", "25", ",", "36"}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The result, as you see above, is the list of the results of applying the \
function to each element of the list",
 ". ",
 "The same result can be obtained with the ",
 StyleBox["/@", "Input"],
 " operator, as shown below."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"f", "/@", 
  RowBox[{"{", 
   RowBox[{"1", ",", "2", ",", "3", ",", "4", ",", "5", ",", "6"}], 
   "}"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "4", ",", "9", ",", "16", ",", "25", ",", "36"}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "When the function has more than one argument, as ",
 StyleBox["and", "Input"],
 " does, ",
 StyleBox[ButtonBox["MapThread",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/MapThread"], "FunctionLink"],
 " can be used. Like ",
 StyleBox[ButtonBox["Map",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Map"], "FunctionLink"],
 ", ",
 StyleBox[ButtonBox["MapThread",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/MapThread"], "FunctionLink"],
 " takes two arguments and the first is a function. The second argument is a \
list of lists. Provided that each of the inner lists is of the same length, \
the result of ",
 StyleBox[ButtonBox["MapThread",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/MapThread"], "FunctionLink"],
 " is the list formed by evaluating the function with arguments from \
corresponding positions in the lists",
 ". ",
 "For example, we can apply ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"g", "(", 
     RowBox[{"x", ",", "y"}], ")"}], "=", 
    RowBox[{
     SuperscriptBox["x", "2"], "+", 
     SuperscriptBox["y", "3"]}]}], TraditionalForm]]],
 " to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "3"}], "}"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"a", ",", "b", ",", "c"}], "}"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"g", "[", 
   RowBox[{"x_", ",", "y_"}], "]"}], ":=", 
  RowBox[{
   RowBox[{"x", "^", "2"}], "+", 
   RowBox[{"y", "^", "3"}]}]}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MapThread", "[", 
  RowBox[{"g", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"1", ",", "2", ",", "3"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"a", ",", "b", ",", "c"}], "}"}]}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"1", "+", 
    SuperscriptBox["a", "3"]}], ",", 
   RowBox[{"4", "+", 
    SuperscriptBox["b", "3"]}], ",", 
   RowBox[{"9", "+", 
    SuperscriptBox["c", "3"]}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Using ",
 StyleBox[ButtonBox["MapThread",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/MapThread"], "FunctionLink"],
 ", we can compute ",
 Cell[BoxData[
  FormBox[
   RowBox[{"10010", "\[And]", "01011"}], TraditionalForm]]],
 " as follows."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MapThread", "[", 
  RowBox[{"and", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"1", ",", "0", ",", "0", ",", "1", ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "1", ",", "0", ",", "1", ",", "1"}], "}"}]}], "}"}]}],
   "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "0", ",", "0", ",", "1", ",", "0"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "This shows how to thread a function in a particular case. But what we \
really want is for our ",
 StyleBox["and", "Input"],
 " function to behave like this automatically. Fortunately, this is such a \
common requirement for functions, that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " provides a very easy way to do this automatically. The attribute ",
 StyleBox[ButtonBox["Listable",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Listable"], "FunctionLink"],
 ", when applied to a function, tells ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " that the function should be automatically threaded over lists whenever the \
function is given a list as its argument. The ",
 StyleBox[ButtonBox["SetAttributes",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/SetAttributes"], "FunctionLink"],
 " function causes ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to associate the attribute specified in the second argument with the \
object in the first argument."
}], "Text"],

Cell[BoxData[
 RowBox[{"SetAttributes", "[", 
  RowBox[{"and", ",", "Listable"}], "]"}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Now applying ",
 StyleBox["and", "Input"],
 " to lists works just as the built-in ",
 StyleBox[ButtonBox["BitAnd",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BitAnd"], "Input"],
 " does."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"and", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "0", ",", "1", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "1", ",", "0", ",", "1", ",", "1"}], "}"}]}], 
  "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "0", ",", "0", ",", "1", ",", "0"}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["1.2 Applications of Propositional Logic", "Section",
 CellTags->"Section2"],

Cell[TextData[{
 "In this section we will describe how ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "\[CloseCurlyQuote]s computational abilities can be used to solve applied \
problems in propositional logic. In particular, we will consider consistency \
for system specifications and Smullyan logic puzzles."
}], "Text"],

Cell[CellGroupData[{

Cell["System Specifications", "Subsection",
 CellTags->"Section2a"],

Cell["\<\
The textbook describes how system specifications can be translated into \
propositional logic and how it is important that the specifications be \
consistent. As suggested by the textbook, one way to determine whether a set \
of specifications is consistent is with truth tables.\
\>", "Text"],

Cell[TextData[{
 "Recall that a collection of propositions is consistent when there is an \
assignment of truth values to the propositional variables that makes all of \
the propositions in the collection true simultaneously",
 ". ",
 "For example, consider the following collection of compound propositions: ",
 Cell[BoxData[
  FormBox[
   RowBox[{"p", "\[Rule]", 
    RowBox[{"(", 
     RowBox[{"q", "\[And]", "r"}], ")"}]}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"p", "\[Or]", "q"}], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"p", "\[Or]", 
    RowBox[{"\[Not]", "r"}]}], TraditionalForm]]],
 ". ",
 "We can see that these propositions are consistent because we can satisfy \
all three with the assignment p = false, q = true, r = false. In ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", we can confirm this by evaluating the list of propositions with that \
assignment of truth values",
 ". "
}], "Text"],

Cell[TextData[{
 "Above we saw that you can evaluate an expression using the replacement \
operator ",
 StyleBox["/.", "Input"],
 ". ",
 "On the left side of the replacement operator, put the expression we want \
evaluated, in this case a list of the three logical propositions",
 ". ",
 "On the right side of the ",
 StyleBox["/.", "Input"],
 ", enter the assignments as a list of rules of the form ",
 StyleBox["s->v", "Input"],
 " for symbol ",
 StyleBox["s", "Input"],
 " and value ",
 StyleBox["v", "Input"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Implies", "[", 
     RowBox[{"p", ",", 
      RowBox[{"q", "&&", "r"}]}], "]"}], ",", 
    RowBox[{"p", "||", "q"}], ",", 
    RowBox[{"p", "||", 
     RowBox[{"(", 
      RowBox[{"!", "r"}], ")"}]}]}], "}"}], "/.", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"p", "\[Rule]", "False"}], ",", 
    RowBox[{"q", "\[Rule]", "True"}], ",", 
    RowBox[{"r", "\[Rule]", "False"}]}], "}"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"True", ",", "True", ",", "True"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "To determine if a collection of propositions is consistent, we can create a \
truth table",
 ". In the previous section, we created truth tables from scratch using the \
",
 StyleBox[ButtonBox["Do",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Do"], "FunctionLink"],
 " function to loop through all possible assignments of truth values to the \
variables",
 ". ",
 "In this section, we\[CloseCurlyQuote]ll instead use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "\[CloseCurlyQuote]s built-in function ",
 StyleBox[ButtonBox["BooleanTable",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BooleanTable"], "FunctionLink"],
 "."
}], "Text"],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["BooleanTable",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BooleanTable"], "FunctionLink"],
 " function produces the truth values obtained by replacing the variables by \
all possible combinations of true and false. Its first argument is the \
expression to be evaluated and the second argument is a list of the \
propositional variables."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BooleanTable", "[", 
  RowBox[{
   RowBox[{"p", "&&", 
    RowBox[{"(", 
     RowBox[{"!", "q"}], ")"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"p", ",", "q"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"False", ",", "True", ",", "False", ",", "False"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Note that, unlike a truth table you construct by hand, ",
 StyleBox[ButtonBox["BooleanTable",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BooleanTable"], "FunctionLink"],
 " does not show the assignments to the propositional variables. We can see \
the values of the propositional variables by making the first argument a list \
that includes them."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BooleanTable", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "q", ",", 
     RowBox[{"p", "&&", 
      RowBox[{"(", 
       RowBox[{"!", "q"}], ")"}]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"p", ",", "q"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"True", ",", "True", ",", "False"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"True", ",", "False", ",", "True"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"False", ",", "True", ",", "False"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"False", ",", "False", ",", "False"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["TableForm",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TableForm"], "FunctionLink"],
 " function will make the output easier to read. We will apply ",
 StyleBox[ButtonBox["TableForm",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TableForm"], "FunctionLink"],
 " with the postfix operator (",
 StyleBox["//", "Input"],
 "). The postfix operator allows you to put the name of a function after an \
expression. It is commonly used for functions that affect the display of a \
result and has the benefit of making the main part of the command being \
evaluated easier to read."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"BooleanTable", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"p", ",", "q", ",", 
      RowBox[{"p", "&&", 
       RowBox[{"(", 
        RowBox[{"!", "q"}], ")"}]}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"p", ",", "q"}], "}"}]}], "]"}], "//", "TableForm"}]], "Input"],

Cell[BoxData[
 TagBox[GridBox[{
    {"True", "True", "False"},
    {"True", "False", "True"},
    {"False", "True", "False"},
    {"False", "False", "False"}
   },
   GridBoxAlignment->{
    "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
     "RowsIndexed" -> {}},
   GridBoxSpacings->{"Columns" -> {
       Offset[0.27999999999999997`], {
        Offset[2.0999999999999996`]}, 
       Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
       Offset[0.2], {
        Offset[0.4]}, 
       Offset[0.2]}, "RowsIndexed" -> {}}],
  Function[BoxForm`e$, 
   TableForm[BoxForm`e$]]]], "Output"]
}, Open  ]],

Cell["\<\
Returning to the question of consistency, consider Example 4 from Section 1.2 \
of the text. We translate the three specifications as the following list of \
propositions.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"specEx4", "=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"p", "||", "q"}], ",", 
    RowBox[{"!", "p"}], ",", 
    RowBox[{"Implies", "[", 
     RowBox[{"p", ",", "q"}], "]"}]}], "}"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"p", "||", "q"}], ",", 
   RowBox[{"!", "p"}], ",", 
   RowBox[{"p", "\[Implies]", "q"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Then we can construct the truth table using ",
 StyleBox[ButtonBox["BooleanTable",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BooleanTable"], "FunctionLink"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"BooleanTable", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"p", ",", "q", ",", "specEx4"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"p", ",", "q"}], "}"}]}], "]"}], "//", "TableForm"}]], "Input"],

Cell[BoxData[
 InterpretationBox[GridBox[{
    {"True", "True", GridBox[{
       {"True"},
       {"False"},
       {"True"}
      },
      GridBoxAlignment->{
       "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
        "RowsIndexed" -> {}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.7]}, 
          Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
          Offset[0.2], {
           Offset[0.1]}, 
          Offset[0.2]}, "RowsIndexed" -> {}}]},
    {"True", "False", GridBox[{
       {"True"},
       {"False"},
       {"False"}
      },
      GridBoxAlignment->{
       "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
        "RowsIndexed" -> {}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.7]}, 
          Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
          Offset[0.2], {
           Offset[0.1]}, 
          Offset[0.2]}, "RowsIndexed" -> {}}]},
    {"False", "True", GridBox[{
       {"True"},
       {"True"},
       {"True"}
      },
      GridBoxAlignment->{
       "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
        "RowsIndexed" -> {}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.7]}, 
          Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
          Offset[0.2], {
           Offset[0.1]}, 
          Offset[0.2]}, "RowsIndexed" -> {}}]},
    {"False", "False", GridBox[{
       {"False"},
       {"True"},
       {"True"}
      },
      GridBoxAlignment->{
       "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
        "RowsIndexed" -> {}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.7]}, 
          Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
          Offset[0.2], {
           Offset[0.1]}, 
          Offset[0.2]}, "RowsIndexed" -> {}}]}
   },
   GridBoxAlignment->{
    "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
     "RowsIndexed" -> {}},
   GridBoxSpacings->{"Columns" -> {
       Offset[0.27999999999999997`], {
        Offset[2.0999999999999996`]}, 
       Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
       Offset[0.2], {
        Offset[0.4]}, 
       Offset[0.2]}, "RowsIndexed" -> {}}],
  TableForm[{{True, True, {True, False, True}}, {
    True, False, {True, False, False}}, {False, True, {True, True, True}}, {
    False, False, {False, True, True}}}]]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Notice that because ",
 StyleBox["specEx4", "Input"],
 " is itself a list, ",
 StyleBox[ButtonBox["TableForm",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TableForm"], "FunctionLink"],
 " displays the results from the three component propositions as a column \
within the row corresponding to the values for ",
 StyleBox["p", "Input"],
 " and ",
 StyleBox["q", "Input"],
 ". ",
 "We see that the only assignment of truth values that results in all three \
statements being satisfied is with ",
 Cell[BoxData[
  FormBox[
   RowBox[{"p", "=", "false"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"q", "=", "true"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[{
 "We can make the output a bit easier to read if, instead of considering the \
truth table for the list of the propositions, we consider the proposition \
formed by the conjunction of the individual propositions:",
 " ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"p", "\[Or]", "q"}], ")"}], "\[And]", 
    RowBox[{"(", 
     RowBox[{"\[Not]", "p"}], ")"}], "\[And]", 
    RowBox[{"(", 
     RowBox[{"p", "\[Rule]", "q"}], ")"}]}], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"specEx4b", "=", 
  RowBox[{"And", "[", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"p", "||", "q"}], ")"}], ",", 
    RowBox[{"!", "p"}], ",", 
    RowBox[{"Implies", "[", 
     RowBox[{"p", ",", "q"}], "]"}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"p", "||", "q"}], ")"}], "&&", 
  RowBox[{"!", "p"}], "&&", 
  RowBox[{"(", 
   RowBox[{"p", "\[Implies]", "q"}], ")"}]}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"BooleanTable", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"p", ",", "q", ",", "specEx4b"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"p", ",", "q"}], "}"}]}], "]"}], "//", "TableForm"}]], "Input"],

Cell[BoxData[
 TagBox[GridBox[{
    {"True", "True", "False"},
    {"True", "False", "False"},
    {"False", "True", "True"},
    {"False", "False", "False"}
   },
   GridBoxAlignment->{
    "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
     "RowsIndexed" -> {}},
   GridBoxSpacings->{"Columns" -> {
       Offset[0.27999999999999997`], {
        Offset[2.0999999999999996`]}, 
       Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
       Offset[0.2], {
        Offset[0.4]}, 
       Offset[0.2]}, "RowsIndexed" -> {}}],
  Function[BoxForm`e$, 
   TableForm[BoxForm`e$]]]], "Output"]
}, Open  ]],

Cell["\<\
In this case, the fact that the final truth value in the third row is true \
tells us that this assignment of truth values satisfies all of the \
propositions in the system specification.\
\>", "Text"],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " also has useful built-in functions for checking for consistency. The ",
 StyleBox[ButtonBox["SatisfiableQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/SatisfiableQ"], "FunctionLink"],
 " function accepts the same arguments as ",
 StyleBox[ButtonBox["BooleanTable",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BooleanTable"], "FunctionLink"],
 " (a Boolean expression and the list of propositional variables)",
 ". ",
 "Note that you may not give a list of expressions as the first argument to \
",
 StyleBox[ButtonBox["SatisfiableQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/SatisfiableQ"], "FunctionLink"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SatisfiableQ", "[", 
  RowBox[{"specEx4b", ",", 
   RowBox[{"{", 
    RowBox[{"p", ",", "q"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["SatisfiabilityInstances",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/SatisfiabilityInstances"], "FunctionLink"],
 " command will generate an assignment of truth values to the variables that \
do in fact satisfy the proposition, assuming it is satisfiable."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SatisfiabilityInstances", "[", 
  RowBox[{"specEx4b", ",", 
   RowBox[{"{", 
    RowBox[{"p", ",", "q"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"{", 
   RowBox[{"False", ",", "True"}], "}"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "By providing a positive integer as an optional third argument, you can ask \
for more choices that make the proposition true",
 ". ",
 "Below, we find all 3 ways that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"p", "\[Rule]", "q"}], TraditionalForm]]],
 " can be satisfied."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SatisfiabilityInstances", "[", 
  RowBox[{
   RowBox[{"Implies", "[", 
    RowBox[{"p", ",", "q"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"p", ",", "q"}], "}"}], ",", "3"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"True", ",", "True"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"False", ",", "True"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"False", ",", "False"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "If we add, as in Example 5, the proposition ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Not]", "q"}], TraditionalForm]]],
 ", we see that all of the assignments yield false for the conjunction of all \
four propositions."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"specEx5", "=", 
  RowBox[{"specEx4b", " ", "&&", " ", 
   RowBox[{"!", "q"}]}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"p", "||", "q"}], ")"}], "&&", 
  RowBox[{"!", "p"}], "&&", 
  RowBox[{"(", 
   RowBox[{"p", "\[Implies]", "q"}], ")"}], "&&", 
  RowBox[{"!", "q"}]}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"BooleanTable", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"p", ",", "q", ",", "specEx5"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"p", ",", "q"}], "}"}]}], "]"}], "//", "TableForm"}]], "Input"],

Cell[BoxData[
 TagBox[GridBox[{
    {"True", "True", "False"},
    {"True", "False", "False"},
    {"False", "True", "False"},
    {"False", "False", "False"}
   },
   GridBoxAlignment->{
    "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
     "RowsIndexed" -> {}},
   GridBoxSpacings->{"Columns" -> {
       Offset[0.27999999999999997`], {
        Offset[2.0999999999999996`]}, 
       Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
       Offset[0.2], {
        Offset[0.4]}, 
       Offset[0.2]}, "RowsIndexed" -> {}}],
  Function[BoxForm`e$, 
   TableForm[BoxForm`e$]]]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Also, note that ",
 StyleBox[ButtonBox["SatisfiableQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/SatisfiableQ"], "FunctionLink"],
 " returns false and ",
 StyleBox[ButtonBox["SatisfiabilityInstances",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/SatisfiabilityInstances"], "FunctionLink"],
 " returns an empty list."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SatisfiableQ", "[", 
  RowBox[{"specEx5", ",", 
   RowBox[{"{", 
    RowBox[{"p", ",", "q"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData["False"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SatisfiabilityInstances", "[", 
  RowBox[{"specEx5", ",", 
   RowBox[{"{", 
    RowBox[{"p", ",", "q"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", "}"}]], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Logic Puzzles", "Subsection",
 CellTags->"Section2b"],

Cell["\<\
Recall the knights and knaves puzzle presented in Example 7 of Section 1.2 of \
the text. In this puzzle, you are asked to imagine an island on which each \
inhabitant is either a knight and always tells the truth or is a knave and \
always lies. You meet two people named A and B. Person A says \
\[OpenCurlyDoubleQuote]B is a knight\[CloseCurlyDoubleQuote] and person B \
says \[OpenCurlyDoubleQuote]The two of us are opposite types.\
\[CloseCurlyDoubleQuote] The puzzle is to determine which kind of inhabitants \
A and B are.\
\>", "Text"],

Cell[TextData[{
 "We can solve this problem with ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " using truth tables. First we must write A and B\[CloseCurlyQuote]s \
statements as propositions. Let ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " represent the statement that A is a knight and ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " the statement that B is a knight. Then A\[CloseCurlyQuote]s statement is \
\[OpenCurlyDoubleQuote]",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 "\[CloseCurlyDoubleQuote], and B\[CloseCurlyQuote]s statement is \
\[OpenCurlyDoubleQuote]",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"a", "\[And]", 
      RowBox[{"\[Not]", "b"}]}], ")"}], "\[Or]", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"\[Not]", "a"}], "\[And]", "b"}], ")"}]}], TraditionalForm]]],
 "\[CloseCurlyDoubleQuote], as discussed in the text."
}], "Text"],

Cell[TextData[{
 "While these propositions precisely express the content of A and B\
\[CloseCurlyQuote]s assertions, it does not capture the additional \
information that A and B are making the statements. We know, for instance, \
that A either always tells the truth (knight) or always lies (knave). If A is \
a knight, then we know the statement \[OpenCurlyDoubleQuote]",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 "\[CloseCurlyDoubleQuote] is true. If A is not a knight, then we know the \
statement is false. In other words, the truth value of the proposition ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 ", that A is a knight, is the same as the truth value of \
A\[CloseCurlyQuote]s statement, and likewise for B. Therefore, we can capture \
the meaning of \[OpenCurlyDoubleQuote]A says proposition ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 "\[CloseCurlyDoubleQuote] by the proposition ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", "\[LeftRightArrow]", "p"}], TraditionalForm]]],
 ". Using the function ",
 StyleBox[ButtonBox["Equivalent",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Equivalent"], "FunctionLink"],
 ", we can express the two statements in the puzzle in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " as follows."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ex7a", "=", 
  RowBox[{"Equivalent", "[", 
   RowBox[{"a", ",", "b"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"a", "\[Equivalent]", "b"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ex7b", "=", 
  RowBox[{"Equivalent", "[", 
   RowBox[{"b", ",", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", "&&", 
       RowBox[{"!", "b"}]}], ")"}], "||", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"!", "a"}], "&&", "b"}], ")"}]}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"b", "\[Equivalent]", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"a", "&&", 
     RowBox[{"!", "b"}]}], ")"}], "||", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"!", "a"}], "&&", "b"}], ")"}]}]}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Like the system specifications above, a solution to this puzzle will \
consist of an assignment of truth values to the propositions ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " that make both people\[CloseCurlyQuote]s statements true."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SatisfiabilityInstances", "[", 
  RowBox[{
   RowBox[{"ex7a", "&&", "ex7b"}], ",", 
   RowBox[{"{", 
    RowBox[{"a", ",", "b"}], "}"}], ",", "4"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"{", 
   RowBox[{"False", ",", "False"}], "}"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "We see that both statements are satisfied when both propositions ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " are false, that is, when A and B are both knaves. Note also that since we \
asked, in the final argument, for as many as 4 different instances but only \
one was returned, we know that this is the only solution to the puzzle."
}], "Text"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["1.3 Propositional Equivalence", "Section",
 CellTags->"Section3"],

Cell[TextData[{
 "In this section we consider logical equivalence of propositions",
 ". ",
 "We will first look at ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "\[CloseCurlyQuote]s built-in functions for testing equivalence, and then we \
will create a function from scratch to accomplish the same goal."
}], "Text"],

Cell[CellGroupData[{

Cell["Built-in Functions", "Subsection",
 CellTags->"Section3a"],

Cell[TextData[{
 "Two propositions ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["q", TraditionalForm]]],
 " are logically equivalent if the proposition ",
 Cell[BoxData[
  FormBox[
   RowBox[{"p", "\[LeftRightArrow]", "q"}], TraditionalForm]]],
 " is a tautology. ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " includes a function for checking whether a proposition is a tautology, ",
 StyleBox[ButtonBox["TautologyQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TautologyQ"], "FunctionLink"],
 ". This function uses the same arguments as ",
 StyleBox[ButtonBox["BooleanTable",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BooleanTable"], "FunctionLink"],
 ", ",
 StyleBox[ButtonBox["SatisfiableQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/SatisfiableQ"], "FunctionLink"],
 ", and ",
 StyleBox[ButtonBox["SatisfiabilityInstances",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/SatisfiabilityInstances"], "FunctionLink"],
 " do, as described above. Specifically, the first argument should be the \
proposition and the second argument should be a list of the propositional \
variables",
 ". "
}], "Text"],

Cell[TextData[{
 "For example, we can confirm that the DeMorgan\[CloseCurlyQuote]s Law ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Not]", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"p", "\[And]", "q"}], ")"}], "\[Congruent]", 
      RowBox[{"\[Not]", "p"}]}]}], "\[Or]", 
    RowBox[{"\[Not]", "q"}]}], TraditionalForm]]],
 " is a propositional equivalence as shown below."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TautologyQ", "[", 
  RowBox[{
   RowBox[{"Equivalent", "[", 
    RowBox[{
     RowBox[{"!", 
      RowBox[{"(", 
       RowBox[{"p", "&&", "q"}], ")"}]}], ",", 
     RowBox[{
      RowBox[{"!", "p"}], "||", 
      RowBox[{"!", "q"}]}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"p", ",", "q"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[TextData[{
 "Remember that the ",
 StyleBox[ButtonBox["Equivalent",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Equivalent"], "FunctionLink"],
 " function, used above, is ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "\[CloseCurlyQuote]s function for forming the biconditional proposition, and \
should not be confused with the notion of equivalence as used in Section 1.3 \
of the textbook",
 ". "
}], "Text"],

Cell[TextData[{
 "Note that the second argument to ",
 StyleBox[ButtonBox["TautologyQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TautologyQ"], "FunctionLink"],
 " is not generally necessary. ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "\[CloseCurlyQuote]s ",
 StyleBox[ButtonBox["BooleanVariables",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BooleanVariables"], "FunctionLink"],
 " function, which determines the variables in a logical expression, will \
invisibly supply the missing argument. This is, in fact, true about most of \
the functions that require the variable list as the second argument. We \
demonstrate with the other DeMorgan\[CloseCurlyQuote]s Law."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TautologyQ", "[", 
  RowBox[{"Equivalent", "[", 
   RowBox[{
    RowBox[{"!", 
     RowBox[{"(", 
      RowBox[{"p", "||", "q"}], ")"}]}], ",", 
    RowBox[{
     RowBox[{"!", "p"}], "&&", 
     RowBox[{"!", "q"}]}]}], "]"}], "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[TextData[{
 "You might find it convenient to have a single function that, given two \
propositions, will determine whether they are logically equivalent. In ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", this is easy to achieve. We just need to create a function that takes two \
propositions, uses the ",
 StyleBox[ButtonBox["Equivalent",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Equivalent"], "FunctionLink"],
 " function to create the biconditional, and then applies ",
 StyleBox[ButtonBox["TautologyQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TautologyQ"], "FunctionLink"],
 "."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"equivalentQ", "[", 
   RowBox[{"p_", ",", "q_"}], "]"}], ":=", 
  RowBox[{"TautologyQ", "[", 
   RowBox[{"Equivalent", "[", 
    RowBox[{"p", ",", "q"}], "]"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell["\<\
We apply this function to see if we can generalize \
DeMorgan\[CloseCurlyQuote]s Laws to three variables.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"equivalentQ", "[", 
  RowBox[{
   RowBox[{"!", 
    RowBox[{"(", 
     RowBox[{"p", "||", "q", "||", "r"}], ")"}]}], ",", 
   RowBox[{
    RowBox[{"!", "p"}], "&&", 
    RowBox[{"!", "q"}], "&&", 
    RowBox[{"!", "r"}]}]}], "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"equivalentQ", "[", 
  RowBox[{
   RowBox[{"!", 
    RowBox[{"(", 
     RowBox[{"p", "&&", "q", "&&", "r"}], ")"}]}], ",", 
   RowBox[{
    RowBox[{"!", "p"}], "||", 
    RowBox[{"!", "q"}], "||", 
    RowBox[{"!", "r"}]}]}], "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Built from Scratch Function", "Subsection",
 CellTags->"Section3b"],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " provides very complete built-in support for working with logical \
propositions and, in particular, checking propositional equivalence. Here, \
however, we are going to build a new function for checking whether or not two \
propositions are logically equivalent using a minimum of existing high-level \
functions. In fact, other than asking ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to evaluate propositional expressions for particular truth values assigned \
to propositional variables, we will make use only of ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "\[CloseCurlyQuote]s essential programming functionality."
}], "Text"],

Cell[TextData[{
 "There are two goals here. First, to illustrate more of ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "\[CloseCurlyQuote]s programming abilities. Second, to reveal some of the \
more fundamental concepts and methods used in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ". "
}], "Text"],

Cell[TextData[{
 "We will create a function ",
 StyleBox["myEquivalentQ", "Input"],
 ", that has the same effect as the ",
 StyleBox["equivalentQ", "Input"],
 " that we built above using ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "\[CloseCurlyQuote]s built-in functions. Specifically, it should take two \
propositions and determine whether or not they are equivalent. This will \
require quite a bit of work. The main hurdles for such a function are: (1) \
having ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " determine what propositional variables are used in the input propositions, \
and (2) without ",
 StyleBox["a priori",
  FontSlant->"Italic"],
 " knowledge of the number of propositional variables, having ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " test every possible assignment of truth values. Note that we could avoid \
both of these hurdles by insisting that the propositional variables be \
limited to a certain small set of symbols, perhaps p, q, r, and s. Then we \
could implement the function using a static nested ",
 StyleBox[ButtonBox["Do",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Do"], "FunctionLink"],
 " loop."
}], "Text"],

Cell["\<\
However, the two hurdles mentioned are not insurmountable, will provide a \
much more elegant and flexible procedure, and will also give us the \
opportunity to see examples of some important programming constructs.\
\>", "Text"],

Cell[CellGroupData[{

Cell["Extracting Variables", "Subsubsection"],

Cell[TextData[{
 "The first hurdle is to get ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to determine the variables used in a logical expression. Consider the \
following example."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"variableEx", "=", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"p", "&&", "q"}], ")"}], "||", 
     RowBox[{"(", 
      RowBox[{"p", "&&", 
       RowBox[{"!", "r"}]}], ")"}]}], ")"}], "&&", 
   RowBox[{"Implies", "[", 
    RowBox[{"s", ",", "r"}], "]"}]}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"p", "&&", "q"}], ")"}], "||", 
    RowBox[{"(", 
     RowBox[{"p", "&&", 
      RowBox[{"!", "r"}]}], ")"}]}], ")"}], "&&", 
  RowBox[{"(", 
   RowBox[{"s", "\[Implies]", "r"}], ")"}]}]], "Output"]
}, Open  ]],

Cell["\<\
Our task is to write a function that will, given the above expression, tell \
us that the variables in use are p, q, r, and s.\
\>", "Text"],

Cell[CellGroupData[{

Cell["Replacing the Head", "Subsubsubsection",
 PageBreakWithin->False,
 PageBreakBelow->False,
 GroupPageBreakWithin->False],

Cell[TextData[{
 "Fundamentally, everything in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " is an expression. And every expression is of the form ",
 Cell[BoxData[
  RowBox[{"head", "[", 
   RowBox[{Cell[TextData[Cell[BoxData[
     FormBox[
      SubscriptBox["arg", "1"], TraditionalForm]]]], "CodeVar"], ",", Cell[
    TextData[Cell[BoxData[
     FormBox[
      SubscriptBox["arg", "2"], TraditionalForm]]]], "CodeVar"], ",", "..."}],
    "]"}]], "myCode"],
 ", that is, a head followed by arguments in brackets and separated by \
commas. You can see this structure at the heart of any expression by using \
the ",
 StyleBox[ButtonBox["FullForm",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/FullForm"], "FunctionLink"],
 " function. Below, we show the full form of three examples. Recall that the \
postfix operator (",
 StyleBox["//", "Input"],
 ") allows us to put the name of the function at the end of the input."
}], "Text",
 PageBreakAbove->False,
 PageBreakWithin->False],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"x", "+", "y"}], "//", "FullForm"}]], "Input"],

Cell[BoxData[
 TagBox[
  StyleBox[
   RowBox[{"Plus", "[", 
    RowBox[{"x", ",", "y"}], "]"}],
   ShowSpecialCharacters->False,
   ShowStringCharacters->True,
   NumberMarks->True],
  FullForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"variableEx", "//", "FullForm"}]], "Input"],

Cell[BoxData[
 TagBox[
  StyleBox[
   RowBox[{"And", "[", 
    RowBox[{
     RowBox[{"Or", "[", 
      RowBox[{
       RowBox[{"And", "[", 
        RowBox[{"p", ",", "q"}], "]"}], ",", 
       RowBox[{"And", "[", 
        RowBox[{"p", ",", 
         RowBox[{"Not", "[", "r", "]"}]}], "]"}]}], "]"}], ",", 
     RowBox[{"Implies", "[", 
      RowBox[{"s", ",", "r"}], "]"}]}], "]"}],
   ShowSpecialCharacters->False,
   ShowStringCharacters->True,
   NumberMarks->True],
  FullForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{"p", ",", "q", ",", "r", ",", "s"}], "}"}], "//", 
  "FullForm"}]], "Input"],

Cell[BoxData[
 TagBox[
  StyleBox[
   RowBox[{"List", "[", 
    RowBox[{"p", ",", "q", ",", "r", ",", "s"}], "]"}],
   ShowSpecialCharacters->False,
   ShowStringCharacters->True,
   NumberMarks->True],
  FullForm]], "Output"]
}, Open  ]],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " provides a function, ",
 StyleBox[ButtonBox["Head",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Head"], "FunctionLink"],
 ", that takes an expression and returns the type of head of that \
expression",
 ". "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Head", "[", 
  RowBox[{"x", "+", "y"}], "]"}]], "Input"],

Cell[BoxData["Plus"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Head", "[", "variableEx", "]"}]], "Input"],

Cell[BoxData["And"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Head", "[", 
  RowBox[{"{", 
   RowBox[{"p", ",", "q", ",", "r", ",", "s"}], "}"}], "]"}]], "Input"],

Cell[BoxData["List"], "Output"]
}, Open  ]],

Cell[TextData[{
 "You can also access the head of an expression using the ",
 StyleBox[ButtonBox["Part",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Part"], "FunctionLink"],
 " (",
 StyleBox["[[\[Ellipsis]]]", "Input"],
 ") operator with index 0."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"x", "+", "y"}], ")"}], "[", 
  RowBox[{"[", "0", "]"}], "]"}]], "Input"],

Cell[BoxData["Plus"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"variableEx", "[", 
  RowBox[{"[", "0", "]"}], "]"}]], "Input"],

Cell[BoxData["And"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{"p", ",", "q", ",", "r", ",", "s"}], "}"}], "[", 
  RowBox[{"[", "0", "]"}], "]"}]], "Input"],

Cell[BoxData["List"], "Output"]
}, Open  ]],

Cell[TextData[{
 "Remember that our goal here is to transform a logical expression, such as \
",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"p", "\[And]", "q"}], ")"}], "\[Or]", 
      RowBox[{"(", 
       RowBox[{"p", "\[And]", 
        RowBox[{"\[Not]", "r"}]}], ")"}]}], ")"}], "\[And]", 
    RowBox[{"(", 
     RowBox[{"s", "\[Rule]", "r"}], ")"}]}], TraditionalForm]]],
 " into a list ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"p", ",", "q", ",", "r", ",", "s"}], "}"}], TraditionalForm]]],
 ". Since the main difference, in terms of the internal representation of the \
two objects, is their heads, it is natural to ask if we can change the head. \
In particular, in our example ",
 StyleBox["variableEx", "Input"],
 ", the head is ",
 StyleBox["And", "Input"],
 ". If we can replace the ",
 StyleBox["And", "Input"],
 " head with a ",
 StyleBox["List", "Input"],
 " head, we would have a list comprised of the two parts of the expression, \
as illustrated below."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"List", "[", 
  RowBox[{
   RowBox[{"Or", "[", 
    RowBox[{
     RowBox[{"And", "[", 
      RowBox[{"p", ",", "q"}], "]"}], ",", 
     RowBox[{"And", "[", 
      RowBox[{"p", ",", 
       RowBox[{"Not", "[", "r", "]"}]}], "]"}]}], "]"}], ",", 
   RowBox[{"Implies", "[", 
    RowBox[{"s", ",", "r"}], "]"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"p", "&&", "q"}], ")"}], "||", 
    RowBox[{"(", 
     RowBox[{"p", "&&", 
      RowBox[{"!", "r"}]}], ")"}]}], ",", 
   RowBox[{"s", "\[Implies]", "r"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Our strategy, broadly, will be to replace all of the heads in the logical \
expression with ",
 StyleBox["List", "Input"],
 " heads. There are two approaches to replacing the head of an expression. \
One is to use the fact that the head lies at index 0 to replace the heads by \
assigning the 0 indexed element to ",
 StyleBox["List", "Input"],
 " using the syntax ",
 Cell[BoxData[
  RowBox[{
   RowBox[{"x", "[", 
    RowBox[{"[", "0", "]"}], "]"}], "=", "List"}]], "myCode"],
 ", as illustrated below."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sumExample", "=", 
  RowBox[{"x", "+", "y"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"x", "+", "y"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"sumExample", "[", 
   RowBox[{"[", "0", "]"}], "]"}], "=", "List"}]], "Input"],

Cell[BoxData["List"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["sumExample"], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"x", ",", "y"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The second approach is to use the ",
 StyleBox[ButtonBox["Apply",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Apply"], "FunctionLink"],
 " (",
 StyleBox["@@", "Input"],
 ") function or operator. An expression formed from the desired head, \
followed by two at symbols and the original expression will output the \
expression with the new head. Unlike the previous approach, if the expression \
is stored as a symbol, the stored expression is not changed, unless you \
explicitly reassign the output to the symbol. We illustrate by transforming \
",
 StyleBox["sumExample", "Input"],
 " from a list into a product."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sumExample", "=", 
  RowBox[{"Times", "@@", "sumExample"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"x", " ", "y"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Note that the ",
 StyleBox[ButtonBox["Head",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Head"], "FunctionLink"],
 " command gives us a way to test what kind of expression we have. In \
particular, we can differentiate between variables, which have head ",
 StyleBox["Symbol", "Input"],
 ", and other expressions. Note that to compare heads, you must use the ",
 StyleBox[ButtonBox["SameQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/SameQ"], "FunctionLink"],
 " relation (",
 StyleBox["===", "Input"],
 ") rather than ",
 StyleBox[ButtonBox["Equal",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Equal"], "FunctionLink"],
 " (",
 StyleBox["==", "Input"],
 "), which only applies to raw data (such as numerical values and strings). "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"If", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"Head", "[", 
     RowBox[{"x", "+", "y"}], "]"}], "===", 
    RowBox[{"Head", "[", 
     RowBox[{"x", "-", "y"}], "]"}]}], ",", "\[IndentingNewLine]", 
   RowBox[{"Print", "[", "\"\<+ equals -\>\"", "]"}], ",", 
   "\[IndentingNewLine]", 
   RowBox[{"Print", "[", "\"\<different\>\"", "]"}]}], "]"}]], "Input"],

Cell[BoxData["\<\"+ equals -\"\>"], "Print"]
}, Open  ]],

Cell[TextData[{
 "The above shows that the heads of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "+", "y"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "-", 
    RowBox[{"y", Cell[""]}]}], TraditionalForm]]],
 "are in fact the same. Both expressions have head ",
 StyleBox["Plus", "Input"],
 ". (We could also do this with the ",
 StyleBox["[[0]]", "Input"],
 " syntax, but the ",
 StyleBox[ButtonBox["Head",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Head"], "FunctionLink"],
 " function makes it clearer what we\[CloseCurlyQuote]re doing.)"
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Illustrating with an Example", "Subsubsubsection"],

Cell[TextData[{
 "We can now remove operators to obtain simpler expressions, and we have a \
way to test whether an expression is a variable or not. The general idea is \
that we keep replacing the heads of the subexpressions until we\
\[CloseCurlyQuote]re down to nothing but names. The strategy we will use is a \
fairly typical one. We illustrate the approach step by step with the ",
 StyleBox["variableEx", "Input"],
 " example first, and then we\[CloseCurlyQuote]ll build a function."
}], "Text"],

Cell[TextData[{
 "First we define a new symbol, ",
 StyleBox["variableExList", "Input"],
 ", to be the result of applying (",
 StyleBox[ButtonBox["Apply",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Apply"], "FunctionLink"],
 ", ",
 StyleBox["@@", "Input"],
 ") the ",
 StyleBox[ButtonBox["List",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/List"], "FunctionLink"],
 " head to the ",
 StyleBox["variableEx", "Input"],
 " expression. Remember that this does not change the expression stored in ",
 StyleBox["variableEx", "Input"],
 ", We wish to preserve ",
 StyleBox["variableEx", "Input"],
 ", which is why we take this approach here. Moving forward, we will use the ",
 StyleBox[ButtonBox["Part",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Part"], "FunctionLink"],
 " (",
 StyleBox["[[\[Ellipsis]]]", "Input"],
 ") approach."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"variableExList", "=", 
  RowBox[{"List", "@@", "variableEx"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"p", "&&", "q"}], ")"}], "||", 
    RowBox[{"(", 
     RowBox[{"p", "&&", 
      RowBox[{"!", "r"}]}], ")"}]}], ",", 
   RowBox[{"s", "\[Implies]", "r"}]}], "}"}]], "Output"]
}, Open  ]],

Cell["\<\
Observe that the top-most conjunction has been removed and we now have a list \
of the two subexpressions.\
\>", "Text"],

Cell[TextData[{
 "Now we need to do the same thing to the elements of this list. Remember \
that the ",
 StyleBox[ButtonBox["Part",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Part"], "FunctionLink"],
 " function (",
 StyleBox["[[\[Ellipsis]]]", "Input"],
 ") is used to obtain and to modify elements of a list. So we can obtain the \
first element in the list as follows."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"variableExList", "[", 
  RowBox[{"[", "1", "]"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"p", "&&", "q"}], ")"}], "||", 
  RowBox[{"(", 
   RowBox[{"p", "&&", 
    RowBox[{"!", "r"}]}], ")"}]}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "We can turn this into a list by assigning the 0-indexed element of ",
 StyleBox["variableExList[[1]]", "Input"],
 " to ",
 StyleBox[ButtonBox["List",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/List"], "FunctionLink"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"variableExList", "[", 
    RowBox[{"[", "1", "]"}], "]"}], "[", 
   RowBox[{"[", "0", "]"}], "]"}], "=", "List"}]], "Input"],

Cell[BoxData["List"], "Output"]
}, Open  ]],

Cell[TextData[{
 "Inspecting ",
 StyleBox["variableExList", "Input"],
 ", we see that this has replaced what was the first element with the new \
result."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 StyleBox["variableExList", "Input"]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"p", "&&", "q"}], ",", 
     RowBox[{"p", "&&", 
      RowBox[{"!", "r"}]}]}], "}"}], ",", 
   RowBox[{"s", "\[Implies]", "r"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "You can see that we\[CloseCurlyQuote]ve already made quite a bit of \
progress. But now we have lists nested together. We can eliminate this \
nesting with the ",
 StyleBox[ButtonBox["Flatten",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Flatten"], "FunctionLink"],
 " function. We assign the result of applying the function to ",
 StyleBox["variableExList", "Input"],
 " back to ",
 StyleBox["variableExList", "Input"],
 ", so the result is kept."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"variableExList", "=", 
  RowBox[{"Flatten", "[", "variableExList", "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"p", "&&", "q"}], ",", 
   RowBox[{"p", "&&", 
    RowBox[{"!", "r"}]}], ",", 
   RowBox[{"s", "\[Implies]", "r"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The first element of ",
 StyleBox["variableExList", "Input"],
 " is still a logical expression, so we repeat.  This time, we\
\[CloseCurlyQuote]ll use ",
 StyleBox["[[1,0]]", "Input"],
 ", which is shorthand for ",
 StyleBox["[[1]][[0]]", "Input"],
 ".  We also combine the asisgnment and the inspection of ",
 StyleBox["variableExList", "Input"],
 " into one input."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"variableExList", "[", 
    RowBox[{"[", 
     RowBox[{"1", ",", "0"}], "]"}], "]"}], "=", "List"}], 
  ";"}], "\[IndentingNewLine]", "variableExList"}], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "q"}], "}"}], ",", 
   RowBox[{"p", "&&", 
    RowBox[{"!", "r"}]}], ",", 
   RowBox[{"s", "\[Implies]", "r"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Again we use ",
 StyleBox[ButtonBox["Flatten",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Flatten"], "FunctionLink"],
 " since this has created a nested list structure."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"variableExList", "=", 
  RowBox[{"Flatten", "[", "variableExList", "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"p", ",", "q", ",", 
   RowBox[{"p", "&&", 
    RowBox[{"!", "r"}]}], ",", 
   RowBox[{"s", "\[Implies]", "r"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The first two elements of ",
 StyleBox["variableExList", "Input"],
 " are now symbols. So we skip to the third element. Again, we change the \
head of the third element to the ",
 StyleBox["List", "Input"],
 " head."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"variableExList", "[", 
    RowBox[{"[", 
     RowBox[{"3", ",", "0"}], "]"}], "]"}], "=", "List"}], 
  ";"}], "\[IndentingNewLine]", "variableExList"}], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"p", ",", "q", ",", 
   RowBox[{"{", 
    RowBox[{"p", ",", 
     RowBox[{"!", "r"}]}], "}"}], ",", 
   RowBox[{"s", "\[Implies]", "r"}]}], "}"}]], "Output"]
}, Open  ]],

Cell["And again flatten the resulting list.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"variableExList", " ", "=", " ", 
  RowBox[{"Flatten", "[", "variableExList", "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"p", ",", "q", ",", "p", ",", 
   RowBox[{"!", "r"}], ",", 
   RowBox[{"s", "\[Implies]", "r"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Now that the third element is a symbol, we do the same thing with the \
fourth element of ",
 StyleBox["variableExList", "Input"],
 ".  We also include the ",
 StyleBox[ButtonBox["Flatten",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Flatten"], "FunctionLink"],
 " step in the same input."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"variableExList", "[", 
    RowBox[{"[", 
     RowBox[{"4", ",", "0"}], "]"}], "]"}], "=", "List"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"variableExList", "=", 
  RowBox[{"Flatten", "[", "variableExList", "]"}]}]}], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"p", ",", "q", ",", "p", ",", "r", ",", 
   RowBox[{"s", "\[Implies]", "r"}]}], "}"}]], "Output"]
}, Open  ]],

Cell["And once more.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"variableExList", "[", 
    RowBox[{"[", 
     RowBox[{"5", ",", "0"}], "]"}], "]"}], "=", "List"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"variableExList", "=", 
  RowBox[{"Flatten", "[", "variableExList", "]"}]}]}], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"p", ",", "q", ",", "p", ",", "r", ",", "s", ",", "r"}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Now that every element in the list is a variable, we remove the duplicate \
elements with ",
 StyleBox[ButtonBox["DeleteDuplicates",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/DeleteDuplicates"], "FunctionLink"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"variableExList", "=", 
  RowBox[{"DeleteDuplicates", "[", "variableExList", "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"p", ",", "q", ",", "r", ",", "s"}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["The Function", "Subsubsubsection"],

Cell["\<\
The explicit example above gives us the outline of our procedure:\
\>", "Text"],

Cell[CellGroupData[{

Cell[TextData[{
 "Initialize a list, ",
 StyleBox["varList", "Input"],
 ", to the list with the given proposition as the sole element. We did not do \
this in the example, but doing so means that we will always be working with a \
list, rather than having the first step be different. "
}], "Item1Numbered"],

Cell[TextData[{
 "We also initialize an index variable, ",
 StyleBox["i", "Input"],
 ", to 1. This will keep track of where we are in the list, taking the place \
of the explicit value 5, for example, in the third to last line above."
}], "Item1Numbered"],

Cell[TextData[{
 "Use ",
 StyleBox[ButtonBox["Head",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Head"], "FunctionLink"],
 " to test whether the element in position ",
 StyleBox["i", "Input"],
 " in the list is a ",
 StyleBox["Symbol", "Input"],
 ". "
}], "Item1Numbered"],

Cell[CellGroupData[{

Cell[TextData[{
 "If it is, then it is the name of a variable, and we move on to the next \
position in the list by increasing ",
 StyleBox["i", "Input"],
 " by 1",
 ". "
}], "Item2"],

Cell[TextData[{
 "If ",
 StyleBox["varList[[i]]", "Input"],
 " is not a symbol, then it must be an expression. So replace its head with \
",
 StyleBox["List", "Input"],
 " and flatten ",
 StyleBox["varList", "Input"],
 ", using the same syntax as above."
}], "Item2"]
}, Open  ]],

Cell[TextData[{
 "Repeat step 3 until the end of the list. This repetition is controlled by a \
",
 StyleBox[ButtonBox["While",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/While"], "FunctionLink"],
 " loop which continues as long as ",
 StyleBox["i", "Input"],
 " is not greater than the number of elements in the list, determined by ",
 StyleBox[ButtonBox["Length",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Length"], "FunctionLink"],
 ". Once the loop is complete, remove duplicate entries."
}], "Item1Numbered"]
}, Open  ]],

Cell["Here is the implementation.", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"getVars", "[", "p_", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"L", "=", 
       RowBox[{"{", "p", "}"}]}], ",", 
      RowBox[{"i", "=", "1"}]}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"While", "[", 
      RowBox[{
       RowBox[{"i", "<=", 
        RowBox[{"Length", "[", "L", "]"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"Head", "[", 
           RowBox[{"L", "[", 
            RowBox[{"[", "i", "]"}], "]"}], "]"}], "===", "Symbol"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{"L", "[", 
            RowBox[{"[", 
             RowBox[{"i", ",", "0"}], "]"}], "]"}], "=", "List"}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"L", "=", 
           RowBox[{"Flatten", "[", "L", "]"}]}]}]}], "\[IndentingNewLine]", 
        "]"}]}], "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"DeleteDuplicates", "[", "L", "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "The use of ",
 StyleBox[ButtonBox["Module",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Module"], "FunctionLink"],
 " requires explanation. The purpose of ",
 StyleBox[ButtonBox["Module",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Module"], "FunctionLink"],
 " is to encapsulate the variables used within a function so that they do not \
change the values of variables used outside of the function. For example, if \
you set ",
 StyleBox["L", "Input"],
 " equal to some value before executing ",
 StyleBox["getVars", "Input"],
 ", it will still have that value afterwards. Likewise, ",
 StyleBox[ButtonBox["Module",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Module"], "FunctionLink"],
 " prevents values set outside the function from affecting the behavior of \
the function. That is, ",
 StyleBox[ButtonBox["Module",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Module"], "FunctionLink"],
 " ensures that the specified variables are treated as local to the module, \
or that they have a local scope, as distinguished from global."
}], "Text"],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["Module",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Module"], "FunctionLink"],
 " function takes two arguments. The first is the list of variables to be \
held local. Within the list of variables, you can either provide just the \
name of the variable, or, if you wish, you can assign the initial value of \
the variable, as was done in ",
 StyleBox["getVars", "Input"],
 ". The expression ",
 Cell[BoxData[
  RowBox[{"{", "L", "=", 
   RowBox[{"{", "p", "}"}], ",", "i", "=", "1", "}"}]], "myCode"],
 " as the first argument to ",
 StyleBox[ButtonBox["Module",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Module"], "FunctionLink"],
 " means that the symbols ",
 StyleBox["L", "Input"],
 " and ",
 StyleBox["i", "Input"],
 " are local and that they are initially assigned values ",
 StyleBox["{p}", "Input"],
 " and ",
 StyleBox["1", "Input"],
 ", respectively",
 ". "
}], "Text"],

Cell[TextData[{
 "The second argument to ",
 StyleBox[ButtonBox["Module",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Module"], "FunctionLink"],
 " is the body of the function definition. Note that semicolons are used to \
separate commands when there is more than one within the body",
 ". ",
 "For example, in the third to last line of ",
 StyleBox["getVars", "Input"],
 ", the semicolon separates the conclusion of the ",
 StyleBox[ButtonBox["While",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/While"], "FunctionLink"],
 " loop from the application of ",
 StyleBox["DeleteDuplicates", "Input"],
 "."
}], "Text"],

Cell[TextData[{
 "Finally, observe that the function ",
 StyleBox["getVars", "Input"],
 " works as expected."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"getVars", "[", "variableEx", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"p", ",", "q", ",", "r", ",", "s"}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"getVars", "[", 
  RowBox[{"Implies", "[", 
   RowBox[{
    RowBox[{"!", "w"}], ",", 
    RowBox[{"Equivalent", "[", 
     RowBox[{
      RowBox[{"Q", "||", "q"}], ",", 
      RowBox[{"P", "&&", "p"}]}], "]"}]}], "]"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"w", ",", "P", ",", "p", ",", "Q", ",", "q"}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Truth Value Assignments", "Subsubsection"],

Cell["\<\
The second hurdle that we mentioned at the beginning of this section is that \
we don't know the number of propositional variables in advance. If we knew \
there would always be two variables, we would use two nested for loops. But \
since we want our procedure to work with any number of variables, we need a \
different approach.\
\>", "Text"],

Cell[TextData[{
 "Since our ",
 StyleBox["getVars", "Input"],
 " function produces a list of variables, it is natural to model an \
assignment of truth values to variables as a list of truth values. For \
example,"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"variableExVars", "=", 
  RowBox[{"getVars", "[", "variableEx", "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"p", ",", "q", ",", "r", ",", "s"}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"truthValEx", "=", 
  RowBox[{"{", 
   RowBox[{"True", ",", "True", ",", "False", ",", "True"}], "}"}]}]], "Input",\

 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"True", ",", "True", ",", "False", ",", "True"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "We consider the ",
 StyleBox["truthValEx", "Input"],
 " (for truth values example) to indicate that we assign the first variable \
of ",
 StyleBox["variableExVars", "Input"],
 " to the value true, the second variable to true, the third to false, and \
the fourth to true."
}], "Text"],

Cell[CellGroupData[{

Cell["Evaluating an expression ", "Subsubsubsection"],

Cell[TextData[{
 "Recall the use of the ",
 StyleBox[ButtonBox["ReplaceAll",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ReplaceAll"], "FunctionLink"],
 " operator (",
 StyleBox["/.", "Input"],
 ") to evaluate an expression. In particular, this operator requires that the \
second operand is a list of rules of the form ",
 Cell[BoxData[
  FormBox[
   RowBox[{"s", "\[Rule]", "v"}], TraditionalForm]]],
 " with ",
 Cell[BoxData[
  FormBox["s", TraditionalForm]]],
 " a symbol and ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]]],
 " a value. So, for example, the following evaluates ",
 StyleBox["variableEx", "Input"],
 " at the values ",
 Cell[BoxData[
  FormBox[
   RowBox[{"p", "=", "true"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"q", "=", "true"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"r", "=", "false"}], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"s", "=", "true"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"variableEx", "/.", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"p", "\[Rule]", "True"}], ",", 
    RowBox[{"q", "\[Rule]", "True"}], ",", 
    RowBox[{"r", "\[Rule]", "False"}], ",", 
    RowBox[{"s", "\[Rule]", "True"}]}], "}"}]}]], "Input"],

Cell[BoxData["False"], "Output"]
}, Open  ]],

Cell[TextData[{
 "In order to perform that evaluation programmatically, using the result of \
",
 StyleBox["getVars", "Input"],
 " and a list representing an assignment of truth values, we need to turn the \
pair of lists into a list of rules. We will demonstrate how to do this with \
the ",
 StyleBox["variableExVars", "Input"],
 " and ",
 StyleBox["truthValEx", "Input"],
 " lists defined above."
}], "Text"],

Cell[TextData[{
 "We introduced the ",
 StyleBox[ButtonBox["MapThread",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/MapThread"], "FunctionLink"],
 " function at the end of Section 1.1 of this manual. Recall that the basic \
purpose of ",
 StyleBox[ButtonBox["MapThread",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/MapThread"], "FunctionLink"],
 " is to take a function of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " variables together with a list of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " lists (with the sublists having the same size) and apply the function to \
corresponding elements of the lists. For example, we can use ",
 StyleBox[ButtonBox["MapThread",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/MapThread"], "FunctionLink"],
 " to add corresponding elements of two lists using the ",
 StyleBox[ButtonBox["Plus",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Plus"], "FunctionLink"],
 " function. (Note that this is generally unnecessary since addition \
automatically threads in ",
 StyleBox["Mathematica,",
  FontSlant->"Italic"],
 " but it serves as an example.)"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MapThread", "[", 
  RowBox[{"Plus", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"1", ",", "2", ",", "3"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"a", ",", "b", ",", "c"}], "}"}]}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"1", "+", "a"}], ",", 
   RowBox[{"2", "+", "b"}], ",", 
   RowBox[{"3", "+", "c"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "In our context, the two lists are the lists of variables, ",
 StyleBox["variableExVars", "Input"],
 ", and the truth value assignment, ",
 StyleBox["truthValEx", "Input"],
 ". ",
 "The function that forms a rule is ",
 StyleBox[ButtonBox["Rule",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Rule"], "FunctionLink"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MapThread", "[", 
  RowBox[{"Rule", ",", 
   RowBox[{"{", 
    RowBox[{"variableExVars", ",", "truthValEx"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"p", "\[Rule]", "True"}], ",", 
   RowBox[{"q", "\[Rule]", "True"}], ",", 
   RowBox[{"r", "\[Rule]", "False"}], ",", 
   RowBox[{"s", "\[Rule]", "True"}]}], "}"}]], "Output"]
}, Open  ]],

Cell["So we can evaluate the expression with the following.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"variableEx", "/.", 
  RowBox[{"MapThread", "[", 
   RowBox[{"Rule", ",", 
    RowBox[{"{", 
     RowBox[{"variableExVars", ",", "truthValEx"}], "}"}]}], "]"}]}]], "Input"],

Cell[BoxData["False"], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Finding All Possible Truth Assignments", "Subsubsubsection"],

Cell["\<\
Now that we know that we can effectively use lists of truth values to \
represent truth value assignments, we need a way to produce all such lists. \
We\[CloseCurlyQuote]ll use a strategy similar to binary counting. Start with \
the list of all falses. Get the next list by changing the first element to \
true. For the next assignment, change the first element back to false and the \
second element to true. Then change the first element to true. Then change \
the first true to false, the second true to false, and the third element \
becomes true. Continue in this pattern: given a list of truth values, obtain \
the next list by changing the left-most false to true and changing all trues \
up until that first false into false. (It is left to the reader to verify \
that this produces all possible truth value assignments.)\
\>", "Text"],

Cell[TextData[{
 "We implement this idea in the ",
 StyleBox["nextTA", "Input"],
 " function (for next truth assignment). The ",
 StyleBox["nextTA", "Input"],
 " function will accept a list of truth values as input and return the \
\[OpenCurlyDoubleQuote]next\[CloseCurlyDoubleQuote] list. The main work of \
this procedure is done inside of a ",
 StyleBox[ButtonBox["For",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/For"], "FunctionLink"],
 " loop. The loop considers each position in the list of truth values in \
turn. If the value in the current position is true, then it is changed to \
false. On the other hand, if the value is false, then it is changed to true \
and the function is terminated by returning the list of truth values. If the ",
 StyleBox[ButtonBox["For",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/For"], "FunctionLink"],
 " loop ends without having returned a new list, then the input to the \
procedure was all trues, which is the last possible truth assignment, and the \
function returns ",
 StyleBox[ButtonBox["Null",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Null"], "FunctionLink"],
 " to indicate that there is no next truth assignment."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"nextTA", "[", "A_", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"i", ",", 
      RowBox[{"newTA", "=", "A"}]}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"Catch", "[", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"For", "[", 
       RowBox[{
        RowBox[{"i", "=", "1"}], ",", 
        RowBox[{"i", "<=", 
         RowBox[{"Length", "[", "A", "]"}]}], ",", 
        RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"newTA", "[", 
           RowBox[{"[", "i", "]"}], "]"}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"newTA", "[", 
            RowBox[{"[", "i", "]"}], "]"}], "=", "False"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{
            RowBox[{"newTA", "[", 
             RowBox[{"[", "i", "]"}], "]"}], "=", "True"}], ";", 
           RowBox[{"Throw", "[", "newTA", "]"}]}]}], "\[IndentingNewLine]", 
         "]"}]}], "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"Throw", "[", "Null", "]"}]}], "\[IndentingNewLine]", "]"}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Once again we use a ",
 StyleBox[ButtonBox["Module",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Module"], "FunctionLink"],
 " structure. This ensures that ",
 StyleBox["i", "Input"],
 ", the loop variable, and ",
 StyleBox["newTA", "Input"],
 ", the truth assignment that is being constructed, are private to the \
function. Note that ",
 StyleBox["newTA", "Input"],
 " is initialized to be a copy of ",
 StyleBox["A", "Input"],
 ", the input list.",
 " ",
 "We will describe ",
 StyleBox[ButtonBox["Catch",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Catch"], "FunctionLink"],
 " momentarily."
}], "Text"],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["For",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/For"], "FunctionLink"],
 " function is ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "\[CloseCurlyQuote]s implementation of a for loop. The first argument \
contains the initialization command, in this case setting the loop variable \
",
 StyleBox["i", "Input"],
 " equal to 1. The second argument defines the test that specifies the \
termination conditions of the loop. In ",
 StyleBox["nextTA", "Input"],
 ", the loop is to run through all of the entries in the list representing \
the truth assignment, so the test is that the value of the index ",
 StyleBox["i", "Input"],
 " has not exceeded the number of entries in the list, determined by the ",
 StyleBox[ButtonBox["Length",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Length"], "FunctionLink"],
 " function. The third argument to ",
 StyleBox[ButtonBox["For",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/For"], "FunctionLink"],
 " is the increment specification. In this case, we\[CloseCurlyQuote]ve used \
the ",
 StyleBox[ButtonBox["Increment",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Increment"], "FunctionLink"],
 " (",
 StyleBox["++", "Input"],
 ")",
 StyleBox[" ", "Input"],
 "operator, which increases the value of ",
 StyleBox["i", "Input"],
 " by 1. It has the same effect as ",
 Cell[BoxData[
  RowBox[{"i", "=", "i", "+", "1"}]], "myCode"],
 ". "
}], "Text"],

Cell[TextData[{
 "The final argument to ",
 StyleBox[ButtonBox["For",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/For"], "FunctionLink"],
 " is the body of the loop. The basic strategy is to work our way through the \
\[OpenCurlyDoubleQuote]old\[CloseCurlyDoubleQuote] truth value assignment \
turning trues into falses until we hit a false. That first false is changed \
to true and we stop. The body of our for loop is dominated by an ",
 StyleBox[ButtonBox["If",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/If"], "FunctionLink"],
 " statement. The first argument of the ",
 StyleBox[ButtonBox["If",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/If"], "FunctionLink"],
 " statement accesses the value in the current position of ",
 StyleBox["newTA", "Input"],
 ". In case that value is true, according to our strategy, we change it to \
false and move on to the next element in the list. If the current value is \
false, we change it to true. "
}], "Text"],

Cell[TextData[{
 "Once a false has been changed to true, we want to stop the evaluation of \
the function and have the current value of ",
 StyleBox["newTA", "Input"],
 " returned as the output of the function. This is the purpose of ",
 StyleBox[ButtonBox["Catch",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Catch"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["Throw",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Throw"], "FunctionLink"],
 ". The ",
 StyleBox[ButtonBox["Throw",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Throw"], "FunctionLink"],
 " function is a way for you to tell ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", \[OpenCurlyDoubleQuote]This (the argument) is the result of this section \
of code.\[CloseCurlyDoubleQuote] ",
 StyleBox[ButtonBox["Catch",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Catch"], "FunctionLink"],
 " defines the scope of the ",
 StyleBox[ButtonBox["Throw",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Throw"], "FunctionLink"],
 ", that is, the argument of the ",
 StyleBox[ButtonBox["Catch",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Catch"], "FunctionLink"],
 " is the block of code to which ",
 StyleBox[ButtonBox["Throw",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Throw"], "FunctionLink"],
 " refers. In other words, when ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " encounters a ",
 StyleBox[ButtonBox["Throw",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Throw"], "FunctionLink"],
 ", it evaluates its argument and considers that result to be the result of \
the entire ",
 StyleBox[ButtonBox["Catch",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Catch"], "FunctionLink"],
 " block. In this case, when the loop encounters a false entry in ",
 StyleBox["newTA", "Input"],
 ", it changes that entry to true and then executes the ",
 StyleBox[ButtonBox["Throw",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Throw"], "FunctionLink"],
 ", which has the effect of ending any further evaluation and declaring the \
result to be the current value of ",
 StyleBox["newTA", "Input"],
 ".",
 " ",
 "Should all of the entries be true initially, then the ",
 Cell[BoxData[
  RowBox[{"Throw", "[", "newTA", "]"}]], "myCode"],
 " will never be encountered and the loop will be allowed to complete. Once \
the loop is complete, the ",
 Cell[BoxData[
  RowBox[{"Throw", "[", "Null", "]"}]], "myCode"],
 " statement will be encountered, causing the ",
 StyleBox[ButtonBox["Catch",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Catch"], "FunctionLink"],
 ", and thus the module, to return ",
 StyleBox[ButtonBox["Null",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Null"], "FunctionLink"],
 "."
}], "Text"],

Cell[TextData[{
 "You may be wondering why we did not use a ",
 StyleBox[ButtonBox["Return",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Return"], "FunctionLink"],
 " statement in the above. While ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " does have a ",
 StyleBox[ButtonBox["Return",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Return"], "FunctionLink"],
 " function, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "\[CloseCurlyQuote]s programming language has functional style, as opposed \
to procedural. Because of this, the behavior of ",
 StyleBox[ButtonBox["Return",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Return"], "FunctionLink"],
 " can be unexpected. In fact, it is impossible for ",
 StyleBox[ButtonBox["Return",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Return"], "FunctionLink"],
 " to have the same behaviour in a functional language such as ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " as it would in a procedural language like C. More than this, ",
 StyleBox[ButtonBox["Return",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Return"], "FunctionLink"],
 " in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " is a bit of a square peg in a round hole situation \[Dash] it does not fit \
with the conceptual framework of a functional language."
}], "Text"],

Cell[TextData[{
 "We can confirm, in the case of three variables, that ",
 StyleBox["nextTA", "Input"],
 " does in fact produce all of the possible truth value assignments using the \
following simple ",
 StyleBox[ButtonBox["While",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/While"], "FunctionLink"],
 " loop. Note that ",
 StyleBox[ButtonBox["While",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/While"], "FunctionLink"],
 " executes the second argument so long as the first argument is true. Also \
note the use of ",
 StyleBox["=!=", "Input"],
 " in the test. This is the ",
 StyleBox[ButtonBox["UnsameQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/UnsameQ"], "FunctionLink"],
 " (",
 StyleBox["=!=", "Input"],
 ") relation, which is the negation of ",
 StyleBox[ButtonBox["SameQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/SameQ"], "FunctionLink"],
 " (",
 StyleBox["===", "Input"],
 "), which we discussed earlier."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"nextTAdemo", "=", 
   RowBox[{"{", 
    RowBox[{"False", ",", "False", ",", "False"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"While", "[", 
  RowBox[{
   RowBox[{"nextTAdemo", "=!=", "Null"}], ",", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"Print", "[", "nextTAdemo", "]"}], ";", "\[IndentingNewLine]", 
    RowBox[{"nextTAdemo", "=", 
     RowBox[{"nextTA", "[", "nextTAdemo", "]"}]}]}]}], "\[IndentingNewLine]", 
  "]"}]}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"False", ",", "False", ",", "False"}], "}"}]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"True", ",", "False", ",", "False"}], "}"}]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"False", ",", "True", ",", "False"}], "}"}]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"True", ",", "True", ",", "False"}], "}"}]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"False", ",", "False", ",", "True"}], "}"}]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"True", ",", "False", ",", "True"}], "}"}]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"False", ",", "True", ",", "True"}], "}"}]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"True", ",", "True", ",", "True"}], "}"}]], "Print"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Logical Equivalence Implementation", "Subsubsection"],

Cell[TextData[{
 "We now have the necessary pieces in place to write the promised ",
 StyleBox["myEquivalentQ", "Input"],
 " function",
 ". ",
 "This function accepts two propositions as arguments and returns true if \
they are equivalent and false otherwise."
}], "Text"],

Cell["The function proceeds as follows:", "Text"],

Cell[CellGroupData[{

Cell[TextData[{
 "First we create the biconditional, which we name ",
 StyleBox["bicond", "Input"],
 ", that asserts the equivalence of the\ntwo propositions. We use the ",
 StyleBox["getVars", "Input"],
 " function to determine the list of variables used in the propositions and \
we initialize the truth assignment variable ",
 StyleBox["TA", "Input"],
 " to the appropriately sized list of all false values using the ",
 StyleBox[ButtonBox["ConstantArray",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ConstantArray"], "FunctionLink"],
 " function applied to the value ",
 StyleBox[ButtonBox["False",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/False"], "FunctionLink"],
 " and the desired length of the list."
}], "Item1Numbered"],

Cell[TextData[{
 "Then we begin a ",
 StyleBox[ButtonBox["While",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/While"], "FunctionLink"],
 " loop. As long as ",
 StyleBox["TA", "Input"],
 " is not ",
 StyleBox[ButtonBox["Null",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Null"], "FunctionLink"],
 ", we evaluate the biconditional ",
 StyleBox["bicond", "Input"],
 " on the truth assignment. If this truth value is false, we know that the \
biconditional is not a tautology and thus the propositions are not equivalent \
and we immediately throw false. Otherwise, we use ",
 StyleBox["nextTA", "Input"],
 " to update ",
 StyleBox["TA", "Input"],
 " to the next truth assignment."
}], "Item1Numbered"],

Cell[TextData[{
 "If the ",
 StyleBox[ButtonBox["While",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/While"], "FunctionLink"],
 " loop terminates, that indicates that all possible truth assignments have \
been applied to the biconditional and that each one evaluated true, otherwise \
the procedure would have returned false and terminated. Thus the \
biconditional is a tautology and true is returned."
}], "Item1Numbered"]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"myEquivalentQ", "[", 
   RowBox[{"p_", ",", "q_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "bicond", ",", "vars", ",", "numVars", ",", "i", ",", "TA", ",", "val"}],
      "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"bicond", "=", 
      RowBox[{"Equivalent", "[", 
       RowBox[{"p", ",", "q"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"vars", "=", 
      RowBox[{"getVars", "[", "bicond", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"numVars", "=", 
      RowBox[{"Length", "[", "vars", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"TA", "=", 
      RowBox[{"ConstantArray", "[", 
       RowBox[{"False", ",", "numVars"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"Catch", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"While", "[", 
        RowBox[{
         RowBox[{"TA", "=!=", "Null"}], ",", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"val", "=", 
           RowBox[{"bicond", "/.", 
            RowBox[{"MapThread", "[", 
             RowBox[{"Rule", ",", 
              RowBox[{"{", 
               RowBox[{"vars", ",", "TA"}], "}"}]}], "]"}]}]}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{"!", "val"}], ",", 
            RowBox[{"Throw", "[", "False", "]"}]}], "]"}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"TA", "=", 
           RowBox[{"nextTA", "[", "TA", "]"}]}]}]}], "\[IndentingNewLine]", 
        "]"}], ";", "\[IndentingNewLine]", 
       RowBox[{"Throw", "[", "True", "]"}]}], "\[IndentingNewLine]", 
      "]"}]}]}], "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "We can use our function to computationally verify that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Not]", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"p", "\[Or]", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"\[Not]", "p"}], "\[And]", "q"}], ")"}]}], ")"}], 
      "\[Congruent]", 
      RowBox[{"\[Not]", "p"}]}]}], "\[And]", 
    RowBox[{"\[Not]", "q"}]}], TraditionalForm]]],
 ". ",
 "This was shown in Example 7 of Section 1.3 of the text via equivalences."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"myEquivalentQ", "[", 
  RowBox[{
   RowBox[{"!", 
    RowBox[{"(", 
     RowBox[{"p", "||", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"!", "p"}], "&&", "q"}], ")"}]}], ")"}]}], ",", 
   RowBox[{
    RowBox[{"!", "p"}], "&&", 
    RowBox[{"!", "q"}]}]}], "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["1.4 Predicates and Quantifiers", "Section",
 CellTags->"Section4"],

Cell[TextData[{
 "In this section we will see how ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can be used to explore propositional functions and their quantification",
 ". ",
 "We can think about a propositional function ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 " as a function that takes as input a member of the domain and that outputs \
a truth value."
}], "Text"],

Cell[TextData[{
 "For example, let ",
 Cell[BoxData[
  FormBox[
   RowBox[{"P", "(", "x", ")"}], TraditionalForm]]],
 " denote the statement \[OpenCurlyDoubleQuote]",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", ">", "0"}], TraditionalForm]]],
 "\[CloseCurlyDoubleQuote]. We can create a ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function, say ",
 StyleBox["gt0", "Input"],
 " (for greater than 0), that takes ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " as input and returns true or false as appropriate",
 ". ",
 "All we have to do is assign the inequality as the body of the function."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"gt0", "[", "x_", "]"}], ":=", 
  RowBox[{"x", ">", "0"}]}]], "Input",
 InitializationCell->True],

Cell["\<\
Evaluating the propositional function at different values demonstrates that \
the result is a truth value.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"gt0", "[", "5", "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"gt0", "[", 
  RowBox[{"-", "3"}], "]"}]], "Input"],

Cell[BoxData["False"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Representation of Quantifiers", "Subsection",
 PageBreakWithin->False,
 CellTags->"Section4a"],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " represents quantification using the functions ",
 StyleBox[ButtonBox["ForAll",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ForAll"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["Exists",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Exists"], "FunctionLink"],
 ". These functions have the same syntax. In their most basic form, they take \
two arguments. The first argument is the variable being bound by the \
quantifier, and the second is the expression being quantified",
 ". ",
 "For example, to represent the statement ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[ForAll]", "x"], 
    RowBox[{"P", "(", "x", ")"}]}], TraditionalForm]]],
 ", you would enter the following."
}], "Text",
 PageBreakAbove->False,
 PageBreakWithin->False],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ForAll", "[", 
  RowBox[{"x", ",", 
   RowBox[{"P", "[", "x", "]"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["\[ForAll]", "x"], 
  RowBox[{"P", "[", "x", "]"}]}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Likewise, we can represent the assertion that there exists an ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " for which the opposite is negative as follows."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Exists", "[", 
  RowBox[{"x", ",", 
   RowBox[{
    RowBox[{"-", "x"}], "<", "0"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["\[Exists]", "x"], 
  RowBox[{
   RowBox[{"-", "x"}], "<", "0"}]}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["ForAll",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ForAll"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["Exists",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Exists"], "FunctionLink"],
 " commands also allow you to express conditions on the bound variable by use \
of an optional second argument",
 ". ",
 "For example, to symbolically express the assertion \
\[OpenCurlyDoubleQuote]For all ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", ">", "0"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"-", "x"}], "<", "0"}], TraditionalForm]]],
 "\[CloseCurlyDoubleQuote] you include the condition ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", ">", "0"}], TraditionalForm]]],
 " as the second argument and the expression ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"-", "x"}], "<", "0"}], TraditionalForm]]],
 " as the third argument."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ForAll", "[", 
  RowBox[{"x", ",", 
   RowBox[{"x", ">", "0"}], ",", 
   RowBox[{
    RowBox[{"-", "x"}], "<", "0"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["\[ForAll]", 
   RowBox[{"x", ",", 
    RowBox[{"x", ">", "0"}]}]], 
  RowBox[{
   RowBox[{"-", "x"}], "<", "0"}]}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "You can, in particular, use the condition to specify the domain, or \
universe of discourse, by asserting that the variable belongs to one of ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "\[CloseCurlyQuote]s recognized domains using the ",
 StyleBox[ButtonBox["Element",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Element"], "FunctionLink"],
 " function. To assert, for example, that ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is a real number, use the ",
 StyleBox[ButtonBox["Element",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Element"], "FunctionLink"],
 " function with first argument ",
 StyleBox["x", "Input"],
 " and second argument ",
 StyleBox[ButtonBox["Reals",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Reals"], "FunctionLink"],
 ", ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "\[CloseCurlyQuote]s symbol for the domain of real numbers."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Exists", "[", 
  RowBox[{"x", ",", 
   RowBox[{"Element", "[", 
    RowBox[{"x", ",", "Reals"}], "]"}], ",", 
   RowBox[{
    RowBox[{"x", "^", "2"}], "<", "0"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["\[Exists]", 
   RowBox[{"x", ",", 
    RowBox[{"x", "\[Element]", "Reals"}]}]], 
  RowBox[{
   SuperscriptBox["x", "2"], "<", "0"}]}]], "Output"]
}, Open  ]],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " has seven defined domains that you can use: ",
 StyleBox[ButtonBox["Reals",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Reals"], "FunctionLink"],
 ", ",
 StyleBox[ButtonBox["Integers",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Integers"], "FunctionLink"],
 ", ",
 StyleBox[ButtonBox["Complexes",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Complexes"], "FunctionLink"],
 ", ",
 StyleBox[ButtonBox["Algebraics",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Algebraics"], "FunctionLink"],
 ", ",
 StyleBox[ButtonBox["Primes",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Primes"], "FunctionLink"],
 ", ",
 StyleBox[ButtonBox["Rationals",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Rationals"], "FunctionLink"],
 ", ",
 StyleBox[ButtonBox["Booleans",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Booleans"], "FunctionLink"],
 "."
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Truth Value of Quantified Statements", "Subsection",
 CellTags->"Section4b"],

Cell[TextData[{
 "In addition to symbolically representing quantified statements, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can determine whether they are true or false. The ",
 StyleBox[ButtonBox["Resolve",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Resolve"], "FunctionLink"],
 " function, applied to an expression involving quantifiers, will eliminate \
the quantifiers",
 ". ",
 "For expressions like the ones given above, this result will be the truth \
value of the statement."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Resolve", "[", 
  RowBox[{"Exists", "[", 
   RowBox[{"x", ",", 
    RowBox[{
     RowBox[{"-", "x"}], "<", "0"}]}], "]"}], "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Resolve", "[", 
  RowBox[{"ForAll", "[", 
   RowBox[{"x", ",", 
    RowBox[{"x", ">", "0"}], ",", 
    RowBox[{
     RowBox[{"-", "x"}], "<", "0"}]}], "]"}], "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Resolve", "[", 
  RowBox[{"Exists", "[", 
   RowBox[{"x", ",", 
    RowBox[{"Element", "[", 
     RowBox[{"x", ",", "Reals"}], "]"}], ",", 
    RowBox[{
     RowBox[{"x", "^", "2"}], "<", "0"}]}], "]"}], "]"}]], "Input"],

Cell[BoxData["False"], "Output"]
}, Open  ]],

Cell[TextData[{
 "The syntax of the last example can be simplified by using a second argument \
to ",
 StyleBox[ButtonBox["Resolve",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Resolve"], "FunctionLink"],
 ". ",
 "Rather than using the ",
 StyleBox[ButtonBox["Element",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Element"], "FunctionLink"],
 " function within the existential statement, we can obtain the same effect \
by putting the domain ",
 StyleBox[ButtonBox["Reals",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Reals"], "FunctionLink"],
 " as a second argument to ",
 StyleBox[ButtonBox["Resolve",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Resolve"], "FunctionLink"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Resolve", "[", 
  RowBox[{
   RowBox[{"Exists", "[", 
    RowBox[{"x", ",", 
     RowBox[{
      RowBox[{"x", "^", "2"}], "<", "0"}]}], "]"}], ",", "Reals"}], 
  "]"}]], "Input"],

Cell[BoxData["False"], "Output"]
}, Open  ]],

Cell["\<\
Note that we obtain a different result by changing the domain.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Resolve", "[", 
  RowBox[{
   RowBox[{"Exists", "[", 
    RowBox[{"x", ",", 
     RowBox[{
      RowBox[{"x", "^", "2"}], "<", "0"}]}], "]"}], ",", "Complexes"}], 
  "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[TextData[{
 "For existential quantification, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can go beyond just finding the truth value and actually give you witnesses \
for the existence of objects with the desired property. This is done using \
the ",
 StyleBox[ButtonBox["FindInstance",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/FindInstance"], "FunctionLink"],
 " function. For example, the statement ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["\[Exists]", "x"], 
     SuperscriptBox["x", "3"]}], "=", "8"}], TraditionalForm]]],
 " is true. (Note that to enter an equation, we must use the ",
 StyleBox[ButtonBox["Equal",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Equal"], "FunctionLink"],
 " (",
 StyleBox["==", "Input"],
 ") relation so as to avoid confusion with assignment.)"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Resolve", "[", 
  RowBox[{"Exists", "[", 
   RowBox[{"x", ",", 
    RowBox[{
     RowBox[{"x", "^", "3"}], "\[Equal]", "8"}]}], "]"}], "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[TextData[{
 "We can find a witness for this by applying ",
 StyleBox[ButtonBox["FindInstance",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/FindInstance"], "FunctionLink"],
 " with the expression ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", "3"], "=", "8"}], TraditionalForm]]],
 " as the first argument and the variable as the second variable",
 ". "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FindInstance", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"x", "^", "3"}], "\[Equal]", "8"}], ",", "x"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"{", 
   RowBox[{"x", "\[Rule]", "2"}], "}"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 StyleBox[ButtonBox["FindInstance",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/FindInstance"], "FunctionLink"],
 " accepts two optional arguments",
 ". ",
 "You can ask for more than one witness just by giving the number of \
instances you would like to find as an argument."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FindInstance", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"x", "^", "3"}], "\[Equal]", "8"}], ",", "x", ",", "3"}], 
  "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"x", "\[Rule]", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", "\[Rule]", 
     RowBox[{
      RowBox[{"-", "1"}], "-", 
      RowBox[{"\[ImaginaryI]", " ", 
       SqrtBox["3"]}]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", "\[Rule]", 
     RowBox[{
      RowBox[{"-", "1"}], "+", 
      RowBox[{"\[ImaginaryI]", " ", 
       SqrtBox["3"]}]}]}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "You can also restrict the results to a particular domain by giving the \
domain as an argument. Note that when giving both a domain and a specific \
number of results, the domain should be the third argument and the number the \
fourth",
 ". ",
 "Below, we have asked for more instances than exist, so ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " just returns the one witness."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FindInstance", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"x", "^", "3"}], "\[Equal]", "8"}], ",", "x", ",", "Integers", 
   ",", "3"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"{", 
   RowBox[{"x", "\[Rule]", "2"}], "}"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "If a statement has one or more free variables, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can be used to find conditions on those variables in order to make a \
statement true. For example, consider the statement ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["\[ForAll]", "x"], 
     RowBox[{"x", "\[CenterDot]", "y"}]}], "=", "0"}], TraditionalForm]]],
 ". In this statement, ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is bound and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " is free. The ",
 StyleBox[ButtonBox["Reduce",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Reduce"], "FunctionLink"],
 " function can be used to solve for free variables. Apply it with the \
statement as the first argument and the free variable (or list of variables) \
as the second."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Reduce", "[", 
  RowBox[{
   RowBox[{"ForAll", "[", 
    RowBox[{"x", ",", 
     RowBox[{
      RowBox[{"x", "*", "y"}], "\[Equal]", "0"}]}], "]"}], ",", "y"}], 
  "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"y", "\[Equal]", "0"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The result, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"y", "=", "0"}], TraditionalForm]]],
 ", means that if the free variable ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " is replaced by the value 0, then the statement will be true."
}], "Text"],

Cell[TextData[{
 StyleBox[ButtonBox["Reduce",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Reduce"], "FunctionLink"],
 " accepts a domain as an optional third argument."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Reduce", "[", 
  RowBox[{
   RowBox[{"Exists", "[", 
    RowBox[{"x", ",", 
     RowBox[{
      RowBox[{"x", "^", "2"}], "\[Equal]", "y"}]}], "]"}], ",", "y", ",", 
   "Reals"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"y", "\[GreaterEqual]", "0"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "This result means that, restricting all variables to the real numbers, if \
",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " is replaced by any non-negative real number, the existential statement ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["\[Exists]", "x"], 
     SuperscriptBox["x", "2"]}], "=", "y"}], TraditionalForm]]],
 " will be true",
 ". ",
 "Note that if the domain restriction is removed, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " defaults to complex numbers and so there would be no restriction on ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Reduce", "[", 
  RowBox[{
   RowBox[{"Exists", "[", 
    RowBox[{"x", ",", 
     RowBox[{
      RowBox[{"x", "^", "2"}], "\[Equal]", "y"}]}], "]"}], ",", "y"}], 
  "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["1.5 Nested Quantifiers", "Section",
 CellTags->"Section5"],

Cell[TextData[{
 "In this section we will see how ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can be used to represent statements with nested quantifiers."
}], "Text"],

Cell[TextData[{
 "For statements in which all the quantifiers are of the same kind, you only \
need to use a single ",
 StyleBox[ButtonBox["Exists",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Exists"], "FunctionLink"],
 " or ",
 StyleBox[ButtonBox["ForAll",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ForAll"], "FunctionLink"],
 " with the list of variables surrounded by braces as the first argument",
 ". ",
 "For example, to represent the statement ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["\[ForAll]", "x"], 
     RowBox[{
      SubscriptBox["\[ForAll]", "y"], 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"x", "\[CenterDot]", "y"}], "=", "0"}], ")"}]}]}], "\[Rule]", 
    
    RowBox[{"(", 
     RowBox[{"x", "=", 
      RowBox[{
       RowBox[{"0", "\[Or]", "y"}], "=", "0"}]}], ")"}]}], 
   TraditionalForm]]],
 ", we only need one ",
 StyleBox[ButtonBox["ForAll",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ForAll"], "FunctionLink"],
 " function with ",
 StyleBox["{x,y}", "Input"],
 " as the first argument."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ForAll", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"x", ",", "y"}], "}"}], ",", 
   RowBox[{"Implies", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"x", "*", "y"}], "\[Equal]", "0"}], ",", 
     RowBox[{
      RowBox[{"x", "\[Equal]", "0"}], "||", 
      RowBox[{"y", "\[Equal]", "0"}]}]}], "]"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["\[ForAll]", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y"}], "}"}]], 
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     RowBox[{"x", " ", "y"}], "\[Equal]", "0"}], "\[Implies]", 
    RowBox[{
     RowBox[{"x", "\[Equal]", "0"}], "||", 
     RowBox[{"y", "\[Equal]", "0"}]}]}], ")"}]}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Using the ",
 StyleBox[ButtonBox["Resolve",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Resolve"], "FunctionLink"],
 " command, we see that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " recognizes this as true."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Resolve", "[", 
  RowBox[{"ForAll", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"x", ",", "y"}], "}"}], ",", 
    RowBox[{"Implies", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"x", "*", "y"}], "\[Equal]", "0"}], ",", 
      RowBox[{
       RowBox[{"x", "\[Equal]", "0"}], "||", 
       RowBox[{"y", "\[Equal]", "0"}]}]}], "]"}]}], "]"}], "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[TextData[{
 "Note that since we did not specify a domain, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " uses the default domain based on the context. In this case, it uses the \
complex numbers as its domain, since the content of the statement is about \
equations",
 ". ",
 "In general, the default domain is the largest domain that makes sense in \
the context."
}], "Text"],

Cell[TextData[{
 "For statements that involve more than one type of quantifier, we must nest \
the ",
 StyleBox[ButtonBox["Exists",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Exists"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["ForAll",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ForAll"], "FunctionLink"],
 " functions",
 ". ",
 "For example, to represent ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["\[ForAll]", 
      RowBox[{"x", "\[NotEqual]", "0"}]], 
     RowBox[{
      SubscriptBox["\[Exists]", "y"], 
      RowBox[{"x", "\[CenterDot]", "y"}]}]}], "=", "1"}], TraditionalForm]]],
 ", we enter the following."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ForAll", "[", 
  RowBox[{"x", ",", 
   RowBox[{"x", "\[NotEqual]", "0"}], ",", 
   RowBox[{"Exists", "[", 
    RowBox[{"y", ",", 
     RowBox[{
      RowBox[{"x", "*", "y"}], "\[Equal]", "1"}]}], "]"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["\[ForAll]", 
   RowBox[{"x", ",", 
    RowBox[{"x", "\[NotEqual]", "0"}]}]], 
  RowBox[{
   SubscriptBox["\[Exists]", "y"], 
   RowBox[{
    RowBox[{"x", " ", "y"}], "\[Equal]", "1"}]}]}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Again, ",
 StyleBox[ButtonBox["Resolve",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Resolve"], "FunctionLink"],
 " recognizes the truth of this statement."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Resolve", "[", 
  RowBox[{"ForAll", "[", 
   RowBox[{"x", ",", 
    RowBox[{"x", "\[NotEqual]", "0"}], ",", 
    RowBox[{"Exists", "[", 
     RowBox[{"y", ",", 
      RowBox[{
       RowBox[{"x", "*", "y"}], "\[Equal]", "1"}]}], "]"}]}], "]"}], 
  "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell["\<\
But limiting the domain to the integers makes the statement false.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Resolve", "[", 
  RowBox[{
   RowBox[{"ForAll", "[", 
    RowBox[{"x", ",", 
     RowBox[{"x", "\[NotEqual]", "0"}], ",", 
     RowBox[{"Exists", "[", 
      RowBox[{"y", ",", 
       RowBox[{
        RowBox[{"x", "*", "y"}], "\[Equal]", "1"}]}], "]"}]}], "]"}], ",", 
   "Integers"}], "]"}]], "Input"],

Cell[BoxData["False"], "Output"]
}, Open  ]],

Cell["\<\
Also observe that reversing the order of the quantifiers changes the meaning \
of the statement.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Exists", "[", 
  RowBox[{"y", ",", 
   RowBox[{"ForAll", "[", 
    RowBox[{"x", ",", 
     RowBox[{"x", "\[NotEqual]", "0"}], ",", 
     RowBox[{
      RowBox[{"x", "*", "y"}], "\[Equal]", "1"}]}], "]"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["\[Exists]", "y"], 
  RowBox[{
   SubscriptBox["\[ForAll]", 
    RowBox[{"x", ",", 
     RowBox[{"x", "\[NotEqual]", "0"}]}]], 
   RowBox[{
    RowBox[{"y", " ", "x"}], "\[Equal]", "1"}]}]}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Resolve", "[", 
  RowBox[{"Exists", "[", 
   RowBox[{"y", ",", 
    RowBox[{"ForAll", "[", 
     RowBox[{"x", ",", 
      RowBox[{"x", "\[NotEqual]", "0"}], ",", 
      RowBox[{
       RowBox[{"x", "*", "y"}], "\[Equal]", "1"}]}], "]"}]}], "]"}], 
  "]"}]], "Input"],

Cell[BoxData["False"], "Output"]
}, Open  ]],

Cell[TextData[{
 "Finally, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " will automatically apply DeMorgan\[CloseCurlyQuote]s laws for quantifiers \
to a statement that you enter."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"!", 
  RowBox[{"ForAll", "[", 
   RowBox[{"x", ",", 
    RowBox[{"Exists", "[", 
     RowBox[{"y", ",", 
      RowBox[{"ForAll", "[", 
       RowBox[{"z", ",", 
        RowBox[{"P", "[", 
         RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "]"}]}], "]"}]}], 
   "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["\[Exists]", "x"], 
  RowBox[{
   SubscriptBox["\[ForAll]", "y"], 
   RowBox[{
    SubscriptBox["\[Exists]", "z"], 
    RowBox[{"!", 
     RowBox[{"P", "[", 
      RowBox[{"x", ",", "y", ",", "z"}], "]"}]}]}]}]}]], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["1.6 Rules of Inference", "Section",
 CellTags->"Section6"],

Cell[TextData[{
 "In this section, we\[CloseCurlyQuote]ll see how ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can be used to verify the validity of arguments in propositional logic. In \
particular, we\[CloseCurlyQuote]ll write a function that, given a list of \
premises and a possible conclusion, will determine whether or not the \
conclusion necessarily follows from the premises. Recall from Definition 1 in \
the text that an argument is defined to be a sequence of propositions, the \
last of which is called the conclusion and all others are premises. Also \
recall that an argument ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["p", "1"], ",", 
    SubscriptBox["p", "2"], ",", "\[Ellipsis]", ",", 
    SubscriptBox["p", "n"], ",", "q"}], TraditionalForm]]],
 " is said to be valid when ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SubscriptBox["p", "1"], "\[And]", 
      SubscriptBox["p", "2"], "\[And]", "\[CenterEllipsis]", "\[And]", 
      SubscriptBox["p", "n"]}], ")"}], "\[Rule]", "q"}], TraditionalForm]]],
 " is a tautology."
}], "Text"],

Cell[TextData[{
 "We can use the ",
 StyleBox[ButtonBox["TautologyQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TautologyQ"], "FunctionLink"],
 " function described in Section 1.3 of this manual to test whether a \
proposition is a tautology. For example, we can confirm ",
 StyleBox["modus tollens",
  FontSlant->"Italic"],
 ". ",
 "(See Table 1 in Section 1.6 of the text for the tautologies associated to \
the rules of inference.)"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TautologyQ", "[", 
  RowBox[{"Implies", "[", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"!", "q"}], "&&", 
      RowBox[{"Implies", "[", 
       RowBox[{"p", ",", "q"}], "]"}]}], ")"}], ",", 
    RowBox[{"!", "p"}]}], "]"}], "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[TextData[{
 "To determine if an argument is valid, we need to: (1) form the conjunction \
of the premises, (2) form the conditional statement that the premises imply \
the conclusion, and (3) test the resulting proposition with ",
 StyleBox[ButtonBox["TautologyQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TautologyQ"], "FunctionLink"],
 ". The ",
 StyleBox["validQ", "Input"],
 " function below will accept as input an argument, i.e., a list of \
propositions, and return true if the argument is valid."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"validQ", "[", "A_", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"premiseList", ",", "premises", ",", "i"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"premiseList", "=", 
      RowBox[{"A", "[", 
       RowBox[{"[", 
        RowBox[{"1", ";;", 
         RowBox[{"-", "2"}]}], "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"premises", "=", 
      RowBox[{"Apply", "[", 
       RowBox[{"And", ",", "premiseList"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"TautologyQ", "[", 
      RowBox[{"Implies", "[", 
       RowBox[{"premises", ",", 
        RowBox[{"A", "[", 
         RowBox[{"[", 
          RowBox[{"-", "1"}], "]"}], "]"}]}], "]"}], "]"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Two comments on the code above are needed. First, the double-semicolons \
used in the definition of ",
 StyleBox["premiseList", "Input"],
 " is the ",
 StyleBox[ButtonBox["Span",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Span"], "FunctionLink"],
 " (",
 StyleBox[";;", "Input"],
 ") operator. When used to refer to a ",
 StyleBox[ButtonBox["Part",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Part"], "FunctionLink"],
 " (",
 StyleBox["[[\[Ellipsis]]]", "Input"],
 ") of a list, ",
 Cell[BoxData[
  RowBox[{
   StyleBox["i", "CodeVar"], ";;", 
   StyleBox["j", "CodeVar"]}]], "myCode"],
 " indicates the range from index ",
 StyleBox["i", "CodeVar"],
 " to index ",
 StyleBox["j", "CodeVar"],
 ". In this case, the -2 indicates the next to last entry of the list. So ",
 Cell[BoxData[
  RowBox[{
   RowBox[{"A", "[", "["}], "1", ";;", 
   RowBox[{"-", "2"}], 
   RowBox[{"]", "]"}]}]], "myCode"],
 " refers to all of ",
 StyleBox["A", "Input"],
 " except the last entry and is thus the premises of the argument ",
 StyleBox["A", "Input"],
 ". Second, the ",
 StyleBox[ButtonBox["Apply",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Apply"], "FunctionLink"],
 " operator is used to apply the function ",
 StyleBox[ButtonBox["And",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/And"], "FunctionLink"],
 " to the arguments contained in the list ",
 StyleBox["premiseList", "Input"],
 ". This is necessary because, while ",
 StyleBox[ButtonBox["And",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/And"], "FunctionLink"],
 " can accept any number of arguments and form the logical conjunction, it \
won\[CloseCurlyQuote]t do anything with a single list like ",
 StyleBox["premiseList", "Input"],
 ". When ",
 StyleBox[ButtonBox["Apply",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Apply"], "FunctionLink"],
 " is given a function and a list, the result is the same as if the elements \
of the list were given as the arguments to the function. Fundamentally, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " is replacing the head of the list, ",
 StyleBox["List", "Input"],
 ", by the name of the function."
}], "Text"],

Cell["\<\
We can use this function to verify that the argument described in Exercise 12 \
of Section 1.6 of the text is in fact valid.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"validQ", "[", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Implies", "[", 
     RowBox[{
      RowBox[{"p", "&&", "t"}], ",", 
      RowBox[{"r", "||", "s"}]}], "]"}], ",", 
    RowBox[{"Implies", "[", 
     RowBox[{"q", ",", 
      RowBox[{"u", "&&", "t"}]}], "]"}], ",", 
    RowBox[{"Implies", "[", 
     RowBox[{"u", ",", "p"}], "]"}], ",", 
    RowBox[{"!", "s"}], ",", 
    RowBox[{"Implies", "[", 
     RowBox[{"q", ",", "r"}], "]"}]}], "}"}], "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[TextData[{
 "Note that Exercise 12, which this example was based on, asks you to verify \
the validity of the argument using rules of inference. It is important to \
note that our function did not do that. It essentially used truth tables to \
check validity. It would be considerably more difficult to program ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to check validity with rules of inference than it was to do so with truth \
tables. On the other hand, for a human it is typically much easier to use \
rules of inference than a truth table. Especially with practice, you will \
develop an intuition about logical arguments that cannot be easily created in \
a computer."
}], "Text"],

Cell[CellGroupData[{

Cell["Finding Conclusions (optional)", "Subsection",
 CellTags->"Section6a"],

Cell[TextData[{
 "In the remainder of this section we\[CloseCurlyQuote]ll consider a slightly \
different question: given a list of premises, what conclusions can you draw \
using valid arguments? We\[CloseCurlyQuote]ll approach this problem in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " in a straightforward (and na\[IDoubleDot]ve) way: generate possible \
conclusions and use ",
 StyleBox["validQ", "Input"],
 " to determine which are valid conclusions."
}], "Text"],

Cell[CellGroupData[{

Cell["Making Compound Propositions", "Subsubsection"],

Cell[TextData[{
 "To generate possible conclusions, we\[CloseCurlyQuote]ll use the following \
function, ",
 StyleBox["allCompound", "Input"],
 ". This function takes a list of propositions and produces all possible \
propositions formed from one logical connective (from and, or, and implies) \
and two of the given propositions, along with the negations of the \
propositions. To avoid including some trivialities, we\[CloseCurlyQuote]ll \
exclude those propositions that are tautologies or contradictions."
}], "Text"],

Cell[TextData[{
 "The function is provided below. Note the use of ",
 StyleBox[ButtonBox["AppendTo",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/AppendTo"], "FunctionLink"],
 ", which accepts a list and an element to be added to the list as arguments. \
It has the result of adding the given element to the list and updating the \
list without the need of an explicit assignment. Also note that at the end of \
the function we apply ",
 StyleBox[ButtonBox["DeleteDuplicates",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/DeleteDuplicates"], "FunctionLink"],
 " so as to remove repeated elements from the list. Also pay attention to the \
uses of ",
 StyleBox[ButtonBox["Do",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Do"], "FunctionLink"],
 ", which allow us to loop over all the elements (or combinations of \
elements) of lists."
}], "Text"],

Cell["\<\
The bulk of the function is taken up by adding the conjunction, disjunction, \
and implication of the chosen pair to the result list, provided that they do \
not form tautologies or contradictions.\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"allCompound", "[", "vars_", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"p", ",", "q", ",", 
      RowBox[{"tempList", "=", "vars"}], ",", "propList"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{"AppendTo", "[", 
        RowBox[{"tempList", ",", 
         RowBox[{"!", "p"}]}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"p", ",", "vars"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"propList", "=", "tempList"}], ";", "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"!", 
            RowBox[{"TautologyQ", "[", 
             RowBox[{"p", "&&", "q"}], "]"}]}], "&&", 
           RowBox[{"!", 
            RowBox[{"TautologyQ", "[", 
             RowBox[{"!", 
              RowBox[{"(", 
               RowBox[{"p", "&&", "q"}], ")"}]}], "]"}]}]}], ",", 
          RowBox[{"AppendTo", "[", 
           RowBox[{"propList", ",", 
            RowBox[{"p", "&&", "q"}]}], "]"}]}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"!", 
            RowBox[{"TautologyQ", "[", 
             RowBox[{"p", "||", "q"}], "]"}]}], "&&", 
           RowBox[{"!", 
            RowBox[{"TautologyQ", "[", 
             RowBox[{"!", 
              RowBox[{"(", 
               RowBox[{"p", "||", "q"}], ")"}]}], "]"}]}]}], ",", 
          RowBox[{"AppendTo", "[", 
           RowBox[{"propList", ",", 
            RowBox[{"p", "||", "q"}]}], "]"}]}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"!", 
            RowBox[{"TautologyQ", "[", 
             RowBox[{"Implies", "[", 
              RowBox[{"p", ",", "q"}], "]"}], "]"}]}], "&&", 
           RowBox[{"!", 
            RowBox[{"TautologyQ", "[", 
             RowBox[{"!", 
              RowBox[{"Implies", "[", 
               RowBox[{"p", ",", "q"}], "]"}]}], "]"}]}]}], ",", 
          RowBox[{"AppendTo", "[", 
           RowBox[{"propList", ",", 
            RowBox[{"Implies", "[", 
             RowBox[{"p", ",", "q"}], "]"}]}], "]"}]}], "]"}]}], 
       "\[IndentingNewLine]", ",", 
       RowBox[{"{", 
        RowBox[{"p", ",", "tempList"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"q", ",", "tempList"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"DeleteDuplicates", "[", "propList", "]"}]}]}], "]"}]}]], "Input",\

 InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Finding Valid Conclusions", "Subsubsection"],

Cell[TextData[{
 "Now we write a function to explore possible conclusions given a set of \
premises. This function will take two arguments. The first will be a list of \
premises. The second a positive integer indicating the number of times that \
",
 StyleBox["allCompound", "Input"],
 " should, recursively, be used to generate possibilities. You will generally \
not want to use any number other than 1 for this second value as the time \
requirement can be quite substantial."
}], "Text"],

Cell[TextData[{
 "The operation of this function is fairly straightforward. First, it \
determines the variables used in the provided premises by applying the ",
 StyleBox["getVars", "Input"],
 " function we wrote above. Second, it recursively applies ",
 StyleBox["allCompound", "Input"],
 ", beginning with the list of variables, a number of times equal to the \
level, which is the second argument to the function",
 ". ",
 "Finally, for each of the possible conclusions generated by the ",
 StyleBox["allCompound", "Input"],
 " function, it uses ",
 StyleBox["validQ", "Input"],
 " to see if it is a valid conclusion from the premises, and adds those that \
are to the output."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"findConsequences", "[", 
   RowBox[{"premises_", ",", "level_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"vars", ",", "P", ",", "possibleC", ",", 
      RowBox[{"conclusions", "=", 
       RowBox[{"{", "}"}]}], ",", "c", ",", "i"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"vars", "=", 
      RowBox[{"getVars", "[", "premises", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"possibleC", "=", "vars"}], ";", "\[IndentingNewLine]", 
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"i", "=", "1"}], ",", 
       RowBox[{"i", "\[LessEqual]", "level"}], ",", 
       RowBox[{"i", "++"}], ",", 
       RowBox[{"possibleC", "=", 
        RowBox[{"allCompound", "[", "possibleC", "]"}]}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Do", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"validQ", "[", 
          RowBox[{"Append", "[", 
           RowBox[{"premises", ",", "c"}], "]"}], "]"}], ",", 
         RowBox[{"AppendTo", "[", 
          RowBox[{"conclusions", ",", "c"}], "]"}]}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"c", ",", "possibleC"}], "}"}]}], "\[IndentingNewLine]", 
      "]"}], ";", "\[IndentingNewLine]", "conclusions"}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Here is the result of applying ",
 StyleBox["findConsequences", "Input"],
 " to the premises of Exercise 12 with only one iteration of ",
 StyleBox["allCompound", "Input"],
 ". ",
 "(With two iterations, the function takes quite some time to complete and \
produces thousands of valid conclusions.)"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"findConsequences", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Implies", "[", 
      RowBox[{
       RowBox[{"p", "&&", "t"}], ",", 
       RowBox[{"r", "||", "s"}]}], "]"}], ",", 
     RowBox[{"Implies", "[", 
      RowBox[{"q", ",", 
       RowBox[{"u", "&&", "t"}]}], "]"}], ",", 
     RowBox[{"Implies", "[", 
      RowBox[{"u", ",", "p"}], "]"}], ",", 
     RowBox[{"!", "s"}]}], "}"}], ",", "1"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"!", "s"}], ",", 
   RowBox[{"p", "||", 
    RowBox[{"!", "s"}]}], ",", 
   RowBox[{"p", "\[Implies]", 
    RowBox[{"!", "s"}]}], ",", 
   RowBox[{"p", "||", 
    RowBox[{"!", "q"}]}], ",", 
   RowBox[{"p", "||", 
    RowBox[{"!", "u"}]}], ",", 
   RowBox[{"t", "||", 
    RowBox[{"!", "s"}]}], ",", 
   RowBox[{"t", "\[Implies]", 
    RowBox[{"!", "s"}]}], ",", 
   RowBox[{"t", "||", 
    RowBox[{"!", "q"}]}], ",", 
   RowBox[{"r", "||", 
    RowBox[{"!", "s"}]}], ",", 
   RowBox[{"r", "\[Implies]", 
    RowBox[{"!", "s"}]}], ",", 
   RowBox[{"r", "||", 
    RowBox[{"!", "q"}]}], ",", 
   RowBox[{"s", "\[Implies]", "p"}], ",", 
   RowBox[{"s", "\[Implies]", "t"}], ",", 
   RowBox[{"s", "\[Implies]", "r"}], ",", 
   RowBox[{"s", "\[Implies]", "q"}], ",", 
   RowBox[{"s", "\[Implies]", "u"}], ",", 
   RowBox[{"s", "\[Implies]", 
    RowBox[{"!", "p"}]}], ",", 
   RowBox[{"s", "\[Implies]", 
    RowBox[{"!", "t"}]}], ",", 
   RowBox[{"s", "\[Implies]", 
    RowBox[{"!", "r"}]}], ",", 
   RowBox[{"s", "\[Implies]", 
    RowBox[{"!", "s"}]}], ",", 
   RowBox[{"s", "\[Implies]", 
    RowBox[{"!", "q"}]}], ",", 
   RowBox[{"s", "\[Implies]", 
    RowBox[{"!", "u"}]}], ",", 
   RowBox[{"q", "\[Implies]", "p"}], ",", 
   RowBox[{"q", "\[Implies]", "t"}], ",", 
   RowBox[{"q", "\[Implies]", "r"}], ",", 
   RowBox[{"q", "\[Implies]", "u"}], ",", 
   RowBox[{"q", "||", 
    RowBox[{"!", "s"}]}], ",", 
   RowBox[{"q", "\[Implies]", 
    RowBox[{"!", "s"}]}], ",", 
   RowBox[{"u", "\[Implies]", "p"}], ",", 
   RowBox[{"u", "||", 
    RowBox[{"!", "s"}]}], ",", 
   RowBox[{"u", "\[Implies]", 
    RowBox[{"!", "s"}]}], ",", 
   RowBox[{"u", "||", 
    RowBox[{"!", "q"}]}], ",", 
   RowBox[{
    RowBox[{"!", "p"}], "||", 
    RowBox[{"!", "s"}]}], ",", 
   RowBox[{
    RowBox[{"!", "p"}], "\[Implies]", 
    RowBox[{"!", "s"}]}], ",", 
   RowBox[{
    RowBox[{"!", "p"}], "\[Implies]", 
    RowBox[{"!", "q"}]}], ",", 
   RowBox[{
    RowBox[{"!", "p"}], "\[Implies]", 
    RowBox[{"!", "u"}]}], ",", 
   RowBox[{
    RowBox[{"!", "t"}], "||", 
    RowBox[{"!", "s"}]}], ",", 
   RowBox[{
    RowBox[{"!", "t"}], "\[Implies]", 
    RowBox[{"!", "s"}]}], ",", 
   RowBox[{
    RowBox[{"!", "t"}], "\[Implies]", 
    RowBox[{"!", "q"}]}], ",", 
   RowBox[{
    RowBox[{"!", "r"}], "||", 
    RowBox[{"!", "s"}]}], ",", 
   RowBox[{
    RowBox[{"!", "r"}], "\[Implies]", 
    RowBox[{"!", "s"}]}], ",", 
   RowBox[{
    RowBox[{"!", "r"}], "\[Implies]", 
    RowBox[{"!", "q"}]}], ",", 
   RowBox[{
    RowBox[{"!", "s"}], "||", "p"}], ",", 
   RowBox[{
    RowBox[{"!", "s"}], "||", "t"}], ",", 
   RowBox[{
    RowBox[{"!", "s"}], "||", "r"}], ",", 
   RowBox[{
    RowBox[{"!", "s"}], "||", "q"}], ",", 
   RowBox[{
    RowBox[{"!", "s"}], "||", "u"}], ",", 
   RowBox[{
    RowBox[{"!", "s"}], "||", 
    RowBox[{"!", "p"}]}], ",", 
   RowBox[{
    RowBox[{"!", "s"}], "||", 
    RowBox[{"!", "t"}]}], ",", 
   RowBox[{
    RowBox[{"!", "s"}], "||", 
    RowBox[{"!", "r"}]}], ",", 
   RowBox[{
    RowBox[{"!", "s"}], "&&", 
    RowBox[{"!", "s"}]}], ",", 
   RowBox[{
    RowBox[{"!", "s"}], "||", 
    RowBox[{"!", "s"}]}], ",", 
   RowBox[{
    RowBox[{"!", "s"}], "||", 
    RowBox[{"!", "q"}]}], ",", 
   RowBox[{
    RowBox[{"!", "s"}], "||", 
    RowBox[{"!", "u"}]}], ",", 
   RowBox[{
    RowBox[{"!", "q"}], "||", "p"}], ",", 
   RowBox[{
    RowBox[{"!", "q"}], "||", "t"}], ",", 
   RowBox[{
    RowBox[{"!", "q"}], "||", "r"}], ",", 
   RowBox[{
    RowBox[{"!", "q"}], "||", "u"}], ",", 
   RowBox[{
    RowBox[{"!", "q"}], "||", 
    RowBox[{"!", "s"}]}], ",", 
   RowBox[{
    RowBox[{"!", "q"}], "\[Implies]", 
    RowBox[{"!", "s"}]}], ",", 
   RowBox[{
    RowBox[{"!", "u"}], "||", "p"}], ",", 
   RowBox[{
    RowBox[{"!", "u"}], "||", 
    RowBox[{"!", "s"}]}], ",", 
   RowBox[{
    RowBox[{"!", "u"}], "\[Implies]", 
    RowBox[{"!", "s"}]}], ",", 
   RowBox[{
    RowBox[{"!", "u"}], "\[Implies]", 
    RowBox[{"!", "q"}]}]}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Length", "[", "%", "]"}]], "Input"],

Cell[BoxData["64"], "Output"]
}, Open  ]],

Cell["\<\
Observe that some of the conclusions are just merely restating premises. But \
even after eliminating those, there are still 60 valid conclusions involving \
at most two of the propositional variables. Most of those conclusions are \
going to be fairly uninteresting in any particular context. This illustrates \
a fundamental difficulty with computer assisted proof. Neither checking the \
validity of conclusions nor generating valid conclusions from a list of \
premises are particularly difficult. The difficulty is in creating heuristics \
and other mechanisms to help direct the computer to useful results.\
\>", "Text"]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["1.7 Introduction to Proofs", "Section",
 CellTags->"Section7"],

Cell[TextData[{
 "In this section we will see how ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can be used to find counterexamples. This is the proof technique most \
suitable to ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "\[CloseCurlyQuote]s computational abilities."
}], "Text"],

Cell["\<\
Example 14 of Section 1.7 of the textbook considers the statement \
\[OpenCurlyDoubleQuote]Every positive integer is the sum of the squares of \
two integers.\[CloseCurlyDoubleQuote] This is demonstrated to be false with 3 \
as a counterexample. Here, we\[CloseCurlyQuote]ll consider the related \
statement that \[OpenCurlyDoubleQuote]Every positive integer is the sum of \
the squares of three integers.\[CloseCurlyDoubleQuote] This statement is also \
false.\
\>", "Text"],

Cell[CellGroupData[{

Cell["Finding a Counterexample", "Subsection",
 CellTags->"Section7a"],

Cell["\<\
To find a counterexample, we\[CloseCurlyQuote]ll create a function that, \
given an integer, looks for three integers such that the sum of their squares \
are equal to the given integer. If the function finds three such integers, it \
will return a list containing them. On the other hand, if it cannot find \
three such integers, it will return false. Here is the function:\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"find3squares", "[", "n_", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"a", ",", "b", ",", "c", ",", 
      RowBox[{"max", "=", 
       RowBox[{"Floor", "[", 
        RowBox[{"Sqrt", "[", "n", "]"}], "]"}]}]}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{"Catch", "[", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"For", "[", 
       RowBox[{
        RowBox[{"a", "=", "0"}], ",", 
        RowBox[{"a", "\[LessEqual]", "max"}], ",", 
        RowBox[{"a", "++"}], ",", "\[IndentingNewLine]", 
        RowBox[{"For", "[", 
         RowBox[{
          RowBox[{"b", "=", "0"}], ",", 
          RowBox[{"b", "\[LessEqual]", "max"}], ",", 
          RowBox[{"b", "++"}], ",", "\[IndentingNewLine]", 
          RowBox[{"For", "[", 
           RowBox[{
            RowBox[{"c", "=", "0"}], ",", 
            RowBox[{"c", "\[LessEqual]", "max"}], ",", 
            RowBox[{"c", "++"}], ",", "\[IndentingNewLine]", 
            RowBox[{"If", "[", 
             RowBox[{
              RowBox[{"n", "\[Equal]", 
               RowBox[{
                RowBox[{"a", "^", "2"}], "+", 
                RowBox[{"b", "^", "2"}], "+", 
                RowBox[{"c", "^", "2"}]}]}], ",", 
              RowBox[{"Throw", "[", 
               RowBox[{"{", 
                RowBox[{"a", ",", "b", ",", "c"}], "}"}], "]"}]}], "]"}]}], 
           "\[IndentingNewLine]", "]"}]}], "\[IndentingNewLine]", "]"}]}], 
       "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"Throw", "[", "False", "]"}]}], "\[IndentingNewLine]", "]"}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "The ",
 StyleBox["find3squares", "Input"],
 " function is straightforward. We use three ",
 StyleBox[ButtonBox["For",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/For"], "FunctionLink"],
 " loops to check all possible values of ",
 StyleBox["a", "Input"],
 ", ",
 StyleBox["b", "Input"],
 ", and ",
 StyleBox["c", "Input"],
 ". Each loop can range from 0 to the floor of ",
 Cell[BoxData[
  FormBox[
   SqrtBox["n"], TraditionalForm]]],
 " (the floor of a number is the largest integer that is less than or equal \
to the number). Note that these bounds are sufficient to guarantee that if ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " can be written as the sum of the squares of three integers, then this \
procedure will find them. We observe that 3, the counterexample from Example \
14, can be written as the sum of three squares."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"find3squares", "[", "3", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "1", ",", "1"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "To find a counterexample to the claim that ",
 "\[OpenCurlyDoubleQuote]Every positive integer is the sum of the squares of \
three integers,\[CloseCurlyDoubleQuote] we write a function that, starting \
with 1, tests numbers using ",
 StyleBox["find3squares", "Input"],
 " until a value is found that causes it to return false."
}], "Text"],

Cell[BoxData[
 RowBox[{"find3counter", ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"n", "=", "1"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"While", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"find3squares", "[", "n", "]"}], "=!=", "False"}], ",", 
       RowBox[{"n", "++"}]}], "]"}], ";", "\[IndentingNewLine]", "n"}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell["\<\
First note that this \[OpenCurlyDoubleQuote]function\[CloseCurlyDoubleQuote] \
does not take an argument, so we will not use brackets when we execute it.\
\>", "Text"],

Cell[TextData[{
 "Also note that the ",
 StyleBox[ButtonBox["While",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/While"], "FunctionLink"],
 " loop is controlled by the return value of ",
 StyleBox["find3squares", "Input"],
 ". This is a fairly common approach when you are looking for an input value \
that will cause another function to return a desired result. As before, when \
comparing non-numerical objects, we use ",
 StyleBox[ButtonBox["UnsameQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/UnsameQ"], "FunctionLink"],
 " (",
 StyleBox["=!=", "Input"],
 ")."
}], "Text"],

Cell["\<\
To find the counterexample, all we need to do is call the function.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData["find3counter"], "Input"],

Cell[BoxData["7"], "Output"]
}, Open  ]],

Cell["\<\
This indicates that 7 is an integer that is not the sum of the squares of \
three integers.\
\>", "Text"],

Cell[TextData[{
 "Let\[CloseCurlyQuote]s take a step back and review what we did. Our goal \
was to disprove the statement ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[ForAll]", "n"], 
    RowBox[{"P", "(", "n", ")"}]}], TraditionalForm]]],
 " where ",
 Cell[BoxData[
  FormBox[
   RowBox[{"P", "(", "n", ")"}], TraditionalForm]]],
 " is the statement that \[OpenCurlyDoubleQuote]",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " can be written as the sum of the squares of three integers.\
\[CloseCurlyDoubleQuote] We first wrote ",
 StyleBox["find3squares", "Input"],
 ", which is a function whose goal is to find three integers whose squares \
add to its argument. Observe that if ",
 StyleBox["find3squares", "Input"],
 " returns three values for a given ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", then we know ",
 Cell[BoxData[
  FormBox[
   RowBox[{"P", "(", "n", ")"}], TraditionalForm]]],
 " is true for that ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". Only after we wrote the ",
 StyleBox["find3squares", "Input"],
 " function did we write ",
 StyleBox["find3counter", "Input"],
 ", whose task was to find a counterexample to the universal statement. This \
is a common strategy when using a computer to find a counterexample \
\[LongDash] write a program that seeks to verify the ",
 Cell[BoxData[
  FormBox[
   RowBox[{"P", "(", "n", ")"}], TraditionalForm]]],
 " statement for input ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " and then look to find a value of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " that causes the program to fail."
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Proof", "Subsection",
 CellTags->"Section7b"],

Cell["\<\
We have not yet actually disproved the statement that \
\[OpenCurlyDoubleQuote]Every positive integer is the sum of the squares of \
three integers.\[CloseCurlyDoubleQuote] The functions we wrote found a \
candidate for a counterexample, but we don\[CloseCurlyQuote]t yet know for \
sure that it is in fact a counterexample (after all, our program could be \
flawed). To prove the statement is false, we must prove that 7 is in fact a \
counterexample. We can approach this in one of two ways. The first approach \
is to follow the Solution to Example 17 in Section 1.8 of the text.\
\>", "Text"],

Cell[TextData[{
 "The alternative is to prove the correctness of our algorithm. Specifically, \
we need to prove the statement: \[OpenCurlyDoubleQuote]The positive integer \
",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " can be written as the sum of the squares of three integers if and only if ",
 Cell["find3squares[n]", "myCode"],
 " returns a list of three integers.\[CloseCurlyDoubleQuote]",
 " ",
 "Let\[CloseCurlyQuote]s prove this biconditional."
}], "Text"],

Cell[TextData[{
 "First we\[CloseCurlyQuote]ll prove the statement: if the positive integer \
",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " can be written as the sum of the squares of three integers, then ",
 Cell["find3squares[n]", "myCode"],
 " returns a list of three integers. We\[CloseCurlyQuote]ll use a direct \
proof. We assume that ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " can be written as the sum of three squares. Say ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "=", 
    RowBox[{
     SuperscriptBox["a", "2"], "+", 
     SuperscriptBox["b", "2"], "+", 
     SuperscriptBox["c", "2"]}]}], TraditionalForm]]],
 " for integers ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["c", TraditionalForm]]],
 ". Note that we may take ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["c", TraditionalForm]]],
 " to be non-negative integers, since an integer and its negative have the \
same square. Also, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "=", 
    RowBox[{
     RowBox[{
      SuperscriptBox["a", "2"], "+", 
      SuperscriptBox["b", "2"], "+", 
      SuperscriptBox["c", "2"]}], "\[GreaterEqual]", 
     SuperscriptBox["a", "2"]}]}], TraditionalForm]]],
 ". So ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "\[GreaterEqual]", 
    SuperscriptBox["a", "2"]}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", "\[GreaterEqual]", "0"}], TraditionalForm]]],
 ", which means that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", "\[LessEqual]", 
    SqrtBox["n"]}], TraditionalForm]]],
 ". Since ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " is an integer and is less than or equal to the square root of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " must be less than or equal to the floor of ",
 Cell[BoxData[
  FormBox[
   SqrtBox["n"], TraditionalForm]]],
 " since the floor of a real number is the greatest integer less than or \
equal to the real number. The same argument applies to ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["c", TraditionalForm]]],
 ". We started with ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "=", 
    RowBox[{
     SuperscriptBox["a", "2"], "+", 
     SuperscriptBox["b", "2"], "+", 
     SuperscriptBox["c", "2"]}]}], TraditionalForm]]],
 " and have now shown that ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["c", TraditionalForm]]],
 " can be assumed to be non-negative integers and must be less than or equal \
to the floor of the square root of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". The nested for loops in ",
 StyleBox["find3squares", "Input"],
 " set ",
 StyleBox["a", "Input"],
 ", ",
 StyleBox["b", "Input"],
 ", and ",
 StyleBox["c", "Input"],
 " equal to every possible combination of integers between 0 and ",
 StyleBox["max", "Input"],
 ", which is the floor of the square root of ",
 StyleBox["n", "Input"],
 ". Hence, ",
 StyleBox["a", "Input"],
 ", ",
 StyleBox["b", "Input"],
 ", and ",
 StyleBox["c", "Input"],
 " must, at some point during the execution of ",
 StyleBox["find3squares", "Input"],
 ", be set to ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["c", TraditionalForm]]],
 ", and thus the condition that ",
 Cell[BoxData[
  RowBox[{"n", "\[Equal]", 
   RowBox[{
    RowBox[{"a", "^", "2"}], "+", 
    RowBox[{"b", "^", "2"}], "+", 
    RowBox[{"c", "^", "2"}]}]}]], "myCode"],
 " will be satisfied and ",
 Cell[BoxData[
  RowBox[{"{", "a", ",", "b", ",", "c", "}"}]], "myCode"],
 " will be returned by the function. We\[CloseCurlyQuote]ve assumed that ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " can be written as the sum of three squares and concluded that ",
 Cell["find3squares[n]", "myCode"],
 " must return a list of the integers."
}], "Text"],

Cell[TextData[{
 "The converse is: if ",
 Cell["find3squares[n]", "myCode"],
 " ",
 "returns a list of three integers, then ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " can be written as the sum of the squares of three integers. This is nearly \
obvious, since if ",
 Cell["find3squares[n]", "myCode"],
 " returns ",
 Cell[BoxData[
  RowBox[{"{", "a", ",", "b", ",", "c", "}"}]], "myCode"],
 ", it must have been because ",
 Cell[BoxData[
  RowBox[{"n", "\[Equal]", 
   RowBox[{
    RowBox[{"a", "^", "2"}], "+", 
    RowBox[{"b", "^", "2"}], "+", 
    RowBox[{"c", "^", "2"}]}]}]], "myCode"],
 " was found to be true."
}], "Text"],

Cell[TextData[{
 "Therefore, the ",
 StyleBox["find3squares", "Input"],
 " procedure is correct and since ",
 Cell["find3squares[n]", "myCode"],
 " returns false, we can conclude that 7 is, in fact, a counterexample to the \
assertion that every positive integer is the sum of the squares of three \
integers."
}], "Text"],

Cell["\<\
We will typically not be proving the correctness of procedures in this manual \
\[LongDash] that is a topic for another course. The above merely serves to \
illustrate how you can approach such a proof and to reinforce the principle \
that just because a program produces output does not guarantee that the \
program or the output is correct.\
\>", "Text"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["1.8 Proof Methods and Strategy", "Section",
 CellTags->"Section8"],

Cell[TextData[{
 "In this section, we will consider two additional proof techniques that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can assist with: exhaustive proofs and existence proofs."
}], "Text"],

Cell[CellGroupData[{

Cell["Exhaustive Proof", "Subsection",
 CellTags->"Section8a"],

Cell[TextData[{
 "In an exhaustive proof we must check all possibilities. For an exhaustive \
proof to be feasible by hand, there must be a fairly small number of \
possibilities. With computer software such as ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", though, the number of possibilities can be greatly expanded. Consider \
Example 2 from Section 1.8 of the text. There it was determined by hand that \
the only consecutive positive integers not exceeding 100 that are perfect \
powers are 8 and 9."
}], "Text"],

Cell["\<\
We will consider a variation of this problem: prove that the only consecutive \
positive integers not exceeding 100,000,000 that are perfect powers are 8 and \
9.\
\>", "Text"],

Cell[TextData[{
 "Our approach will be the same as was used in the text. We will generate all \
the perfect powers not exceeding the maximum value and then we will check to \
see which of the perfect powers occur as a consecutive pair. We will \
implement this strategy with two procedures. The first function, ",
 StyleBox["findPowers", "Input"],
 ", will accept as an argument the maximum value to consider (e.g., 100) and \
will return all of the perfect powers no greater than that maximum. The \
second function, ",
 StyleBox["findConsecutivePowers", "Input"],
 ", will also accept the maximum value as its input. It will use ",
 StyleBox["findPowers", "Input"],
 " to generate the powers and then check them for consecutive pairs."
}], "Text"],

Cell[TextData[{
 "For the first function, ",
 StyleBox["findPowers", "Input"],
 ", we need to generate all perfect powers up to the given limit. To do this, \
we\[CloseCurlyQuote]ll use a nested pair of loops for the exponent (",
 StyleBox["p", "Input"],
 ") and the base (",
 StyleBox["b", "Input"],
 "). Both of the loops will be ",
 StyleBox[ButtonBox["While",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/While"], "FunctionLink"],
 " loops controlled by a Boolean variable, ",
 StyleBox["continuep", "Input"],
 " and ",
 StyleBox["continueb", "Input"],
 ". In the inner loop, we check to see if ",
 StyleBox["b^p", "Input"],
 " is greater than the limit, ",
 StyleBox["n", "Input"],
 ", given as the input to the function. If it is, then we set ",
 StyleBox["continueb", "Input"],
 " to false, which terminates the inner loop, and if not, we add ",
 StyleBox["b^p", "Input"],
 " to the list of perfect powers, ",
 StyleBox["L", "Input"],
 ", and increment ",
 StyleBox["b", "Input"],
 ". Once the inner ",
 StyleBox["b", "Input"],
 " loop has terminated, we increment the power ",
 StyleBox["p", "Input"],
 ". If ",
 StyleBox["2^p", "Input"],
 " exceeds the limit, then we know that no more exponents need to be checked \
and we terminate the outer loop by setting ",
 StyleBox["continuep", "Input"],
 " to false."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"findPowers", "[", "n_", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"L", "=", 
       RowBox[{"{", "}"}]}], ",", "b", ",", 
      RowBox[{"p", "=", "2"}], ",", 
      RowBox[{"continuep", "=", "True"}], ",", "continueb"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"While", "[", 
      RowBox[{"continuep", ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"b", "=", "1"}], ";", "\[IndentingNewLine]", 
        RowBox[{"continueb", "=", "True"}], ";", "\[IndentingNewLine]", 
        RowBox[{"While", "[", 
         RowBox[{"continueb", ",", "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"b", "^", "p"}], ">", "n"}], ",", 
            RowBox[{"continueb", "=", "False"}], ",", 
            RowBox[{
             RowBox[{"AppendTo", "[", 
              RowBox[{"L", ",", 
               RowBox[{"b", "^", "p"}]}], "]"}], ";", 
             RowBox[{"b", "++"}]}]}], "]"}]}], "\[IndentingNewLine]", "]"}], 
        ";", "\[IndentingNewLine]", 
        RowBox[{"p", "++"}], ";", "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"2", "^", "p"}], ">", "n"}], ",", 
          RowBox[{"continuep", "=", "False"}]}], "]"}]}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"Union", "[", "L", "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Note that the ",
 StyleBox[ButtonBox["Union",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Union"], "FunctionLink"],
 " function, applied to a single list, returns the list sorted and with \
duplicates removed",
 ". ",
 "We confirm that the list of powers produced by this algorithm is the same \
as the powers considered in Example 2 from the text."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"findPowers", "[", "100", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "1", ",", "4", ",", "8", ",", "9", ",", "16", ",", "25", ",", "27", ",", 
   "32", ",", "36", ",", "49", ",", "64", ",", "81", ",", "100"}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The second function, ",
 StyleBox["findConsecutivePowers", "Input"],
 ", begins by calling ",
 StyleBox["findPowers", "Input"],
 " and storing the list of perfect powers as ",
 StyleBox["powers", "Input"],
 ". Then we use a ",
 StyleBox[ButtonBox["Do",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Do"], "FunctionLink"],
 " loop with second argument ",
 StyleBox["{x,powers}", "Input"],
 ". This sets the variable ",
 StyleBox["x", "Input"],
 " equal to each element of the list ",
 StyleBox["powers", "Input"],
 ". In our procedure, this means that ",
 StyleBox["x", "Input"],
 " is set to each of the perfect powers in turn. In the body of the loop, we \
check to see if the next consecutive integer, ",
 StyleBox["x+1", "Input"],
 ", is also a perfect power using the ",
 StyleBox[ButtonBox["MemberQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/MemberQ"], "FunctionLink"],
 " function. The ",
 StyleBox[ButtonBox["MemberQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/MemberQ"], "FunctionLink"],
 " function requires two arguments. The first is a list to search and the \
second specifies what is being sought. When we find consecutive perfect \
powers, we ",
 StyleBox[ButtonBox["Print",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Print"], "FunctionLink"],
 " them. "
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"findConsecutivePowers", "[", "n_", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"powers", ",", "x"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"powers", "=", 
      RowBox[{"findPowers", "[", "n", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"MemberQ", "[", 
          RowBox[{"powers", ",", 
           RowBox[{"x", "+", "1"}]}], "]"}], ",", 
         RowBox[{"Print", "[", 
          RowBox[{"x", ",", "\"\< \>\"", ",", 
           RowBox[{"x", "+", "1"}]}], "]"}]}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"x", ",", "powers"}], "}"}]}], "]"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Subject to the correctness of our procedures, we can demonstrate that the \
only consecutive perfect powers less than ",
 Cell[BoxData[
  FormBox[
   RowBox[{"100", ",", "000", ",", "000"}], TraditionalForm]]],
 " are 8 and 9 by running the function."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"findConsecutivePowers", "[", "100000000", "]"}]], "Input"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"8", "\[InvisibleSpace]", "\<\" \"\>", "\[InvisibleSpace]", "9"}],
  SequenceForm[8, " ", 9],
  Editable->False]], "Print"]
}, Open  ]],

Cell["\<\
It is worth pointing out that in fact, 8 and 9 are the only consecutive \
perfect powers. That assertion was conjectured by Eug\[EAcute]ne Charles \
Catalan in 1844 and was finally proven in 2002 by Preda Mih\[ACup]ilescu.\
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Existence Proofs", "Subsection",
 CellTags->"Section8b"],

Cell[TextData[{
 "Proofs of existence can also benefit from ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ". Consider Example 10 in Section 1.8 of the text. This example asks, \
\[OpenCurlyDoubleQuote]Show that there is a positive integer that can be \
written as the sum of cubes of positive integers in two different ways.\
\[CloseCurlyDoubleQuote] The solution reports that 1729 is such an integer \
and indicates that a computer search was used to find that value. Let\
\[CloseCurlyQuote]s see how this can be done."
}], "Text"],

Cell[TextData[{
 "The basic idea will be to generate numbers that can be written as the sum \
of cubes. If we generate a number twice, that will tell us that the number \
can be written as the sum of cubes in two different ways. \
We\[CloseCurlyQuote]ll create a list ",
 StyleBox["L", "Input"],
 " and every time we generate a new sum of two cubes, we\[CloseCurlyQuote]ll \
check to see if that number is already in ",
 StyleBox["L", "Input"],
 " using the ",
 StyleBox[ButtonBox["MemberQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/MemberQ"], "FunctionLink"],
 " function. If the new value is already in ",
 StyleBox["L", "Input"],
 ", then that\[CloseCurlyQuote]s the number we\[CloseCurlyQuote]re looking \
for. Otherwise, we add the new number to ",
 StyleBox["L", "Input"],
 " and generate a new sum of two cubes."
}], "Text"],

Cell[TextData[{
 "We generate the sums of cubes with two nested loops that control integers \
",
 StyleBox["a", "Input"],
 " and ",
 StyleBox["b", "Input"],
 ". The inner loop will be a ",
 StyleBox[ButtonBox["For",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/For"], "FunctionLink"],
 " loop that causes ",
 StyleBox["b", "Input"],
 " to range from ",
 StyleBox["1", "Input"],
 " to the value of ",
 StyleBox["a", "Input"],
 ". Using ",
 StyleBox["a", "Input"],
 " as the maximum value means that ",
 StyleBox["b", "Input"],
 " will always be less than or equal to ",
 StyleBox["a", "Input"],
 " and so the procedure will not falsely report results coming from \
commutativity of addition (e.g., ",
 Cell[BoxData[
  FormBox[
   RowBox[{"9", "=", 
    RowBox[{
     RowBox[{
      SuperscriptBox["2", "3"], "+", 
      SuperscriptBox["1", "3"]}], "=", 
     RowBox[{
      SuperscriptBox["1", "3"], "+", 
      SuperscriptBox["2", "3"]}]}]}], TraditionalForm]]],
 "). The outer loop will be a ",
 StyleBox[ButtonBox["While",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/While"], "FunctionLink"],
 " loop with condition (first argument) ",
 StyleBox[ButtonBox["True",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/True"], "FunctionLink"],
 ". The value of ",
 StyleBox["a", "Input"],
 " will be initialized to 1 and incremented by 1 after the inner ",
 StyleBox["b", "Input"],
 " loop completes. The ",
 StyleBox[ButtonBox["While",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/While"], "FunctionLink"],
 " loop in this case is called an infinite loop because it will never stop on \
its own. When the function finds an integer which can be written as the sum \
of cubes in two different ways, the function will ",
 StyleBox[ButtonBox["Throw",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Throw"], "FunctionLink"],
 " that value. That ends the loop and is sent to the ",
 StyleBox[ButtonBox["Catch",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Catch"], "FunctionLink"],
 ", which encompasses the entire body. The infinite loop means that the value \
of ",
 StyleBox["a", "Input"],
 " will continue getting larger and larger with no upper bound. This is \
useful because we don\[CloseCurlyQuote]t know how large the numbers will need \
to be in order to find the example. However, infinite loops should be used \
with caution, especially if you\[CloseCurlyQuote]re not certain that the \
procedure will terminate in a reasonable amount of time."
}], "Text"],

Cell["Here is the function and its result.", "Text"],

Cell[BoxData[
 RowBox[{"twoCubes", ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"L", "=", 
       RowBox[{"{", "}"}]}], ",", 
      RowBox[{"a", "=", "1"}], ",", "b", ",", "n"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{"Catch", "[", "\[IndentingNewLine]", 
     RowBox[{"While", "[", 
      RowBox[{"True", ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"For", "[", 
         RowBox[{
          RowBox[{"b", "=", "1"}], ",", 
          RowBox[{"b", "\[LessEqual]", "a"}], ",", 
          RowBox[{"b", "++"}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"n", "=", 
            RowBox[{
             RowBox[{"a", "^", "3"}], "+", 
             RowBox[{"b", "^", "3"}]}]}], ";", "\[IndentingNewLine]", 
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{"MemberQ", "[", 
              RowBox[{"L", ",", "n"}], "]"}], ",", 
             RowBox[{"Throw", "[", "n", "]"}], ",", 
             RowBox[{"AppendTo", "[", 
              RowBox[{"L", ",", "n"}], "]"}]}], "]"}]}]}], 
         "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"a", "++"}]}]}], "\[IndentingNewLine]", "]"}], "]"}]}], 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData["twoCubes"], "Input"],

Cell[BoxData["1729"], "Output"]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Solutions to Computer Projects and Computations and Explorations\
\>", "Section",
 CellTags->"Solutions"],

Cell[CellGroupData[{

Cell["Computer Projects 3", "Subsection",
 CellTags->"SolutionsCP3"],

Cell["\<\
Given a compound proposition, determine whether it is satisfiable by checking \
its truth value for all positive assignments of truth values to its \
propositional variables.\
\>", "Problem"],

Cell[TextData[{
 StyleBox["Solution:",
  FontSlant->"Italic"],
 " Recall that a proposition is satisfiable if there is at least one \
assignment of truth values to variables that results in a true proposition. \
Our approach will be similar to the way we checked for logical equivalence in \
the ",
 StyleBox["myEquivalentQ", "Input"],
 " function in Section 1.3. Note, of course, that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " provides a built-in function, ",
 StyleBox[ButtonBox["SatisfiableQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/SatisfiableQ"], "FunctionLink"],
 ", that performs this function",
 ". ",
 "The goal of this exercise is to see how such a function might be \
implemented."
}], "Text"],

Cell[TextData[{
 "We create a function, ",
 StyleBox["mySatisfiableQ", "Input"],
 ", that checks all possible assignments of truth values to the propositional \
variables. The ",
 StyleBox["mySatisfiableQ", "Input"],
 " function accepts one argument, a logical expression. It will print out \
all, if any, truth value assignments that satisfy the proposition. We will \
initialize a ",
 StyleBox["result", "Input"],
 " variable to ",
 StyleBox[ButtonBox["False",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/False"], "FunctionLink"],
 ". When an assignment that satisfies the proposition is found, this variable \
is set to ",
 StyleBox[ButtonBox["True",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/True"], "FunctionLink"],
 " and the assignment is printed. After all possible assignments are \
considered, the function returns the ",
 StyleBox["result", "Input"],
 " variable."
}], "Text"],

Cell[TextData[{
 "Since this function is otherwise very similar to ",
 StyleBox["myEquivalentQ", "Input"],
 ", we offer no further explanation."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"mySatisfiableQ", "[", "p_", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"result", "=", "False"}], ",", "vars", ",", "numVars", ",", "i",
       ",", "TA", ",", "val"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"vars", "=", 
      RowBox[{"getVars", "[", "p", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"numVars", "=", 
      RowBox[{"Length", "[", "vars", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"TA", "=", 
      RowBox[{"ConstantArray", "[", 
       RowBox[{"False", ",", "numVars"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"While", "[", 
      RowBox[{
       RowBox[{"TA", "=!=", "Null"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"val", "=", 
         RowBox[{"p", "/.", 
          RowBox[{"MapThread", "[", 
           RowBox[{"Rule", ",", 
            RowBox[{"{", 
             RowBox[{"vars", ",", "TA"}], "}"}]}], "]"}]}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{"val", ",", 
          RowBox[{
           RowBox[{"result", "=", "True"}], ";", 
           RowBox[{"Print", "[", "TA", "]"}]}]}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"TA", "=", 
         RowBox[{"nextTA", "[", "TA", "]"}]}], ";"}]}], "\[IndentingNewLine]",
       "]"}], ";", "\[IndentingNewLine]", "result"}]}], "\[IndentingNewLine]",
    "]"}]}]], "Input",
 InitializationCell->True],

Cell["\<\
We apply this function to the propositions in Example 9 of Section 1.3 of the \
text.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"mySatisfiableQ", "[", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"p", "||", 
     RowBox[{"!", "q"}]}], ")"}], "&&", 
   RowBox[{"(", 
    RowBox[{"q", "||", 
     RowBox[{"!", "r"}]}], ")"}], "&&", 
   RowBox[{"(", 
    RowBox[{"r", "||", 
     RowBox[{"!", "p"}]}], ")"}]}], "]"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"False", ",", "False", ",", "False"}], "}"}]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"True", ",", "True", ",", "True"}], "}"}]], "Print"]
}, Open  ]],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"mySatisfiableQ", "[", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"p", "||", "q", "||", "r"}], ")"}], "&&", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"!", "p"}], "||", 
     RowBox[{"!", "q"}], "||", 
     RowBox[{"!", "r"}]}], ")"}]}], "]"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"True", ",", "False", ",", "False"}], "}"}]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"False", ",", "True", ",", "False"}], "}"}]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"True", ",", "True", ",", "False"}], "}"}]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"False", ",", "False", ",", "True"}], "}"}]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"True", ",", "False", ",", "True"}], "}"}]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"False", ",", "True", ",", "True"}], "}"}]], "Print"]
}, Open  ]],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"mySatisfiableQ", "[", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"p", "||", 
     RowBox[{"!", "q"}]}], ")"}], "&&", 
   RowBox[{"(", 
    RowBox[{"q", "||", 
     RowBox[{"!", "r"}]}], ")"}], "&&", 
   RowBox[{"(", 
    RowBox[{"r", "||", 
     RowBox[{"!", "p"}]}], ")"}], "&&", 
   RowBox[{"(", 
    RowBox[{"p", "||", "q", "||", "r"}], ")"}], "&&", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"!", "p"}], "||", 
     RowBox[{"!", "q"}], "||", 
     RowBox[{"!", "r"}]}], ")"}]}], "]"}]], "Input"],

Cell[BoxData["False"], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Computations and Explorations 1", "Subsection",
 CellTags->"SolutionsCE1"],

Cell["\<\
Look for positive integers that are not the sum of the cubes of eight \
positive integers.\
\>", "Problem"],

Cell[TextData[{
 StyleBox["Solution:",
  FontSlant->"Italic"],
 " We will find integers ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "\[NotEqual]", 
    RowBox[{
     SubsuperscriptBox["a", "1", "3"], "+", 
     SubsuperscriptBox["a", "2", "3"], "+", "\[CenterEllipsis]", "+", 
     SubsuperscriptBox["a", "8", "3"]}]}], TraditionalForm]]],
 " for any integers ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "1"], ",", 
    SubscriptBox["a", "2"], ",", "\[Ellipsis]", ",", 
    SubscriptBox["a", "8"]}], TraditionalForm]]],
 ". ",
 "We can restate the problem as finding a counterexample to the assertion \
that every integer can be written as the sum of eight cubes."
}], "Text"],

Cell[TextData[{
 "Our approach will be to generate all of the integers that are equal to the \
sum of eight cubes and then check to see what integers are missing. For this, \
we need to set a limit ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", i.e., the maximum integer that we\[CloseCurlyQuote]re considering as a \
possible answer to the question. For instance, we might restrict our search \
to integers less than 100. Then we know that each ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["a", "i"], TraditionalForm]]],
 " is at most the cube root of this limit, since ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubsuperscriptBox["a", "i", "3"], "\[LessEqual]", "n"}], 
   TraditionalForm]]],
 "."
}], "Text"],

Cell["\<\
We\[CloseCurlyQuote]ll also want to make our approach as efficient as \
possible in order to find as many such integers as we can. So we make the \
following observations.\
\>", "Text"],

Cell["\<\
Every number that can be expressed as the sum of eight cubes can be expressed \
as the sum of two integers each of which is the sum of four cubes. Those, in \
turn, can be expresses as the sum of two integers which are the sum of two \
cubes each. That is,\
\>", "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{"n", "=", 
   RowBox[{
    RowBox[{"[", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SubsuperscriptBox["a", "1", "3"], "+", 
        SubsuperscriptBox["a", "2", "3"]}], ")"}], "+", 
      RowBox[{"(", 
       RowBox[{
        SubsuperscriptBox["a", "3", "3"], "+", 
        SubsuperscriptBox["a", "4", "3"]}], ")"}]}], "]"}], "+", 
    RowBox[{"[", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SubsuperscriptBox["a", "5", "3"], "+", 
        SubsuperscriptBox["a", "6", "3"]}], ")"}], "+", 
      RowBox[{"(", 
       RowBox[{
        SubsuperscriptBox["a", "7", "3"], "+", 
        SubsuperscriptBox["a", "8", "3"]}], ")"}]}], "]"}]}]}], 
  TraditionalForm]], "Equation"],

Cell[TextData[{
 "This means that we don\[CloseCurlyQuote]t need to write a function to find \
all possible sums of eight cubes. Instead, we\[CloseCurlyQuote]ll write a \
function that, given a list of numbers, will find all possible sums of two \
numbers that are both in that list. If we apply this function to the cubes of \
the numbers from 0 through ",
 Cell[BoxData[
  FormBox[
   RadicalBox["n", "3"], TraditionalForm]]],
 ", that will produce all numbers that are the sums of two cubes. Applying \
the function again to that result will give all numbers that are the sum of \
four cubes. And applying it once again to that result will produce the \
numbers (up to ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ") that are the sum of eight cubes."
}], "Text"],

Cell[TextData[{
 "Additionally, when we find all the possible sums of two integers, we will \
exclude any sum that exceeds our maximum. Recall that we\[CloseCurlyQuote]ve \
determined that if an integer less than or equal to ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " can be written as the sum of cubes, then it can be written as the sum of \
cubes with each ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["a", "i"], TraditionalForm]]],
 " between 0 and ",
 Cell[BoxData[
  FormBox[
   RadicalBox["n", "3"], TraditionalForm]]],
 ". There will be numbers greater than ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " that are generated as the sum of cubes of integers less than ",
 Cell[BoxData[
  FormBox[
   RadicalBox["n", "3"], TraditionalForm]]],
 ", however, these do not provide us with any information about numbers that \
cannot be generated as the sum of eight cubes. And excluding them at each \
step of the process decreases the number of sums that need to be computed."
}], "Text"],

Cell[TextData[{
 "Finally, we may assume that the second number is at least as large as the \
first",
 ". ",
 "Since if we add ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["2", "3"], "+", 
    SuperscriptBox["5", "3"]}], TraditionalForm]]],
 " to our list of sums, there is no need to also include ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["5", "3"], "+", 
    SuperscriptBox["2", "3"]}], TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[{
 "Here is the function that finds all possible sums of pairs of integers from \
the given list ",
 StyleBox["L", "Input"],
 " up to the specified maximum value ",
 StyleBox["max", "Input"],
 ". Note that we use the ",
 StyleBox[ButtonBox["Union",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Union"], "FunctionLink"],
 " function to remove redundancies and also put the list in increasing \
order."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"allPairSums", "[", 
   RowBox[{"L_", ",", "max_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"a", "=", "1"}], ",", "b", ",", "s", ",", 
      RowBox[{"sumList", "=", 
       RowBox[{"{", "}"}]}], ",", 
      RowBox[{"num", "=", 
       RowBox[{"Length", "[", "L", "]"}]}]}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"While", "[", 
      RowBox[{
       RowBox[{"a", "\[LessEqual]", "num"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"b", "=", "a"}], ";", "\[IndentingNewLine]", 
        RowBox[{"While", "[", 
         RowBox[{
          RowBox[{"b", "\[LessEqual]", "num"}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"s", "=", 
            RowBox[{
             RowBox[{"L", "[", 
              RowBox[{"[", "a", "]"}], "]"}], "+", 
             RowBox[{"L", "[", 
              RowBox[{"[", "b", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
           
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{"s", "\[LessEqual]", "max"}], ",", 
             RowBox[{"AppendTo", "[", 
              RowBox[{"sumList", ",", "s"}], "]"}], ",", 
             RowBox[{"b", "=", "num"}]}], "]"}], ";", "\[IndentingNewLine]", 
           RowBox[{"b", "++"}], ";"}]}], "\[IndentingNewLine]", "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"a", "++"}], ";"}]}], "\[IndentingNewLine]", "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Union", "[", "sumList", "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "With this function in place, we need to apply it (three times) to a list of \
cubes. We\[CloseCurlyQuote]ll consider\:fffccubes up to ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["7", "3"], TraditionalForm]]],
 ", and including 0. The ",
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 " function used below forms the list of all values obtained by evaluating \
the first argument after replacing the variable ",
 StyleBox["i", "Input"],
 " by every integer between (inclusive) the two given in the second \
argument."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"someCubes", "=", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"i", "^", "3"}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "0", ",", "7"}], "}"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0", ",", "1", ",", "8", ",", "27", ",", "64", ",", "125", ",", "216", ",", 
   "343"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Applying the ",
 StyleBox["allPairSums", "Input"],
 " function once gives us all the sums of pairs of cubes (up to ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["7", "3"], "=", "343"}], TraditionalForm]]],
 ")."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"twoCubes", "=", 
  RowBox[{"allPairSums", "[", 
   RowBox[{"someCubes", ",", "343"}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0", ",", "1", ",", "2", ",", "8", ",", "9", ",", "16", ",", "27", ",", 
   "28", ",", "35", ",", "54", ",", "64", ",", "65", ",", "72", ",", "91", 
   ",", "125", ",", "126", ",", "128", ",", "133", ",", "152", ",", "189", 
   ",", "216", ",", "217", ",", "224", ",", "243", ",", "250", ",", "280", 
   ",", "341", ",", "343"}], "}"}]], "Output"]
}, Open  ]],

Cell["\<\
Applying it to that result gives all possible sums of four cubes.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"fourCubes", "=", 
  RowBox[{"allPairSums", "[", 
   RowBox[{"twoCubes", ",", "343"}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0", ",", "1", ",", "2", ",", "3", ",", "4", ",", "8", ",", "9", ",", "10", 
   ",", "11", ",", "16", ",", "17", ",", "18", ",", "24", ",", "25", ",", 
   "27", ",", "28", ",", "29", ",", "30", ",", "32", ",", "35", ",", "36", 
   ",", "37", ",", "43", ",", "44", ",", "51", ",", "54", ",", "55", ",", 
   "56", ",", "62", ",", "63", ",", "64", ",", "65", ",", "66", ",", "67", 
   ",", "70", ",", "72", ",", "73", ",", "74", ",", "80", ",", "81", ",", 
   "82", ",", "88", ",", "89", ",", "91", ",", "92", ",", "93", ",", "99", 
   ",", "100", ",", "107", ",", "108", ",", "118", ",", "119", ",", "125", 
   ",", "126", ",", "127", ",", "128", ",", "129", ",", "130", ",", "133", 
   ",", "134", ",", "135", ",", "136", ",", "137", ",", "141", ",", "142", 
   ",", "144", ",", "145", ",", "149", ",", "152", ",", "153", ",", "154", 
   ",", "155", ",", "156", ",", "160", ",", "161", ",", "163", ",", "168", 
   ",", "179", ",", "180", ",", "182", ",", "187", ",", "189", ",", "190", 
   ",", "191", ",", "192", ",", "193", ",", "197", ",", "198", ",", "200", 
   ",", "205", ",", "206", ",", "216", ",", "217", ",", "218", ",", "219", 
   ",", "224", ",", "225", ",", "226", ",", "232", ",", "233", ",", "240", 
   ",", "243", ",", "244", ",", "245", ",", "250", ",", "251", ",", "252", 
   ",", "253", ",", "254", ",", "256", ",", "258", ",", "259", ",", "261", 
   ",", "266", ",", "270", ",", "271", ",", "277", ",", "278", ",", "280", 
   ",", "281", ",", "282", ",", "285", ",", "288", ",", "289", ",", "296", 
   ",", "297", ",", "304", ",", "307", ",", "308", ",", "314", ",", "315", 
   ",", "317", ",", "322", ",", "334", ",", "341", ",", "342", ",", "343"}], 
  "}"}]], "Output"]
}, Open  ]],

Cell["\<\
And once again we obtain all integers up to 343 which can be obtained as the \
sum of eight cubes.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"eightCubes", "=", 
  RowBox[{"allPairSums", "[", 
   RowBox[{"fourCubes", ",", "343"}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0", ",", "1", ",", "2", ",", "3", ",", "4", ",", "5", ",", "6", ",", "7", 
   ",", "8", ",", "9", ",", "10", ",", "11", ",", "12", ",", "13", ",", "14", 
   ",", "15", ",", "16", ",", "17", ",", "18", ",", "19", ",", "20", ",", 
   "21", ",", "22", ",", "24", ",", "25", ",", "26", ",", "27", ",", "28", 
   ",", "29", ",", "30", ",", "31", ",", "32", ",", "33", ",", "34", ",", 
   "35", ",", "36", ",", "37", ",", "38", ",", "39", ",", "40", ",", "41", 
   ",", "42", ",", "43", ",", "44", ",", "45", ",", "46", ",", "47", ",", 
   "48", ",", "49", ",", "50", ",", "51", ",", "52", ",", "53", ",", "54", 
   ",", "55", ",", "56", ",", "57", ",", "58", ",", "59", ",", "60", ",", 
   "61", ",", "62", ",", "63", ",", "64", ",", "65", ",", "66", ",", "67", 
   ",", "68", ",", "69", ",", "70", ",", "71", ",", "72", ",", "73", ",", 
   "74", ",", "75", ",", "76", ",", "77", ",", "78", ",", "79", ",", "80", 
   ",", "81", ",", "82", ",", "83", ",", "84", ",", "85", ",", "86", ",", 
   "87", ",", "88", ",", "89", ",", "90", ",", "91", ",", "92", ",", "93", 
   ",", "94", ",", "95", ",", "96", ",", "97", ",", "98", ",", "99", ",", 
   "100", ",", "101", ",", "102", ",", "103", ",", "104", ",", "105", ",", 
   "106", ",", "107", ",", "108", ",", "109", ",", "110", ",", "111", ",", 
   "112", ",", "113", ",", "114", ",", "115", ",", "116", ",", "117", ",", 
   "118", ",", "119", ",", "120", ",", "121", ",", "122", ",", "123", ",", 
   "124", ",", "125", ",", "126", ",", "127", ",", "128", ",", "129", ",", 
   "130", ",", "131", ",", "132", ",", "133", ",", "134", ",", "135", ",", 
   "136", ",", "137", ",", "138", ",", "139", ",", "140", ",", "141", ",", 
   "142", ",", "143", ",", "144", ",", "145", ",", "146", ",", "147", ",", 
   "148", ",", "149", ",", "150", ",", "151", ",", "152", ",", "153", ",", 
   "154", ",", "155", ",", "156", ",", "157", ",", "158", ",", "159", ",", 
   "160", ",", "161", ",", "162", ",", "163", ",", "164", ",", "165", ",", 
   "166", ",", "167", ",", "168", ",", "169", ",", "170", ",", "171", ",", 
   "172", ",", "173", ",", "174", ",", "175", ",", "176", ",", "177", ",", 
   "178", ",", "179", ",", "180", ",", "181", ",", "182", ",", "183", ",", 
   "184", ",", "185", ",", "186", ",", "187", ",", "188", ",", "189", ",", 
   "190", ",", "191", ",", "192", ",", "193", ",", "194", ",", "195", ",", 
   "196", ",", "197", ",", "198", ",", "199", ",", "200", ",", "201", ",", 
   "202", ",", "203", ",", "204", ",", "205", ",", "206", ",", "207", ",", 
   "208", ",", "209", ",", "210", ",", "211", ",", "212", ",", "213", ",", 
   "214", ",", "215", ",", "216", ",", "217", ",", "218", ",", "219", ",", 
   "220", ",", "221", ",", "222", ",", "223", ",", "224", ",", "225", ",", 
   "226", ",", "227", ",", "228", ",", "229", ",", "230", ",", "231", ",", 
   "232", ",", "233", ",", "234", ",", "235", ",", "236", ",", "237", ",", 
   "238", ",", "240", ",", "241", ",", "242", ",", "243", ",", "244", ",", 
   "245", ",", "246", ",", "247", ",", "248", ",", "249", ",", "250", ",", 
   "251", ",", "252", ",", "253", ",", "254", ",", "255", ",", "256", ",", 
   "257", ",", "258", ",", "259", ",", "260", ",", "261", ",", "262", ",", 
   "263", ",", "264", ",", "265", ",", "266", ",", "267", ",", "268", ",", 
   "269", ",", "270", ",", "271", ",", "272", ",", "273", ",", "274", ",", 
   "275", ",", "276", ",", "277", ",", "278", ",", "279", ",", "280", ",", 
   "281", ",", "282", ",", "283", ",", "284", ",", "285", ",", "286", ",", 
   "287", ",", "288", ",", "289", ",", "290", ",", "291", ",", "292", ",", 
   "293", ",", "294", ",", "295", ",", "296", ",", "297", ",", "298", ",", 
   "299", ",", "300", ",", "301", ",", "302", ",", "303", ",", "304", ",", 
   "305", ",", "306", ",", "307", ",", "308", ",", "309", ",", "310", ",", 
   "311", ",", "312", ",", "313", ",", "314", ",", "315", ",", "316", ",", 
   "317", ",", "318", ",", "319", ",", "320", ",", "321", ",", "322", ",", 
   "323", ",", "324", ",", "325", ",", "326", ",", "327", ",", "328", ",", 
   "329", ",", "330", ",", "331", ",", "332", ",", "333", ",", "334", ",", 
   "335", ",", "336", ",", "337", ",", "338", ",", "339", ",", "340", ",", 
   "341", ",", "342", ",", "343"}], "}"}]], "Output",
 PageBreakWithin->True]
}, Open  ]],

Cell["\<\
And finally, we print out the integers that are missing from the list.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"For", "[", 
  RowBox[{
   RowBox[{"i", "=", "1"}], ",", 
   RowBox[{"i", "\[LessEqual]", "343"}], ",", 
   RowBox[{"i", "++"}], ",", 
   RowBox[{"If", "[", 
    RowBox[{
     RowBox[{"!", 
      RowBox[{"MemberQ", "[", 
       RowBox[{"eightCubes", ",", "i"}], "]"}]}], ",", 
     RowBox[{"Print", "[", "i", "]"}]}], "]"}]}], "]"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData["23"], "Print"],

Cell[BoxData["239"], "Print"]
}, Open  ]]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Exercises", "Section",
 CellTags->"Exercises"],

Cell[CellGroupData[{

Cell[TextData[{
 "Write functions ",
 StyleBox["or", "Input"],
 ", ",
 StyleBox["xor", "Input"],
 ", and ",
 StyleBox["not", "Input"],
 " to implement those bit string operators."
}], "Item1Exercise"],

Cell[TextData[{
 "Use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " ",
 "to solve exercises 19 through 23 in Section 1.2, using the knights and \
knaves puzzle that was solved earlier in this chapter as a guide."
}], "Item1Exercise"],

Cell[TextData[{
 "Write a ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function to find the dual of a proposition. Dual is defined in the \
Exercises of Section 1.3",
 ". ",
 "(Hint: you may find it useful to know that ",
 StyleBox[ButtonBox["And",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/And"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["Or",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Or"], "FunctionLink"],
 " are heads in logical expressions.)"
}], "Item1Exercise"],

Cell[TextData[{
 "Write a function ",
 StyleBox["uniqueness", "Input"],
 ", based on the built-in ",
 StyleBox[ButtonBox["Exists",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Exists"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["ForAll",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ForAll"], "FunctionLink"],
 " functions, to implement the uniqueness quantifier, described in Section \
1.4 of the text",
 ". "
}], "Item1Exercise"],

Cell[CellGroupData[{

Cell[TextData[{
 "Write a ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function that plays the obligato game in the role of the student, as \
described in the Supplementary Exercises of Chapter 1. Specifically, the \
function should accept two arguments. The first argument is the new statement \
that you, as the teacher, provide. The second argument should be the list of ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "\[CloseCurlyQuote]s responses to all the previous statements. For example, \
suppose the teacher\[CloseCurlyQuote]s first statement is ",
 Cell[BoxData[
  FormBox[
   RowBox[{"p", "\[Rule]", 
    RowBox[{"(", 
     RowBox[{"q", "\[Or]", "r"}], ")"}]}], TraditionalForm]]],
 ", the second statement is ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Not]", "p"}], "\[Or]", "q"}], TraditionalForm]]],
 ", and the third statement is ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 ". If the function/student accepts the first statement and denies the second \
statement, then you would obtain the response to the third statement by \
executing"
}], "Item1Exercise",
 CellGroupingRules->{GroupTogetherGrouping, 17001.}],

Cell[BoxData[
 RowBox[{"obligato", "[", 
  RowBox[{"r", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Implies", "[", 
      RowBox[{"p", ",", 
       RowBox[{"q", "||", "r"}]}], "]"}], ",", 
     RowBox[{"!", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"!", "p"}], "||", "q"}], ")"}]}]}], "}"}]}], "]"}]], "myCode",\

 Evaluatable->False,
 CellGroupingRules->{GroupTogetherGrouping, 17001.},
 InitializationCell->False],

Cell[TextData[{
 "The function must accept the statement ",
 StyleBox["r",
  FontWeight->"Bold"],
 " and thus returns the list with that response included, as shown below:"
}], "ExerciseContinuation",
 CellGroupingRules->{GroupTogetherGrouping, 17001.}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"p", "\[Implies]", 
    RowBox[{"q", "||", "r"}]}], ",", 
   RowBox[{"!", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"!", "p"}], "||", "q"}], ")"}]}], ",", "r"}], "}"}]], "Output",
 CellMargins->{{80, 10}, {10, 5}},
 CellGroupingRules->{GroupTogetherGrouping, 17001.}]
}, Open  ]]
}, Open  ]]
}, Closed]]
}, Open  ]]
},
WindowSize->{725, 850},
WindowMargins->{{163, Automatic}, {Automatic, 9}},
DockedCells->Cell[
  BoxData[
   TagBox[
    GridBox[{{"\"Exploring Discrete Mathematics with Mathematica\"", 
       ActionMenuBox[
       "\"Navigation\"", {
        "\"CloseAll\"" :> Module[{$CellContext`DJSection$}, SelectionMove[
             SelectedNotebook[], All, Notebook]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; 
           Do[NotebookLocate[$CellContext`DJSection$]; FrontEndExecute[
               FrontEndToken["OpenCloseGroup"]], {$CellContext`DJSection$, {
              "Section1a", "Section1b", "Section1c", "Section1d", "Section1e",
                "Section2a", "Section2b", "Section3a", "Section3b", 
               "Section4a", "Section4b", "Section6a", "Section7a", 
               "Section7b", "Section8a", "Section8b", "SolutionsCE1", 
               "SolutionsCP3", "Section1", "Section2", "Section3", "Section4",
                "Section5", "Section6", "Section7", "Section8", "Solutions", 
               "Exercises"}}]; SelectionMove[
             SelectedNotebook[], Before, Notebook]], "\"Section1\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Section6", "Section7", "Section8", "Solutions", 
                "Exercises"}}] NotebookLocate["Section1"]; FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Section2\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Section6", "Section7", "Section8", "Solutions", 
                "Exercises"}}] NotebookLocate["Section2"]; FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Section3\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Section6", "Section7", "Section8", "Solutions", 
                "Exercises"}}] NotebookLocate["Section3"]; FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Section4\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Section6", "Section7", "Section8", "Solutions", 
                "Exercises"}}] NotebookLocate["Section4"]; FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Section5\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Section6", "Section7", "Section8", "Solutions", 
                "Exercises"}}] NotebookLocate["Section5"]; FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Section6\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Section6", "Section7", "Section8", "Solutions", 
                "Exercises"}}] NotebookLocate["Section6"]; FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Section7\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Section6", "Section7", "Section8", "Solutions", 
                "Exercises"}}] NotebookLocate["Section7"]; FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Section8\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Section6", "Section7", "Section8", "Solutions", 
                "Exercises"}}] NotebookLocate["Section8"]; FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Solutions\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Section6", "Section7", "Section8", "Solutions", 
                "Exercises"}}] NotebookLocate["Solutions"]; FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Exercises\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Section6", "Section7", "Section8", "Solutions", 
                "Exercises"}}] NotebookLocate["Exercises"]; FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Open All\"" :> 
         Module[{$CellContext`DJSection$}, 
           Do[NotebookLocate[$CellContext`DJSection$]; FrontEndExecute[
               FrontEndToken[
               "SelectionOpenAllGroups"]], {$CellContext`DJSection$, {
              "Exercises", "Section1", "Section1a", "Section1b", "Section1c", 
               "Section1d", "Section1e", "Section2", "Section2a", "Section2b",
                "Section3", "Section3a", "Section3b", "Section4", "Section4a",
                "Section4b", "Section5", "Section6", "Section6a", "Section7", 
               "Section7a", "Section7b", "Section8", "Section8a", "Section8b",
                "Solutions", "SolutionsCE1", "SolutionsCP3"}}] SelectionMove[
             SelectedNotebook[], Before, Notebook]]}]}}, 
     GridBoxAlignment -> {
      "Columns" -> {Left, Baseline}, "Rows" -> {Right, Baseline}}, AutoDelete -> 
     False, GridBoxItemSize -> {"Columns" -> {
         Scaled[0.8], 
         Scaled[0.2]}}], "Grid"]], "DockedCell", FontFamily -> "Times"],
PrintingCopies->1,
PrintingPageRange->{1, Automatic},
ShowSelection->True,
Magnification->1,
FrontEndVersion->"9.0 for Mac OS X x86 (32-bit, 64-bit Kernel) (January 25, \
2013)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> 
     FrontEnd`FileName[{"Book"}, "Textbook.nb", CharacterEncoding -> 
       "UTF-8"]]], 
   Cell[
    StyleData[All, "Printout"], PageWidth -> PaperWidth, 
    CellHorizontalScrolling -> True, 
    CellLabelMargins -> {{2, Inherited}, {Inherited, Inherited}}, 
    ShowAutoStyles -> False, 
    AutoStyleOptions -> {"HighlightMisspelledWords" -> False}, ScriptMinSize -> 
    9, MenuSortingValue -> 1500, 
    PrivateFontOptions -> {"FontType" -> "Outline"}, Magnification -> 1], 
   Cell[
    StyleData["Title"], Editable -> False, ShowCellBracket -> Automatic, 
    CellGroupingRules -> {"TitleGrouping", 0}, PageBreakBelow -> False, 
    DefaultNewInlineCellStyle -> {}, LanguageCategory -> "NaturalLanguage", 
    FontFamily -> "Helvetica", FontSize -> 25, FontWeight -> "Bold", 
    FontColor -> GrayLevel[0.137255]], 
   Cell[
    StyleData["BookChapterTitle", StyleDefinitions -> StyleData["Title"]], 
    CellFrame -> {{0, 0}, {1, 1}}, CellDingbat -> Cell[
      BoxData[
       GraphicsBox[{{
          RGBColor[0.09375, 0.570313, 0.816406], 
          RectangleBox[{0, 0}]}, {
          GrayLevel[1], FontSize -> 35, FontWeight -> Bold, FontFamily -> 
          "Helvetica", 
          InsetBox[
           CounterBox["ChapterNumber"], {0.5, 0.45}, {Center, Center}]}}, 
        ImageSize -> {70, 70}]], "Output"], CellMargins -> {{75, 0}, {20, 5}},
     CellFrameColor -> GrayLevel[1]], 
   Cell[
    StyleData["DockedCell"], CellMargins -> {{0, 12}, {0, 0}}, 
    CellGroupingRules -> "TitleGrouping", 
    ConversionRules :> {"TeX" -> {"\n\\chapter{", "}"}}, 
    TaggingRules :> {"CounterSettings" -> {"CounterTarget" -> "CellGroup"}}, 
    PageBreakAbove -> False, PageBreakBelow -> False, LineSpacing -> {1, 2}, 
    MenuSortingValue -> 1105, FontFamily -> "Helvetica", FontSize -> 20, 
    FontWeight -> "Bold", Background -> 
    RGBColor[0.09375, 0.570313, 0.816406]], 
   Cell[
    StyleData["FunctionLink"], ConversionRules :> {"TeX" -> {{"", 
         $CellContext`ExportAsHyperRef[], ""}, {"", Automatic, ""}}}, 
    MenuSortingValue -> 1105, ButtonStyleMenuListing -> None, FontFamily -> 
    "Courier", FontVariations -> {"Underline" -> True}, FontColor -> 
    RGBColor[0, 0, 1], 
    ButtonBoxOptions -> {Active -> True, ButtonFunction :> (FrontEndExecute[{
         FrontEnd`NotebookLocate[#2]}]& ), ButtonNote -> ButtonData}], 
   Cell[
    StyleData["Problem"], CellMargins -> {{40, 12}, {15, 15}}, 
    CellGroupingRules -> {"GroupTogetherNestedGrouping", 15010}, 
    ConversionRules :> {
     "TeX" -> {{"", Automatic, ""}, {"\n\n", Automatic, ""}}}, 
    TaggingRules :> {
     "ExtraConversionRules" -> {"General" -> {"RestartList" -> False}}}], 
   Cell[
    CellGroupData[{
      Cell[
       StyleData["Item1Exercise"], CellDingbat -> Cell[
         TextData[{
           CounterBox["Item1Exercise"], "."}], CellSize -> {30, Inherited}, 
         TextAlignment -> Right, FontWeight -> "Bold"], 
       CellMargins -> {{50, 12}, {3, 5}}, 
       CellGroupingRules -> {"GroupTogetherNestedGrouping", 17000}, 
       CellFrameLabels -> {{None, None}, {None, None}}, CellFrameLabelMargins -> 
       6, CounterIncrements -> "Item1Exercise", 
       CounterAssignments -> {{"Item2Exercise", 0}}, MenuSortingValue -> 
       4100], 
      Cell[
       StyleData["Item1Exercise", "Printout"], CellDingbat -> Cell[
         TextData[{
           CounterBox["Item1Exercise"], "."}], CellSize -> {20, Inherited}, 
         TextAlignment -> Right, FontWeight -> "Bold"], 
       CellMargins -> {{35, 12}, {3, 5}}, 
       CellGroupingRules -> {"GroupTogetherNestedGrouping", 17000}, 
       CellFrameLabels -> {{None, None}, {None, None}}, CellFrameLabelMargins -> 
       6]}, Open]], 
   Cell[
    CellGroupData[{
      Cell[
       StyleData["ExerciseContinuation"], CellMargins -> {{50, 12}, {1, 3}}, 
       CellGroupingRules -> {"GroupTogetherNestedGrouping", 17000}, 
       CellFrameLabels -> {{None, None}, {None, None}}, CellFrameLabelMargins -> 
       6, MenuSortingValue -> 4101], 
      Cell[
       StyleData["ExerciseContinuation", "Printout"], 
       CellMargins -> {{35, 12}, {1, 3}}, 
       CellFrameLabels -> {{None, None}, {None, None}}, CellFrameLabelMargins -> 
       6]}, Open]], 
   Cell[
    CellGroupData[{
      Cell[
       StyleData["Item2Exercise"], CellDingbat -> Cell[
         TextData[{
           CounterBox["Item2Exercise"], "."}], CellSize -> {32, Inherited}, 
         TextAlignment -> Right, FontWeight -> "Bold"], 
       CellMargins -> {{75, 12}, {1, 2}}, 
       CellGroupingRules -> {"GroupTogetherNestedGrouping", 17100}, 
       CellFrameLabels -> {{None, None}, {None, None}}, CellFrameLabelMargins -> 
       6, CounterIncrements -> "Item2Exercise", MenuSortingValue -> 4200, 
       CounterBoxOptions -> {CounterFunction :> (Part[
           CharacterRange["a", "z"], #]& )}], 
      Cell[
       StyleData["Item2Exercise", "Printout"], CellDingbat -> Cell[
         TextData[{
           CounterBox["Item2Exercise"], "."}], CellSize -> {20, Inherited}, 
         TextAlignment -> Right, FontWeight -> "Bold"], 
       CellMargins -> {{50, 12}, {1, 2}}, 
       CellFrameLabels -> {{None, None}, {None, None}}, CellFrameLabelMargins -> 
       6, CounterBoxOptions -> {CounterFunction :> (Part[
           CharacterRange["a", "z"], #]& )}]}, Open]], 
   Cell[
    StyleData["ExerciseEquation"], CellMargins -> {{36, 12}, {7, 7}}, 
    CellGroupingRules -> {"GroupTogetherNestedGrouping", 17200}, 
    CellHorizontalScrolling -> True, 
    CellFrameLabels -> {{None, None}, {None, None}}, DefaultFormatType -> 
    TraditionalForm, InputAutoReplacements -> {}, TextAlignment -> Center, 
    MenuSortingValue -> 4300, 
    ButtonBoxOptions -> {Appearance -> {Automatic, None}}], 
   Cell[
    StyleData["myCode"], CellMargins -> {{66, 10}, {5, 10}}, 
    StyleKeyMapping -> {"=" -> "WolframAlphaShort"}, Evaluatable -> False, 
    CellGroupingRules -> "InputGrouping", TextClipboardType -> "InputText", 
    PageBreakWithin -> False, GroupPageBreakWithin -> False, 
    DefaultFormatType -> DefaultInputFormatType, ContextMenu -> 
    FEPrivate`FrontEndResource["ContextMenus", "myCode"], ShowAutoStyles -> 
    True, "TwoByteSyntaxCharacterAutoReplacement" -> True, 
    HyphenationOptions -> {"HyphenationCharacter" -> "\[Continuation]"}, 
    AutoItalicWords -> {}, LanguageCategory -> "Mathematica", FormatType -> 
    InputForm, NumberMarks -> True, 
    LinebreakAdjustments -> {0.85, 2, 10, 0, 1}, CounterIncrements -> "Code", 
    MenuSortingValue -> 1495, MenuCommandKey -> "8", FontWeight -> 
    "Bold", $CellContext`StripStyleOnPaste -> 
    True, $CellContext`AutoQuoteCharacters -> {}, \
$CellContext`PasteAutoQuoteCharacters -> {}], 
   Cell[
    StyleData["CodeVar"], DefaultFormatType -> DefaultInputFormatType, 
    LineBreakWithin -> False, MenuSortingValue -> 10000, FontWeight -> "Bold",
     FontSlant -> "Italic", FontColor -> GrayLevel[0.4]], 
   Cell[
    CellGroupData[{
      Cell[
       StyleData["Section"], CellFrame -> {{6, 0}, {0, 1}}, ShowCellBracket -> 
       Automatic, CellMargins -> {{12, 12}, {3, 24}}, 
       CellGroupingRules -> {"SectionGrouping", 30}, 
       ConversionRules :> {
        "TeX" -> {"\n\\section{", "}"}, 
         "HTML" -> {"<h3 class=\"Section\">", "</h3>"}}, 
       TaggingRules :> {
        "CounterSettings" -> {
          "CounterFormat" -> {"BookChapterNumber", ".", "Section"}, 
           "CounterTarget" -> "Cell", "XRefPrefix" -> "Section"}}, 
       PageBreakBelow -> False, CellFrameMargins -> {{8, 8}, {4, 6}}, 
       CellFrameColor -> RGBColor[0.09375, 0.570313, 0.816406], TabSpacings -> 
       2.1, CounterIncrements -> {
        "Section", "SectionHeader", "SubsectionHeader", "SubsubsectionHeader",
          "SubsubsubsectionHeader"}, FontFamily -> "Helvetica", FontSize -> 
       16, FontWeight -> "Bold", Background -> GrayLevel[0.999985]], 
      Cell[
       StyleData["Section", "Printout"], CellFrame -> {{4, 0}, {0, 0.5}}, 
       CellMargins -> {{2, 2}, {3, 21}}, PageBreakBelow -> False, 
       CellFrameMargins -> {{6, 6}, {3, 5}}, CellFrameColor -> 
       RGBColor[0.09375, 0.570313, 0.816406]], 
      Cell[
       StyleData["Section", "DoubleSpacedPrintout"], 
       CellFrame -> {{4, 0}, {0, 0.5}}, CellMargins -> {{2, 2}, {3, 21}}, 
       PageBreakBelow -> False, CellFrameMargins -> {{6, 6}, {3, 5}}, 
       CellFrameColor -> RGBColor[0.09375, 0.570313, 0.816406], FontSize -> 
       14]}, Open]], 
   Cell[
    CellGroupData[{
      Cell[
       StyleData["Subsubsection"], CellDingbat -> None, 
       CellMargins -> {{12, 12}, {0, 6}}, 
       CellElementSpacings -> {"ClosedGroupTopMargin" -> 12}, 
       CellGroupingRules -> {"SectionGrouping", 50}, 
       ConversionRules :> {
        "TeX" -> {"\n\\subsubsection{", "}"}, 
         "HTML" -> {"<h5 class=\"Subsubsection\">", "</h5>"}}, PageBreakBelow -> 
       False, CellFrameLabelMargins -> 4, 
       CounterIncrements -> {
        "Subsubsection", "SubsubsectionHeader", "SubsubsubsectionHeader"}, 
       FontFamily -> "Helvetica", FontSize -> 11, FontWeight -> "Bold", 
       FontSlant -> "Italic"], 
      Cell[
       StyleData["Subsubsection", "Printout"], PageBreakBelow -> False, 
       FontFamily -> "Helvetica", FontSize -> 11, FontWeight -> "Bold", 
       FontSlant -> "Italic"]}, Open]], 
   Cell[
    StyleData["Equation"], CellMargins -> {{26, 10}, {7, 7}}, 
    ConversionRules :> {
     "HTML" -> {{"", ""}, {"<p class=\"Equation\">", "</p>"}}}, 
    CellHorizontalScrolling -> True, DefaultFormatType -> TraditionalForm, 
    InputAutoReplacements -> {}, TextAlignment -> Center, CounterIncrements -> 
    "Equation", ButtonBoxOptions -> {Appearance -> {Automatic, None}}], 
   Cell[
    CellGroupData[{
      Cell[
       StyleData["Subsubsubsection"], CellDingbat -> None, 
       CellMargins -> {{12, 12}, {0, 6}}, 
       CellGroupingRules -> {"SectionGrouping", 60}, 
       ConversionRules :> {
        "TeX" -> {"\n\\subsubsubsection{", "}"}, 
         "HTML" -> {"<h6 class=\"Subsubsubsection\">", "</h6>"}}, 
       PageBreakBelow -> False, CellFrameLabelMargins -> 4, 
       CounterIncrements -> {"Subsubsubsection", "SubsubsubsectionHeader"}, 
       FontFamily -> "Helvetica", FontSize -> 11, FontSlant -> "Italic"], 
      Cell[
       StyleData["Subsubsubsubsection", "Printout"], 
       CellMargins -> {{49, Inherited}, {2, 10}}, PageBreakBelow -> False]}, 
     Open]]}, Visible -> False, FrontEndVersion -> 
  "9.0 for Mac OS X x86 (32-bit, 64-bit Kernel) (January 25, 2013)", 
  StyleDefinitions -> "PrivateStylesheetFormatting.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "Section1"->{
  Cell[1355, 46, 65, 1, 55, "Section",
   CellTags->"Section1"]},
 "Section1a"->{
  Cell[3925, 145, 65, 1, 32, "Subsection",
   CellTags->"Section1a"]},
 "Section1b"->{
  Cell[10860, 397, 68, 1, 26, "Subsection",
   CellTags->"Section1b"]},
 "Section1c"->{
  Cell[19636, 665, 68, 1, 26, "Subsection",
   CellTags->"Section1c"]},
 "Section1d"->{
  Cell[23591, 812, 68, 1, 26, "Subsection",
   CellTags->"Section1d"]},
 "Section1e"->{
  Cell[32381, 1092, 70, 1, 26, "Subsection",
   CellTags->"Section1e"]},
 "Section2"->{
  Cell[55283, 1927, 81, 1, 35, "Section",
   CellTags->"Section2"]},
 "Section2a"->{
  Cell[55721, 1941, 67, 1, 32, "Subsection",
   CellTags->"Section2a"]},
 "Section2b"->{
  Cell[73703, 2569, 59, 1, 26, "Subsection",
   CellTags->"Section2b"]},
 "Section3"->{
  Cell[78521, 2723, 71, 1, 35, "Section",
   CellTags->"Section3"]},
 "Section3a"->{
  Cell[78944, 2738, 64, 1, 32, "Subsection",
   CellTags->"Section3a"]},
 "Section3b"->{
  Cell[84175, 2924, 73, 1, 26, "Subsection",
   CellTags->"Section3b"]},
 "Section4"->{
  Cell[136639, 4717, 72, 1, 35, "Section",
   CellTags->"Section4"]},
 "Section4a"->{
  Cell[138294, 4785, 100, 2, 32, "Subsection",
   PageBreakWithin->False,
   CellTags->"Section4a"]},
 "Section4b"->{
  Cell[143589, 4986, 82, 1, 26, "Subsection",
   CellTags->"Section4b"]},
 "Section5"->{
  Cell[152746, 5345, 64, 1, 35, "Section",
   CellTags->"Section5"]},
 "Section6"->{
  Cell[160001, 5636, 64, 1, 35, "Section",
   CellTags->"Section6"]},
 "Section6a"->{
  Cell[166967, 5846, 76, 1, 32, "Subsection",
   CellTags->"Section6a"]},
 "Section7"->{
  Cell[180279, 6248, 68, 1, 35, "Section",
   CellTags->"Section7"]},
 "Section7a"->{
  Cell[181163, 6274, 70, 1, 32, "Subsection",
   CellTags->"Section7a"]},
 "Section7b"->{
  Cell[187979, 6481, 51, 1, 26, "Subsection",
   CellTags->"Section7b"]},
 "Section8"->{
  Cell[194715, 6703, 72, 1, 35, "Section",
   CellTags->"Section8"]},
 "Section8a"->{
  Cell[195028, 6715, 62, 1, 32, "Subsection",
   CellTags->"Section8a"]},
 "Section8b"->{
  Cell[203146, 6952, 62, 1, 26, "Subsection",
   CellTags->"Section8b"]},
 "Solutions"->{
  Cell[208550, 7107, 115, 3, 35, "Section",
   CellTags->"Solutions"]},
 "SolutionsCP3"->{
  Cell[208690, 7114, 68, 1, 32, "Subsection",
   CellTags->"SolutionsCP3"]},
 "SolutionsCE1"->{
  Cell[214549, 7320, 80, 1, 26, "Subsection",
   CellTags->"SolutionsCE1"]},
 "Exercises"->{
  Cell[230819, 7743, 52, 1, 35, "Section",
   CellTags->"Exercises"]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"Section1", 253803, 8229},
 {"Section1a", 253885, 8232},
 {"Section1b", 253972, 8235},
 {"Section1c", 254060, 8238},
 {"Section1d", 254148, 8241},
 {"Section1e", 254236, 8244},
 {"Section2", 254324, 8247},
 {"Section2a", 254409, 8250},
 {"Section2b", 254498, 8253},
 {"Section3", 254586, 8256},
 {"Section3a", 254671, 8259},
 {"Section3b", 254760, 8262},
 {"Section4", 254848, 8265},
 {"Section4a", 254934, 8268},
 {"Section4b", 255052, 8272},
 {"Section5", 255141, 8275},
 {"Section6", 255226, 8278},
 {"Section6a", 255312, 8281},
 {"Section7", 255401, 8284},
 {"Section7a", 255487, 8287},
 {"Section7b", 255577, 8290},
 {"Section8", 255666, 8293},
 {"Section8a", 255752, 8296},
 {"Section8b", 255842, 8299},
 {"Solutions", 255932, 8302},
 {"SolutionsCP3", 256023, 8305},
 {"SolutionsCE1", 256119, 8308},
 {"Exercises", 256212, 8311}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[579, 22, 106, 1, 96, "BookChapterTitle",
 CounterAssignments->{{"ChapterNumber", 1}}],
Cell[CellGroupData[{
Cell[710, 27, 31, 0, 55, "Section"],
Cell[744, 29, 574, 12, 65, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1355, 46, 65, 1, 55, "Section",
 CellTags->"Section1"],
Cell[1423, 49, 560, 14, 50, "Text"],
Cell[1986, 65, 635, 20, 52, "Text"],
Cell[CellGroupData[{
Cell[2646, 89, 31, 0, 29, "Input"],
Cell[2680, 91, 32, 0, 29, "Output"]
}, Open  ]],
Cell[2727, 94, 185, 5, 20, "Text"],
Cell[CellGroupData[{
Cell[2937, 103, 56, 1, 29, "Input"],
Cell[2996, 106, 31, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[3064, 111, 31, 0, 29, "Input"],
Cell[3098, 113, 31, 0, 28, "Output"]
}, Open  ]],
Cell[3144, 116, 639, 19, 37, "Text"],
Cell[3786, 137, 62, 1, 29, "Input"],
Cell[3851, 140, 49, 1, 29, "Input"],
Cell[CellGroupData[{
Cell[3925, 145, 65, 1, 32, "Subsection",
 CellTags->"Section1a"],
Cell[3993, 148, 1152, 37, 52, "Text"],
Cell[5148, 187, 651, 20, 50, "Text"],
Cell[CellGroupData[{
Cell[5824, 211, 85, 2, 28, "Input"],
Cell[5912, 215, 31, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[5980, 220, 149, 4, 29, "Input"],
Cell[6132, 226, 32, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[6201, 231, 85, 2, 20, "Input"],
Cell[6289, 235, 32, 0, 20, "Output"]
}, Open  ]],
Cell[6336, 238, 713, 27, 46, "Text"],
Cell[CellGroupData[{
Cell[7074, 269, 47, 1, 20, "Input"],
Cell[7124, 272, 32, 0, 20, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[7193, 277, 108, 3, 21, "Input"],
Cell[7304, 282, 31, 0, 20, "Output"]
}, Open  ]],
Cell[7350, 285, 519, 12, 60, "Text"],
Cell[CellGroupData[{
Cell[7894, 301, 71, 1, 20, "Input"],
Cell[7968, 304, 32, 0, 20, "Output"]
}, Open  ]],
Cell[8015, 307, 261, 6, 32, "Text"],
Cell[CellGroupData[{
Cell[8301, 317, 76, 1, 20, "Input"],
Cell[8380, 320, 31, 0, 20, "Output"]
}, Open  ]],
Cell[8426, 323, 407, 9, 60, "Text"],
Cell[8836, 334, 1422, 42, 134, "Text"],
Cell[10261, 378, 562, 14, 74, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[10860, 397, 68, 1, 26, "Subsection",
 CellTags->"Section1b"],
Cell[10931, 400, 619, 16, 51, "Text"],
Cell[11553, 418, 512, 8, 65, "Text"],
Cell[12068, 428, 248, 7, 29, "Input",
 InitializationCell->True],
Cell[12319, 437, 429, 9, 35, "Text"],
Cell[CellGroupData[{
Cell[12773, 450, 62, 1, 29, "Input"],
Cell[12838, 453, 28, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[12903, 458, 62, 1, 29, "Input"],
Cell[12968, 461, 28, 0, 28, "Output"]
}, Open  ]],
Cell[13011, 464, 655, 14, 66, "Text"],
Cell[13669, 480, 517, 13, 51, "Text"],
Cell[14189, 495, 1051, 34, 52, "Text"],
Cell[15243, 531, 1292, 30, 98, "Text"],
Cell[16538, 563, 668, 16, 52, "Text"],
Cell[17209, 581, 102, 3, 28, "Input"],
Cell[17314, 586, 1139, 33, 85, "Text"],
Cell[CellGroupData[{
Cell[18478, 623, 113, 3, 28, "Input"],
Cell[18594, 628, 114, 3, 28, "Output"]
}, Open  ]],
Cell[18723, 634, 186, 5, 32, "Text"],
Cell[CellGroupData[{
Cell[18934, 643, 123, 3, 20, "Input"],
Cell[19060, 648, 28, 0, 20, "Output"]
}, Open  ]],
Cell[19103, 651, 496, 9, 60, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[19636, 665, 68, 1, 26, "Subsection",
 CellTags->"Section1c"],
Cell[19707, 668, 357, 8, 35, "Text"],
Cell[20067, 678, 109, 3, 29, "Input",
 InitializationCell->True],
Cell[20179, 683, 523, 15, 53, "Text"],
Cell[CellGroupData[{
Cell[20727, 702, 63, 1, 29, "Input"],
Cell[20793, 705, 32, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[20862, 710, 31, 0, 29, "Input"],
Cell[20896, 712, 31, 0, 28, "Output"]
}, Open  ]],
Cell[20942, 715, 228, 5, 36, "Text"],
Cell[21173, 722, 45, 1, 29, "Input"],
Cell[21221, 725, 611, 15, 52, "Text"],
Cell[CellGroupData[{
Cell[21857, 744, 86, 2, 29, "Input"],
Cell[21946, 748, 32, 0, 28, "Output"]
}, Open  ]],
Cell[21993, 751, 776, 22, 68, "Text"],
Cell[22772, 775, 477, 17, 35, "Text"],
Cell[CellGroupData[{
Cell[23274, 796, 234, 8, 29, "Input"],
Cell[23511, 806, 31, 0, 28, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[23591, 812, 68, 1, 26, "Subsection",
 CellTags->"Section1d"],
Cell[23662, 815, 534, 11, 51, "Text"],
Cell[24199, 828, 501, 11, 51, "Text"],
Cell[24703, 841, 1315, 39, 81, "Text"],
Cell[26021, 882, 1623, 45, 99, "Text"],
Cell[27647, 929, 431, 13, 52, "Text"],
Cell[CellGroupData[{
Cell[28103, 946, 284, 9, 29, "Input"],
Cell[CellGroupData[{
Cell[28412, 959, 184, 5, 21, "Print"],
Cell[28599, 966, 184, 5, 21, "Print"]
}, Open  ]]
}, Open  ]],
Cell[28810, 975, 1534, 43, 115, "Text"],
Cell[CellGroupData[{
Cell[30369, 1022, 503, 15, 29, "Input"],
Cell[CellGroupData[{
Cell[30897, 1041, 260, 6, 21, "Print"],
Cell[31160, 1049, 262, 6, 21, "Print"],
Cell[31425, 1057, 262, 6, 21, "Print"],
Cell[31690, 1065, 264, 6, 21, "Print"]
}, Open  ]]
}, Open  ]],
Cell[31981, 1075, 363, 12, 35, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[32381, 1092, 70, 1, 26, "Subsection",
 CellTags->"Section1e"],
Cell[32454, 1095, 713, 17, 65, "Text"],
Cell[CellGroupData[{
Cell[33192, 1116, 47, 0, 18, "Subsubsection"],
Cell[33242, 1118, 927, 30, 52, "Text"],
Cell[CellGroupData[{
Cell[34194, 1152, 82, 2, 28, "Input"],
Cell[34279, 1156, 28, 0, 28, "Output"]
}, Open  ]],
Cell[34322, 1159, 361, 12, 21, "Text"],
Cell[CellGroupData[{
Cell[34708, 1175, 101, 2, 28, "Input"],
Cell[34812, 1179, 28, 0, 28, "Output"]
}, Open  ]],
Cell[34855, 1182, 368, 8, 36, "Text"],
Cell[CellGroupData[{
Cell[35248, 1194, 163, 5, 28, "Input"],
Cell[35414, 1201, 155, 5, 28, "Output"]
}, Open  ]],
Cell[35584, 1209, 463, 11, 50, "Text"],
Cell[CellGroupData[{
Cell[36072, 1224, 243, 7, 28, "Input"],
Cell[36318, 1233, 103, 2, 28, "Output"]
}, Open  ]],
Cell[36436, 1238, 268, 9, 21, "Text"],
Cell[CellGroupData[{
Cell[36729, 1251, 82, 2, 28, "Input"],
Cell[36814, 1255, 29, 0, 28, "Output"]
}, Open  ]],
Cell[36858, 1258, 481, 13, 51, "Text"],
Cell[CellGroupData[{
Cell[37364, 1275, 90, 2, 29, "Input"],
Cell[37457, 1279, 103, 2, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[37597, 1286, 89, 2, 29, "Input"],
Cell[37689, 1290, 83, 2, 28, "Output"]
}, Open  ]],
Cell[37787, 1295, 277, 7, 36, "Text"],
Cell[CellGroupData[{
Cell[38089, 1306, 242, 7, 28, "Input"],
Cell[38334, 1315, 103, 2, 20, "Output"]
}, Open  ]],
Cell[38452, 1320, 406, 12, 32, "Text"],
Cell[CellGroupData[{
Cell[38883, 1336, 168, 5, 21, "Input"],
Cell[39054, 1343, 29, 0, 20, "Output"]
}, Open  ]],
Cell[39098, 1346, 370, 8, 46, "Text"],
Cell[CellGroupData[{
Cell[39493, 1358, 59, 1, 20, "Input"],
Cell[39555, 1361, 45, 1, 20, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[39637, 1367, 59, 1, 20, "Input"],
Cell[39699, 1370, 45, 1, 20, "Output"]
}, Open  ]],
Cell[39759, 1374, 996, 28, 74, "Text"],
Cell[CellGroupData[{
Cell[40780, 1406, 116, 3, 20, "Input"],
Cell[40899, 1411, 28, 0, 20, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[40976, 1417, 51, 0, 17, "Subsubsection"],
Cell[41030, 1419, 447, 12, 46, "Text"],
Cell[41480, 1433, 454, 13, 46, "Text"],
Cell[41937, 1448, 652, 19, 46, "Text"],
Cell[42592, 1469, 567, 14, 27, "myCode"],
Cell[43162, 1485, 578, 13, 60, "Text"],
Cell[43743, 1500, 570, 14, 46, "Text"],
Cell[44316, 1516, 488, 15, 36, "Input"],
Cell[44807, 1533, 164, 5, 18, "Text"],
Cell[CellGroupData[{
Cell[44996, 1542, 79, 2, 20, "Input"],
Cell[45078, 1546, 28, 0, 20, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[45143, 1551, 79, 2, 20, "Input"],
Cell[45225, 1555, 28, 0, 20, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[45290, 1560, 80, 2, 20, "Input"],
Cell[45373, 1564, 491, 12, 81, "Output"]
}, Open  ]],
Cell[45879, 1579, 939, 24, 88, "Text"],
Cell[46821, 1605, 168, 5, 21, "Input",
 InitializationCell->True],
Cell[46992, 1612, 602, 18, 36, "Input",
 InitializationCell->True],
Cell[47597, 1632, 71, 0, 18, "Text"],
Cell[CellGroupData[{
Cell[47693, 1636, 80, 2, 20, "Input"],
Cell[47776, 1640, 179, 6, 22, "Message"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[48004, 1652, 47, 0, 17, "Subsubsection"],
Cell[48054, 1654, 333, 9, 32, "Text"],
Cell[48390, 1665, 513, 17, 32, "Text"],
Cell[48906, 1684, 475, 17, 34, "Text"],
Cell[49384, 1703, 139, 4, 21, "Input",
 InitializationCell->True],
Cell[49526, 1709, 242, 7, 32, "Text"],
Cell[CellGroupData[{
Cell[49793, 1720, 171, 5, 21, "Input"],
Cell[49967, 1727, 119, 3, 20, "Output"]
}, Open  ]],
Cell[50101, 1733, 252, 7, 32, "Text"],
Cell[CellGroupData[{
Cell[50378, 1744, 141, 4, 20, "Input"],
Cell[50522, 1750, 119, 3, 20, "Output"]
}, Open  ]],
Cell[50656, 1756, 1352, 44, 76, "Text"],
Cell[52011, 1802, 202, 7, 21, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[52238, 1813, 260, 8, 21, "Input"],
Cell[52501, 1823, 233, 8, 25, "Output"]
}, Open  ]],
Cell[52749, 1834, 273, 10, 18, "Text"],
Cell[CellGroupData[{
Cell[53047, 1848, 305, 9, 21, "Input"],
Cell[53355, 1859, 103, 2, 20, "Output"]
}, Open  ]],
Cell[53473, 1864, 1014, 25, 88, "Text"],
Cell[54490, 1891, 125, 3, 20, "Input",
 InitializationCell->True],
Cell[54618, 1896, 221, 8, 18, "Text"],
Cell[CellGroupData[{
Cell[54864, 1908, 240, 7, 20, "Input"],
Cell[55107, 1917, 103, 2, 20, "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[55283, 1927, 81, 1, 35, "Section",
 CellTags->"Section2"],
Cell[55367, 1930, 329, 7, 35, "Text"],
Cell[CellGroupData[{
Cell[55721, 1941, 67, 1, 32, "Subsection",
 CellTags->"Section2a"],
Cell[55791, 1944, 303, 5, 50, "Text"],
Cell[56097, 1951, 969, 28, 80, "Text"],
Cell[57069, 1981, 530, 17, 53, "Text"],
Cell[CellGroupData[{
Cell[57624, 2002, 458, 15, 29, "Input"],
Cell[58085, 2019, 92, 2, 28, "Output"]
}, Open  ]],
Cell[58192, 2024, 665, 19, 52, "Text"],
Cell[58860, 2045, 400, 9, 36, "Text"],
Cell[CellGroupData[{
Cell[59285, 2058, 208, 7, 29, "Input"],
Cell[59496, 2067, 108, 2, 28, "Output"]
}, Open  ]],
Cell[59619, 2072, 382, 8, 36, "Text"],
Cell[CellGroupData[{
Cell[60026, 2084, 273, 9, 29, "Input"],
Cell[60302, 2095, 372, 10, 28, "Output"]
}, Open  ]],
Cell[60689, 2108, 637, 15, 51, "Text"],
Cell[CellGroupData[{
Cell[61351, 2127, 313, 10, 29, "Input"],
Cell[61667, 2139, 629, 18, 88, "Output"]
}, Open  ]],
Cell[62311, 2160, 195, 4, 35, "Text"],
Cell[CellGroupData[{
Cell[62531, 2168, 247, 8, 21, "Input",
 InitializationCell->True],
Cell[62781, 2178, 160, 5, 21, "Output"]
}, Open  ]],
Cell[62956, 2186, 197, 6, 18, "Text"],
Cell[CellGroupData[{
Cell[63178, 2196, 239, 7, 21, "Input"],
Cell[63420, 2205, 2671, 75, 170, "Output"]
}, Open  ]],
Cell[66106, 2283, 696, 23, 60, "Text"],
Cell[66805, 2308, 510, 15, 46, "Text"],
Cell[CellGroupData[{
Cell[67340, 2327, 281, 9, 21, "Input",
 InitializationCell->True],
Cell[67624, 2338, 184, 6, 21, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[67845, 2349, 240, 7, 21, "Input"],
Cell[68088, 2358, 629, 18, 78, "Output"]
}, Open  ]],
Cell[68732, 2379, 211, 4, 32, "Text"],
Cell[68946, 2385, 705, 19, 60, "Text"],
Cell[CellGroupData[{
Cell[69676, 2408, 144, 4, 21, "Input"],
Cell[69823, 2414, 31, 0, 20, "Output"]
}, Open  ]],
Cell[69869, 2417, 318, 7, 32, "Text"],
Cell[CellGroupData[{
Cell[70212, 2428, 155, 4, 21, "Input"],
Cell[70370, 2434, 104, 3, 20, "Output"]
}, Open  ]],
Cell[70489, 2440, 301, 9, 32, "Text"],
Cell[CellGroupData[{
Cell[70815, 2453, 218, 6, 21, "Input"],
Cell[71036, 2461, 251, 8, 20, "Output"]
}, Open  ]],
Cell[71302, 2472, 250, 7, 32, "Text"],
Cell[CellGroupData[{
Cell[71577, 2483, 142, 4, 21, "Input",
 InitializationCell->True],
Cell[71722, 2489, 213, 7, 21, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[71972, 2501, 239, 7, 21, "Input"],
Cell[72214, 2510, 630, 18, 78, "Output"]
}, Open  ]],
Cell[72859, 2531, 353, 10, 32, "Text"],
Cell[CellGroupData[{
Cell[73237, 2545, 143, 4, 21, "Input"],
Cell[73383, 2551, 32, 0, 20, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[73452, 2556, 154, 4, 21, "Input"],
Cell[73609, 2562, 45, 1, 20, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[73703, 2569, 59, 1, 26, "Subsection",
 CellTags->"Section2b"],
Cell[73765, 2572, 553, 9, 65, "Text"],
Cell[74321, 2583, 919, 27, 50, "Text"],
Cell[75243, 2612, 1286, 29, 96, "Text"],
Cell[CellGroupData[{
Cell[76554, 2645, 140, 4, 29, "Input",
 InitializationCell->True],
Cell[76697, 2651, 62, 1, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[76796, 2657, 320, 11, 29, "Input",
 InitializationCell->True],
Cell[77119, 2670, 228, 8, 29, "Output"]
}, Open  ]],
Cell[77362, 2681, 336, 9, 35, "Text"],
Cell[CellGroupData[{
Cell[77723, 2694, 189, 5, 29, "Input"],
Cell[77915, 2701, 105, 3, 28, "Output"]
}, Open  ]],
Cell[78035, 2707, 437, 10, 50, "Text"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[78521, 2723, 71, 1, 35, "Section",
 CellTags->"Section3"],
Cell[78595, 2726, 324, 8, 35, "Text"],
Cell[CellGroupData[{
Cell[78944, 2738, 64, 1, 32, "Subsection",
 CellTags->"Section3a"],
Cell[79011, 2741, 1165, 34, 67, "Text"],
Cell[80179, 2777, 404, 12, 20, "Text"],
Cell[CellGroupData[{
Cell[80608, 2793, 346, 12, 29, "Input"],
Cell[80957, 2807, 31, 0, 28, "Output"]
}, Open  ]],
Cell[81003, 2810, 425, 12, 36, "Text"],
Cell[81431, 2824, 698, 16, 51, "Text"],
Cell[CellGroupData[{
Cell[82154, 2844, 266, 9, 29, "Input"],
Cell[82423, 2855, 31, 0, 28, "Output"]
}, Open  ]],
Cell[82469, 2858, 621, 15, 51, "Text"],
Cell[83093, 2875, 241, 7, 29, "Input",
 InitializationCell->True],
Cell[83337, 2884, 129, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[83491, 2891, 265, 9, 29, "Input"],
Cell[83759, 2902, 31, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[83827, 2907, 265, 9, 29, "Input"],
Cell[84095, 2918, 31, 0, 28, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[84175, 2924, 73, 1, 26, "Subsection",
 CellTags->"Section3b"],
Cell[84251, 2927, 717, 15, 80, "Text"],
Cell[84971, 2944, 317, 9, 35, "Text"],
Cell[85291, 2955, 1185, 29, 97, "Text"],
Cell[86479, 2986, 239, 4, 35, "Text"],
Cell[CellGroupData[{
Cell[86743, 2994, 45, 0, 18, "Subsubsection"],
Cell[86791, 2996, 203, 6, 20, "Text"],
Cell[CellGroupData[{
Cell[87019, 3006, 350, 12, 29, "Input",
 InitializationCell->True],
Cell[87372, 3020, 281, 10, 29, "Output"]
}, Open  ]],
Cell[87668, 3033, 150, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[87843, 3040, 125, 3, 18, "Subsubsubsection",
 PageBreakWithin->False,
 PageBreakBelow->False],
Cell[87971, 3045, 993, 26, 73, "Text",
 PageBreakAbove->False,
 PageBreakWithin->False],
Cell[CellGroupData[{
Cell[88989, 3075, 80, 2, 29, "Input"],
Cell[89072, 3079, 206, 8, 40, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[89315, 3092, 66, 1, 28, "Input"],
Cell[89384, 3095, 493, 17, 40, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[89914, 3117, 127, 4, 29, "Input"],
Cell[90044, 3123, 226, 8, 40, "Output"]
}, Open  ]],
Cell[90285, 3134, 290, 10, 21, "Text"],
Cell[CellGroupData[{
Cell[90600, 3148, 80, 2, 29, "Input"],
Cell[90683, 3152, 31, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[90751, 3157, 66, 1, 28, "Input"],
Cell[90820, 3160, 30, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[90887, 3165, 124, 3, 29, "Input"],
Cell[91014, 3170, 31, 0, 28, "Output"]
}, Open  ]],
Cell[91060, 3173, 262, 8, 21, "Text"],
Cell[CellGroupData[{
Cell[91347, 3185, 124, 4, 29, "Input"],
Cell[91474, 3191, 31, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[91542, 3196, 86, 2, 28, "Input"],
Cell[91631, 3200, 30, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[91698, 3205, 144, 4, 29, "Input"],
Cell[91845, 3211, 31, 0, 28, "Output"]
}, Open  ]],
Cell[91891, 3214, 1060, 32, 66, "Text"],
Cell[CellGroupData[{
Cell[92976, 3250, 350, 11, 29, "Input"],
Cell[93329, 3263, 257, 9, 29, "Output"]
}, Open  ]],
Cell[93601, 3275, 535, 14, 53, "Text"],
Cell[CellGroupData[{
Cell[94161, 3293, 81, 2, 29, "Input"],
Cell[94245, 3297, 50, 1, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[94332, 3303, 113, 3, 29, "Input"],
Cell[94448, 3308, 31, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[94516, 3313, 36, 0, 29, "Input"],
Cell[94555, 3315, 73, 2, 29, "Output"]
}, Open  ]],
Cell[94643, 3320, 647, 15, 67, "Text"],
Cell[CellGroupData[{
Cell[95315, 3339, 95, 2, 29, "Input"],
Cell[95413, 3343, 50, 1, 29, "Output"]
}, Open  ]],
Cell[95478, 3347, 771, 21, 53, "Text"],
Cell[CellGroupData[{
Cell[96274, 3372, 381, 10, 61, "Input"],
Cell[96658, 3384, 44, 0, 22, "Print"]
}, Open  ]],
Cell[96717, 3387, 587, 19, 37, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[97341, 3411, 56, 0, 18, "Subsubsubsection"],
Cell[97400, 3413, 502, 8, 66, "Text"],
Cell[97905, 3423, 849, 26, 53, "Text"],
Cell[CellGroupData[{
Cell[98779, 3453, 98, 2, 28, "Input"],
Cell[98880, 3457, 257, 9, 29, "Output"]
}, Open  ]],
Cell[99152, 3469, 130, 3, 20, "Text"],
Cell[99285, 3474, 388, 10, 36, "Text"],
Cell[CellGroupData[{
Cell[99698, 3488, 90, 2, 28, "Input"],
Cell[99791, 3492, 167, 6, 29, "Output"]
}, Open  ]],
Cell[99973, 3501, 256, 8, 21, "Text"],
Cell[CellGroupData[{
Cell[100254, 3513, 171, 5, 28, "Input"],
Cell[100428, 3520, 31, 0, 28, "Output"]
}, Open  ]],
Cell[100474, 3523, 166, 5, 21, "Text"],
Cell[CellGroupData[{
Cell[100665, 3532, 61, 1, 28, "Input"],
Cell[100729, 3535, 231, 8, 29, "Output"]
}, Open  ]],
Cell[100975, 3546, 481, 12, 51, "Text"],
Cell[CellGroupData[{
Cell[101481, 3562, 109, 2, 28, "Input"],
Cell[101593, 3566, 186, 6, 29, "Output"]
}, Open  ]],
Cell[101794, 3575, 397, 11, 37, "Text"],
Cell[CellGroupData[{
Cell[102216, 3590, 209, 6, 44, "Input"],
Cell[102428, 3598, 210, 7, 29, "Output"]
}, Open  ]],
Cell[102653, 3608, 203, 6, 21, "Text"],
Cell[CellGroupData[{
Cell[102881, 3618, 109, 2, 28, "Input"],
Cell[102993, 3622, 171, 5, 29, "Output"]
}, Open  ]],
Cell[103179, 3630, 246, 7, 37, "Text"],
Cell[CellGroupData[{
Cell[103450, 3641, 209, 6, 44, "Input"],
Cell[103662, 3649, 196, 6, 29, "Output"]
}, Open  ]],
Cell[103873, 3658, 53, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[103951, 3662, 119, 2, 28, "Input"],
Cell[104073, 3666, 155, 4, 29, "Output"]
}, Open  ]],
Cell[104243, 3673, 322, 9, 37, "Text"],
Cell[CellGroupData[{
Cell[104590, 3686, 278, 8, 44, "Input"],
Cell[104871, 3696, 136, 3, 29, "Output"]
}, Open  ]],
Cell[105022, 3702, 30, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[105077, 3706, 278, 8, 44, "Input"],
Cell[105358, 3716, 116, 3, 29, "Output"]
}, Open  ]],
Cell[105489, 3722, 251, 7, 21, "Text"],
Cell[CellGroupData[{
Cell[105765, 3733, 118, 2, 29, "Input"],
Cell[105886, 3737, 93, 2, 29, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[106028, 3745, 40, 0, 18, "Subsubsubsection"],
Cell[106071, 3747, 89, 2, 20, "Text"],
Cell[CellGroupData[{
Cell[106185, 3753, 307, 6, 30, "Item1Numbered"],
Cell[106495, 3761, 255, 5, 30, "Item1Numbered"],
Cell[106753, 3768, 275, 10, 17, "Item1Numbered"],
Cell[CellGroupData[{
Cell[107053, 3782, 183, 6, 16, "Item2"],
Cell[107239, 3790, 267, 9, 30, "Item2"]
}, Open  ]],
Cell[107521, 3802, 522, 13, 31, "Item1Numbered"]
}, Open  ]],
Cell[108058, 3818, 43, 0, 20, "Text"],
Cell[108104, 3820, 1231, 34, 169, "Input",
 InitializationCell->True],
Cell[109338, 3856, 1075, 26, 84, "Text"],
Cell[110416, 3884, 933, 28, 71, "Text"],
Cell[111352, 3914, 621, 17, 53, "Text"],
Cell[111976, 3933, 121, 4, 21, "Text"],
Cell[CellGroupData[{
Cell[112122, 3941, 69, 1, 29, "Input"],
Cell[112194, 3944, 93, 2, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[112324, 3951, 259, 8, 29, "Input"],
Cell[112586, 3961, 103, 2, 29, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[112750, 3970, 48, 0, 18, "Subsubsection"],
Cell[112801, 3972, 355, 6, 50, "Text"],
Cell[113159, 3980, 226, 6, 36, "Text"],
Cell[CellGroupData[{
Cell[113410, 3990, 132, 3, 29, "Input",
 InitializationCell->True],
Cell[113545, 3995, 93, 2, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[113675, 4002, 167, 5, 28, "Input",
 InitializationCell->True],
Cell[113845, 4009, 106, 2, 28, "Output"]
}, Open  ]],
Cell[113966, 4014, 302, 8, 36, "Text"],
Cell[CellGroupData[{
Cell[114293, 4026, 53, 0, 18, "Subsubsubsection"],
Cell[114349, 4028, 996, 37, 52, "Text"],
Cell[CellGroupData[{
Cell[115370, 4069, 263, 7, 29, "Input"],
Cell[115636, 4078, 32, 0, 28, "Output"]
}, Open  ]],
Cell[115683, 4081, 411, 11, 53, "Text"],
Cell[116097, 4094, 1127, 30, 83, "Text"],
Cell[CellGroupData[{
Cell[117249, 4128, 263, 8, 29, "Input"],
Cell[117515, 4138, 155, 5, 28, "Output"]
}, Open  ]],
Cell[117685, 4146, 352, 11, 37, "Text"],
Cell[CellGroupData[{
Cell[118062, 4161, 159, 4, 29, "Input"],
Cell[118224, 4167, 226, 6, 29, "Output"]
}, Open  ]],
Cell[118465, 4176, 69, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[118559, 4180, 195, 5, 29, "Input"],
Cell[118757, 4187, 32, 0, 28, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[118838, 4193, 66, 0, 18, "Subsubsubsection"],
Cell[118907, 4195, 853, 12, 95, "Text"],
Cell[119763, 4209, 1192, 25, 99, "Text"],
Cell[120958, 4236, 1253, 32, 182, "Input",
 InitializationCell->True],
Cell[122214, 4270, 631, 21, 52, "Text"],
Cell[122848, 4293, 1448, 41, 101, "Text"],
Cell[124299, 4336, 965, 21, 82, "Text"],
Cell[125267, 4359, 2697, 75, 150, "Text"],
Cell[127967, 4436, 1323, 36, 84, "Text"],
Cell[129293, 4474, 948, 28, 53, "Text"],
Cell[CellGroupData[{
Cell[130266, 4506, 490, 13, 89, "Input"],
Cell[CellGroupData[{
Cell[130781, 4523, 94, 2, 21, "Print"],
Cell[130878, 4527, 93, 2, 21, "Print"],
Cell[130974, 4531, 93, 2, 21, "Print"],
Cell[131070, 4535, 92, 2, 21, "Print"],
Cell[131165, 4539, 93, 2, 21, "Print"],
Cell[131261, 4543, 92, 2, 21, "Print"],
Cell[131356, 4547, 92, 2, 21, "Print"],
Cell[131451, 4551, 91, 2, 21, "Print"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[131615, 4561, 59, 0, 18, "Subsubsection"],
Cell[131677, 4563, 272, 7, 36, "Text"],
Cell[131952, 4572, 49, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[132026, 4576, 739, 17, 59, "Item1Numbered"],
Cell[132768, 4595, 704, 20, 44, "Item1Numbered"],
Cell[133475, 4617, 427, 9, 43, "Item1Numbered"]
}, Open  ]],
Cell[133917, 4629, 1758, 44, 231, "Input",
 InitializationCell->True],
Cell[135678, 4675, 524, 17, 35, "Text"],
Cell[CellGroupData[{
Cell[136227, 4696, 305, 11, 29, "Input"],
Cell[136535, 4709, 31, 0, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[136639, 4717, 72, 1, 35, "Section",
 CellTags->"Section4"],
Cell[136714, 4720, 391, 11, 35, "Text"],
Cell[137108, 4733, 621, 20, 36, "Text"],
Cell[137732, 4755, 131, 4, 29, "Input",
 InitializationCell->True],
Cell[137866, 4761, 130, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[138021, 4768, 56, 1, 29, "Input"],
Cell[138080, 4771, 31, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[138148, 4776, 74, 2, 29, "Input"],
Cell[138225, 4780, 32, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[138294, 4785, 100, 2, 32, "Subsection",
 PageBreakWithin->False,
 CellTags->"Section4a"],
Cell[138397, 4789, 829, 24, 51, "Text",
 PageBreakAbove->False,
 PageBreakWithin->False],
Cell[CellGroupData[{
Cell[139251, 4817, 111, 3, 28, "Input"],
Cell[139365, 4822, 103, 3, 29, "Output"]
}, Open  ]],
Cell[139483, 4828, 195, 5, 20, "Text"],
Cell[CellGroupData[{
Cell[139703, 4837, 126, 4, 28, "Input"],
Cell[139832, 4843, 117, 4, 29, "Output"]
}, Open  ]],
Cell[139964, 4850, 937, 32, 51, "Text"],
Cell[CellGroupData[{
Cell[140926, 4886, 160, 5, 28, "Input"],
Cell[141089, 4893, 166, 6, 31, "Output"]
}, Open  ]],
Cell[141270, 4902, 922, 26, 52, "Text"],
Cell[CellGroupData[{
Cell[142217, 4932, 205, 6, 28, "Input"],
Cell[142425, 4940, 185, 6, 33, "Output"]
}, Open  ]],
Cell[142625, 4949, 927, 32, 37, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[143589, 4986, 82, 1, 26, "Subsection",
 CellTags->"Section4b"],
Cell[143674, 4989, 512, 13, 51, "Text"],
Cell[CellGroupData[{
Cell[144211, 5006, 163, 5, 28, "Input"],
Cell[144377, 5013, 31, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[144445, 5018, 198, 6, 28, "Input"],
Cell[144646, 5026, 31, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[144714, 5031, 244, 7, 28, "Input"],
Cell[144961, 5040, 32, 0, 28, "Output"]
}, Open  ]],
Cell[145008, 5043, 705, 21, 37, "Text"],
Cell[CellGroupData[{
Cell[145738, 5068, 202, 7, 28, "Input"],
Cell[145943, 5077, 32, 0, 28, "Output"]
}, Open  ]],
Cell[145990, 5080, 86, 2, 20, "Text"],
Cell[CellGroupData[{
Cell[146101, 5086, 206, 7, 29, "Input"],
Cell[146310, 5095, 31, 0, 28, "Output"]
}, Open  ]],
Cell[146356, 5098, 846, 24, 54, "Text"],
Cell[CellGroupData[{
Cell[147227, 5126, 175, 5, 28, "Input"],
Cell[147405, 5133, 31, 0, 28, "Output"]
}, Open  ]],
Cell[147451, 5136, 390, 12, 38, "Text"],
Cell[CellGroupData[{
Cell[147866, 5152, 144, 4, 28, "Input"],
Cell[148013, 5158, 103, 3, 28, "Output"]
}, Open  ]],
Cell[148131, 5164, 307, 8, 36, "Text"],
Cell[CellGroupData[{
Cell[148463, 5176, 157, 5, 28, "Input"],
Cell[148623, 5183, 456, 16, 35, "Output"]
}, Open  ]],
Cell[149094, 5202, 417, 10, 50, "Text"],
Cell[CellGroupData[{
Cell[149536, 5216, 175, 5, 29, "Input"],
Cell[149714, 5223, 103, 3, 20, "Output"]
}, Open  ]],
Cell[149832, 5229, 857, 25, 74, "Text"],
Cell[CellGroupData[{
Cell[150714, 5258, 204, 7, 21, "Input"],
Cell[150921, 5267, 57, 1, 21, "Output"]
}, Open  ]],
Cell[150993, 5271, 272, 9, 32, "Text"],
Cell[151268, 5282, 183, 5, 18, "Text"],
Cell[CellGroupData[{
Cell[151476, 5291, 219, 7, 21, "Input"],
Cell[151698, 5300, 64, 1, 21, "Output"]
}, Open  ]],
Cell[151777, 5304, 645, 21, 48, "Text"],
Cell[CellGroupData[{
Cell[152447, 5329, 204, 7, 21, "Input"],
Cell[152654, 5338, 31, 0, 20, "Output"]
}, Open  ]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[152746, 5345, 64, 1, 35, "Section",
 CellTags->"Section5"],
Cell[152813, 5348, 178, 5, 20, "Text"],
Cell[152994, 5355, 1080, 36, 54, "Text"],
Cell[CellGroupData[{
Cell[154099, 5395, 346, 11, 29, "Input"],
Cell[154448, 5408, 330, 11, 31, "Output"]
}, Open  ]],
Cell[154793, 5422, 253, 9, 21, "Text"],
Cell[CellGroupData[{
Cell[155071, 5435, 390, 12, 29, "Input"],
Cell[155464, 5449, 31, 0, 28, "Output"]
}, Open  ]],
Cell[155510, 5452, 390, 10, 50, "Text"],
Cell[155903, 5464, 670, 23, 38, "Text"],
Cell[CellGroupData[{
Cell[156598, 5491, 245, 7, 29, "Input"],
Cell[156846, 5500, 239, 8, 31, "Output"]
}, Open  ]],
Cell[157100, 5511, 189, 6, 21, "Text"],
Cell[CellGroupData[{
Cell[157314, 5521, 288, 9, 29, "Input"],
Cell[157605, 5532, 31, 0, 28, "Output"]
}, Open  ]],
Cell[157651, 5535, 90, 2, 20, "Text"],
Cell[CellGroupData[{
Cell[157766, 5541, 326, 10, 29, "Input"],
Cell[158095, 5553, 32, 0, 28, "Output"]
}, Open  ]],
Cell[158142, 5556, 120, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[158287, 5563, 247, 7, 29, "Input"],
Cell[158537, 5572, 241, 8, 31, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[158815, 5585, 290, 9, 29, "Input"],
Cell[159108, 5596, 32, 0, 28, "Output"]
}, Open  ]],
Cell[159155, 5599, 201, 6, 20, "Text"],
Cell[CellGroupData[{
Cell[159381, 5609, 305, 10, 29, "Input"],
Cell[159689, 5621, 263, 9, 31, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[160001, 5636, 64, 1, 35, "Section",
 CellTags->"Section6"],
Cell[160068, 5639, 1113, 27, 80, "Text"],
Cell[161184, 5668, 452, 12, 36, "Text"],
Cell[CellGroupData[{
Cell[161661, 5684, 281, 9, 29, "Input"],
Cell[161945, 5695, 31, 0, 28, "Output"]
}, Open  ]],
Cell[161991, 5698, 527, 11, 51, "Text"],
Cell[162521, 5711, 851, 25, 93, "Input",
 InitializationCell->True],
Cell[163375, 5738, 2151, 63, 120, "Text"],
Cell[165529, 5803, 148, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[165702, 5810, 488, 15, 29, "Input"],
Cell[166193, 5827, 31, 0, 28, "Output"]
}, Open  ]],
Cell[166239, 5830, 703, 12, 80, "Text"],
Cell[CellGroupData[{
Cell[166967, 5846, 76, 1, 32, "Subsection",
 CellTags->"Section6a"],
Cell[167046, 5849, 480, 10, 60, "Text"],
Cell[CellGroupData[{
Cell[167551, 5863, 53, 0, 17, "Subsubsection"],
Cell[167607, 5865, 521, 9, 74, "Text"],
Cell[168131, 5876, 853, 19, 74, "Text"],
Cell[168987, 5897, 221, 4, 32, "Text"],
Cell[169211, 5903, 2680, 76, 186, "Input",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[171928, 5984, 50, 0, 17, "Subsubsection"],
Cell[171981, 5986, 492, 9, 74, "Text"],
Cell[172476, 5997, 693, 15, 88, "Text"],
Cell[173172, 6014, 1411, 36, 186, "Input",
 InitializationCell->True],
Cell[174586, 6052, 329, 8, 46, "Text"],
Cell[CellGroupData[{
Cell[174940, 6064, 461, 14, 36, "Input"],
Cell[175404, 6080, 4035, 141, 141, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[179476, 6226, 59, 1, 21, "Input"],
Cell[179538, 6229, 29, 0, 20, "Output"]
}, Open  ]],
Cell[179582, 6232, 636, 9, 88, "Text"]
}, Open  ]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[180279, 6248, 68, 1, 35, "Section",
 CellTags->"Section7"],
Cell[180350, 6251, 300, 9, 35, "Text"],
Cell[180653, 6262, 485, 8, 50, "Text"],
Cell[CellGroupData[{
Cell[181163, 6274, 70, 1, 32, "Subsection",
 CellTags->"Section7a"],
Cell[181236, 6277, 398, 6, 50, "Text"],
Cell[181637, 6285, 1693, 42, 195, "Input",
 InitializationCell->True],
Cell[183333, 6329, 877, 24, 71, "Text"],
Cell[CellGroupData[{
Cell[184235, 6357, 65, 1, 29, "Input"],
Cell[184303, 6360, 83, 2, 28, "Output"]
}, Open  ]],
Cell[184401, 6365, 357, 7, 36, "Text"],
Cell[184761, 6374, 459, 13, 76, "Input",
 InitializationCell->True],
Cell[185223, 6389, 177, 3, 20, "Text"],
Cell[185403, 6394, 584, 16, 52, "Text"],
Cell[185990, 6412, 91, 2, 20, "Text"],
Cell[CellGroupData[{
Cell[186106, 6418, 38, 0, 28, "Input"],
Cell[186147, 6420, 28, 0, 28, "Output"]
}, Open  ]],
Cell[186190, 6423, 115, 3, 18, "Text"],
Cell[186308, 6428, 1634, 48, 116, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[187979, 6481, 51, 1, 26, "Subsection",
 CellTags->"Section7b"],
Cell[188033, 6484, 606, 9, 65, "Text"],
Cell[188642, 6495, 475, 11, 51, "Text"],
Cell[189120, 6508, 4207, 148, 191, "Text"],
Cell[193330, 6658, 642, 21, 54, "Text"],
Cell[193975, 6681, 322, 8, 36, "Text"],
Cell[194300, 6691, 366, 6, 50, "Text"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[194715, 6703, 72, 1, 35, "Section",
 CellTags->"Section8"],
Cell[194790, 6706, 213, 5, 35, "Text"],
Cell[CellGroupData[{
Cell[195028, 6715, 62, 1, 32, "Subsection",
 CellTags->"Section8a"],
Cell[195093, 6718, 526, 10, 65, "Text"],
Cell[195622, 6730, 186, 4, 35, "Text"],
Cell[195811, 6736, 749, 13, 83, "Text"],
Cell[196563, 6751, 1332, 38, 101, "Text"],
Cell[197898, 6791, 1554, 40, 201, "Input",
 InitializationCell->True],
Cell[199455, 6833, 382, 10, 36, "Text"],
Cell[CellGroupData[{
Cell[199862, 6847, 65, 1, 28, "Input"],
Cell[199930, 6850, 203, 5, 28, "Output"]
}, Open  ]],
Cell[200148, 6858, 1314, 37, 85, "Text"],
Cell[201465, 6897, 822, 23, 77, "Input",
 InitializationCell->True],
Cell[202290, 6922, 281, 7, 32, "Text"],
Cell[CellGroupData[{
Cell[202596, 6933, 82, 1, 20, "Input"],
Cell[202681, 6936, 167, 4, 20, "Print"]
}, Open  ]],
Cell[202863, 6943, 246, 4, 32, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[203146, 6952, 62, 1, 26, "Subsection",
 CellTags->"Section8b"],
Cell[203211, 6955, 539, 10, 50, "Text"],
Cell[203753, 6967, 837, 19, 68, "Text"],
Cell[204593, 6988, 2466, 68, 164, "Text"],
Cell[207062, 7058, 52, 0, 20, "Text"],
Cell[207117, 7060, 1279, 34, 151, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[208421, 7098, 34, 0, 28, "Input"],
Cell[208458, 7100, 31, 0, 28, "Output"]
}, Open  ]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[208550, 7107, 115, 3, 35, "Section",
 CellTags->"Solutions"],
Cell[CellGroupData[{
Cell[208690, 7114, 68, 1, 32, "Subsection",
 CellTags->"SolutionsCP3"],
Cell[208761, 7117, 201, 4, 55, "Problem"],
Cell[208965, 7123, 725, 19, 67, "Text"],
Cell[209693, 7144, 895, 23, 84, "Text"],
Cell[210591, 7169, 156, 4, 21, "Text"],
Cell[210750, 7175, 1491, 38, 185, "Input",
 InitializationCell->True],
Cell[212244, 7215, 109, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[212378, 7222, 315, 11, 29, "Input"],
Cell[CellGroupData[{
Cell[212718, 7237, 94, 2, 21, "Print"],
Cell[212815, 7241, 91, 2, 21, "Print"]
}, Open  ]],
Cell[212921, 7246, 31, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[212989, 7251, 276, 9, 29, "Input"],
Cell[CellGroupData[{
Cell[213290, 7264, 93, 2, 21, "Print"],
Cell[213386, 7268, 93, 2, 21, "Print"],
Cell[213482, 7272, 92, 2, 21, "Print"],
Cell[213577, 7276, 93, 2, 21, "Print"],
Cell[213673, 7280, 92, 2, 21, "Print"],
Cell[213768, 7284, 92, 2, 21, "Print"]
}, Open  ]],
Cell[213875, 7289, 31, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[213943, 7294, 522, 18, 29, "Input"],
Cell[214468, 7314, 32, 0, 28, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[214549, 7320, 80, 1, 26, "Subsection",
 CellTags->"SolutionsCE1"],
Cell[214632, 7323, 117, 3, 42, "Problem"],
Cell[214752, 7328, 762, 24, 49, "Text"],
Cell[215517, 7354, 723, 19, 63, "Text"],
Cell[216243, 7375, 195, 4, 32, "Text"],
Cell[216441, 7381, 280, 5, 46, "Text"],
Cell[216724, 7388, 734, 24, 27, "Equation"],
Cell[217461, 7414, 774, 16, 94, "Text"],
Cell[218238, 7432, 1011, 25, 115, "Text"],
Cell[219252, 7459, 458, 17, 36, "Text"],
Cell[219713, 7478, 428, 12, 46, "Text"],
Cell[220144, 7492, 1636, 43, 201, "Input",
 InitializationCell->True],
Cell[221783, 7537, 605, 15, 62, "Text"],
Cell[CellGroupData[{
Cell[222413, 7556, 199, 6, 20, "Input"],
Cell[222615, 7564, 148, 4, 20, "Output"]
}, Open  ]],
Cell[222778, 7571, 255, 9, 20, "Text"],
Cell[CellGroupData[{
Cell[223058, 7584, 128, 3, 20, "Input"],
Cell[223189, 7589, 389, 7, 36, "Output"]
}, Open  ]],
Cell[223593, 7599, 89, 2, 18, "Text"],
Cell[CellGroupData[{
Cell[223707, 7605, 128, 3, 28, "Input"],
Cell[223838, 7610, 1733, 25, 134, "Output"]
}, Open  ]],
Cell[225586, 7638, 122, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[225733, 7645, 130, 3, 29, "Input"],
Cell[225866, 7650, 4295, 59, 314, "Output",
 PageBreakWithin->True]
}, Open  ]],
Cell[230176, 7712, 94, 2, 20, "Text"],
Cell[CellGroupData[{
Cell[230295, 7718, 366, 11, 29, "Input"],
Cell[CellGroupData[{
Cell[230686, 7733, 28, 0, 21, "Print"],
Cell[230717, 7735, 29, 0, 21, "Print"]
}, Open  ]]
}, Open  ]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[230819, 7743, 52, 1, 35, "Section",
 CellTags->"Exercises"],
Cell[CellGroupData[{
Cell[230896, 7748, 200, 8, 21, "Item1Exercise"],
Cell[231099, 7758, 242, 7, 33, "Item1Exercise"],
Cell[231344, 7767, 495, 16, 34, "Item1Exercise"],
Cell[231842, 7785, 440, 14, 34, "Item1Exercise"],
Cell[CellGroupData[{
Cell[232307, 7803, 1167, 29, 85, "Item1Exercise",
 CellGroupingRules->{GroupTogetherGrouping, 17001.}],
Cell[233477, 7834, 432, 15, 29, "myCode",
 Evaluatable->False,
 CellGroupingRules->{GroupTogetherGrouping, 17001.},
 InitializationCell->False],
Cell[233912, 7851, 253, 6, 16, "ExerciseContinuation",
 CellGroupingRules->{GroupTogetherGrouping, 17001.}],
Cell[234168, 7859, 323, 10, 29, "Output",
 CellGroupingRules->{GroupTogetherGrouping, 17001.}]
}, Open  ]]
}, Open  ]]
}, Closed]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
